SystemOrganization addCategory: #'OmniBrowser-Tests'!
SystemOrganization addCategory: #'OmniBrowser-Fakes'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Morphs'!
SystemOrganization addCategory: #'OmniBrowser-Panels'!
SystemOrganization addCategory: #'OmniBrowser-Nodes'!
SystemOrganization addCategory: #'OmniBrowser-Actors'!
SystemOrganization addCategory: #'OmniBrowser-Notifications'!
SystemOrganization addCategory: #'OmniBrowser-Utilities'!

RectangleMorph subclass: #OBPane
	instanceVariableNames: 'model list button'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBPane commentStamp: 'cwp 3/5/2004 11:52' prior: 0!
An OBPane is the visual representation of a column in a browser. It contains a morph to display nodes (typically a PluggableListMorph) and (optionally) a morph for communicating with the column's filter. It's main responsibility is to layout it's submorphs as the filter controls are added and removed.

iVars:

model	- the OBColumn that controls the node list displayed in this pane
list		- the morph which displays the node list, usually a PluggableListMorph
button	- the morph which controls the column's filter, usually an OBRadioButtonBar.!

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:35'!
adjustList
	self hasButton
			ifFalse: [list bounds: self innerBounds]
			ifTrue: [list bounds: (self innerBounds withHeight: (self height - self buttonHeight))]! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 20:30'!
addButton: aButton
	self hasButton ifTrue: [self removeMorph: button].
	button _ aButton.
	self 
		addMorph: button;
		adjustList;
		adjustButton! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 19:17'!
setColumn: aColumn filter: aFilter
	self model: aColumn.
	list _ aColumn listMorph.
	self initGeometry.
	aFilter ifNotNil: [self addButton: aFilter buttonMorph].! !

!OBPane methodsFor: 'as yet unclassified' stamp: 'cwp 2/12/2004 20:29'!
removeButton
	self hasButton 
		ifTrue: [self removeMorph: button.
				button _ nil.
				self adjustList]! !

!OBPane class methodsFor: 'as yet unclassified' stamp: 'cwp 2/12/2004 18:12'!
forColumn: aColumn withFilter: aFilter
	^ self new setColumn: aColumn filter: aFilter! !

!OBPane class methodsFor: 'as yet unclassified' stamp: 'cwp 2/12/2004 18:11'!
forColumn: aColumn
	^ self forColumn: aColumn withFilter: nil! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 20:23'!
update: aSelector
	aSelector = #filter ifFalse: [^ self].
	(model wantsButton)
			ifTrue: [self addButton: model filter buttonMorph]
			ifFalse: [self removeButton]! !

!OBPane methodsFor: 'as yet unclassified' stamp: 'cwp 2/12/2004 18:47'!
buttonHeight
	^ 20! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:39'!
adjustButton
	| inner |
	self hasButton ifTrue: 	[inner _ self innerBounds.
							button bounds: (inner withTop: inner bottom - self buttonHeight)]! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:26'!
initGeometry
	self	
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
		
	self addMorph: list.
	list bounds: self innerBounds.
		! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:32'!
bounds: aRectangle
	super bounds: aRectangle.
	self adjustList.
	self adjustButton.! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:44'!
model: anObject
	"Set my model and make me me a dependent of the given object."

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model _ anObject.
! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:35'!
hasButton
	^ button notNil! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 2/12/2004 18:14'!
listMorphForColumn: aColumn
	^ (PluggableListMorph
		on: aColumn
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			borderWidth: 0! !

RectangleMorph subclass: #OBRadioButtonBar
	instanceVariableNames: 'model buttons selection getListSelector getSelectionSelector setSelectionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBRadioButtonBar commentStamp: 'cwp 3/5/2004 11:59' prior: 0!
An OBRadioButtonBar is similar to a PluggableListMorph except that it displays a row of buttons rather than a vertical list. Clicking on a button selects it.

model				- the model for this button bar
buttons				- a collection of OBButtonModels, which are derived from the model's list
selection			- the index of the currently selected button
getListSelector	 	- the message sent to the model to get the list of labels for the buttons
getSelectionSelector	- the message sent to the model to get the index of the currently selected item
setSelectionSelector	- the message sent to the model to change the current selection after a button
					   is clicked!

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/25/2004 00:36'!
on: aModel list: listSelector selected: selectionGetter changeSelected: selectionSetter 
	self model: aModel.
	selection _ 0.
	getListSelector _ listSelector.
	getSelectionSelector _ selectionGetter.
	setSelectionSelector _ selectionSetter.
	self initGeometry.
	self updateList.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:28'!
updateMorphs
	self removeAllMorphs.
	buttons do: [:button | self addMorphBack: button morph]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withSelectedButtonDo: aBlock
	^ self withButtonAt: selection do: aBlock! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:39'!
updateSelection
	| oldSelection |
	oldSelection _ selection.
	selection _ self getSelectionIndex.
	self withButtonAt: oldSelection do: [:button | button selectionChanged].
	self withSelectedButtonDo: [:button | button selectionChanged]! !

!OBRadioButtonBar methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:38'!
list 
	buttons ifNil: [self updateButtons].
	^ buttons collect: [:b | b label]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withButtonAt: index do: aBlock
	^ (buttons at: index ifAbsent: [nil]) ifNotNilDo: aBlock! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/25/2004 00:35'!
update: aSymbol
	aSymbol = getListSelector ifTrue: [self updateList. ^ self].
	aSymbol = getSelectionSelector ifTrue: [self updateSelection]! !

!OBRadioButtonBar class methodsFor: 'as yet unclassified' stamp: 'cwp 2/22/2004 16:44'!
on: aModel list: listSelector selected: selectionSelector changeSelected: changedSelector 
	^ self new
			on: aModel
			list: listSelector
			selected: selectionSelector
			changeSelected: changedSelector! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/25/2004 00:35'!
updateList
	self 
		updateButtons; 
		updateMorphs. ! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/24/2004 17:42'!
initGeometry
	self 
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		rubberBandCells: true;
		borderWidth: 0
! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/22/2004 16:51'!
model: aModel
	model ifNotNil: [model removeDependent: self].
	model _ aModel.
	model addDependent: self.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:53'!
getSelectionIndex
	^ model perform: getSelectionSelector! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:53'!
updateButtons
	| labels |
	labels _ model perform: getListSelector.
	buttons _ Array new: labels size.
	labels withIndexDo: [:label :index | 
						buttons at: index put: (OBButtonModel withLabel: label inBar: self)].
	selection _ self getSelectionIndex! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:54'!
isSelected: aButton
	^ (buttons at: selection ifAbsent: [^ false]) == aButton! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:47'!
push: aButton
	| index |
	index _ buttons indexOf: aButton.
	model perform: setSelectionSelector with: index.! !

RectangleMorph subclass: #OBPaneScroller
	instanceVariableNames: 'sizing transform scrollBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBPaneScroller commentStamp: 'cwp 3/5/2004 12:13' prior: 0!
Instances of OBPaneScroller contain the panes which represent columns in a browser. Their primary responsibilities are laying out panes to fit the space available and scrolling them horizontally when there isn't sufficient space. 

iVars:

sizing		- The number of panes which should exactly fit the available space.
		   	   During layout, the  width of the panes is determined accordingly.
transform	- A TransformMorph used for scrolling
scrollBar	- An OBHorizontalScrollBar used for scrolling!

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:10'!
adjustPaneHeight
	"This gets called after the scrollbar has been shown or hidden, to move the bottom
	of the panes to the right place."
	
	transform bounds: self innerBounds.
	transform submorphsDo: [:m | m bounds: (m bounds withHeight: self paneHeight)]
		! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 10:53'!
fullLayout
	self layoutWidgets.
	self layoutPanes.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:04'!
leftoverScrollRange
	^ (self totalScrollRange - self innerBounds width roundTo: self scrollDeltaWidth) max: 0
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:42'!
isScrollable
	^ self leftoverScrollRange > 0! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 19:27'!
totalScrollRange
	| submorphBounds |
	submorphBounds := transform localSubmorphBounds ifNil: [^ 0].
	^ submorphBounds width
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:10'!
hideScrollBar
	self removeMorph: scrollBar.
	self adjustPaneHeight.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 2/11/2004 22:44'!
pushPane: aMorph
	aMorph 
		borderWidth: 0;
		hResizing: #rigid;
		vResizing: #rigid;
		layoutInset: 0.
	transform hasSubmorphs ifTrue: [transform addMorphBack: self separator].
	transform addMorphBack: aMorph.
	self doLayout! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
paneWidthsToFit: limit
	| padded |
	padded _ Array new: self paneCount.
	padded atAllPut: (limit / self sizing) floor.
	(1 to: limit - padded sum) do: [:i | padded at: i put: (padded at: i) + 1].
	^ padded
	
	! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 10:52'!
initializeTransform
	transform _ TransformMorph new.
	transform
		color: Color transparent;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		disableTableLayout;
		bounds: super innerBounds.
	self addMorphBack: transform.
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 12/8/2003 21:42'!
scrollToRight
	^ scrollBar setValue: 1.! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 00:53'!
separatorCount
	^ self paneCount - 1! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:11'!
showScrollBar
	self scrollBarIsVisible ifTrue: [^ self].
	self resizeScrollBar.
	self addMorphFront: scrollBar.
	self adjustPaneHeight.
	! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:03'!
resizeScrollBar
	| inner outer |
	outer _ super innerBounds.
	inner _ outer withHeight: outer height - self scrollBarHeight - 1.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 11:01'!
initializeScrollbar
	scrollBar _ OBScrollBar new model: self slotName: 'scrollBar'.
	scrollBar 
		borderWidth: 0;
		borderColor: #inset;
		height: self scrollBarHeight.
	self resizeScrollBar.
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:44'!
hideOrShowScrollBar
	self isScrollable ifTrue: [self showScrollBar] ifFalse: [self hideScrollBar]! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 21:20'!
bounds: aRectangle
	super bounds: aRectangle.
	self doLayout.! !

!OBPaneScroller methodsFor: 'input events' stamp: 'cwp 8/25/2003 20:33'!
scrollBarValue: value
	transform hasSubmorphs ifFalse: [^ self].
	transform offset: (self leftoverScrollRange * value) rounded @ 0.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:14'!
setScrollDeltas
	| range interval value |
	transform hasSubmorphs ifFalse: [scrollBar interval: 1.0. ^ self].
	range _ self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar interval: 1.0; setValue: 0].
	interval _ ((self innerBounds width) / self totalScrollRange) asFloat.
	value _ (transform offset x / range min: 1.0) asFloat.
	scrollBar interval: interval.
	scrollBar setValue: value.! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:06'!
doLayout
	self layoutWidgets.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 2/8/2004 01:21'!
panes
	| panes subs |
	subs _ transform submorphs.
	panes _ Array new: (subs size / 2) ceiling.
	1 to: subs size by: 2 do: [:i | panes at: (i / 2) ceiling put: (subs at: i)].
	^ panes! !

!OBPaneScroller methodsFor: 'defaults' stamp: 'cwp 8/24/2003 15:13'!
separatorWidth
	^ 4! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 2/11/2004 22:46'!
popPane
	2 timesRepeat: [transform removeMorph: transform lastSubmorph].
	self doLayout! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 22:13'!
layoutWidgets
	| inner outer |
	outer _ super innerBounds.
	inner _ self innerBounds.
	transform bounds: inner.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/23/2003 16:21'!
scrollDeltaWidth
	^ 1! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 12/6/2003 17:07'!
paneCount
	^ (transform submorphs size / 2) ceiling.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 8/24/2003 15:15'!
separator
	^ BorderedSubpaneDividerMorph vertical 
		borderWidth: self separatorWidth / 2;
		color: Color lightGray;
		borderRaised.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 12/6/2003 17:06'!
clearPanes
	transform removeAllMorphs! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
computeMorphWidths
	| paneWidths widths |
	paneWidths _ self paneWidthsToFit: self totalPaneWidth.
	widths _ OrderedCollection new.
	paneWidths do: [:w | widths add: w] separatedBy: [widths add: self separatorWidth].
	^ widths asArray
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/25/2004 20:23'!
totalPaneWidth
	^ self innerBounds width - ((self sizing - 1) * self separatorWidth)! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 14:29'!
scrollBarHeight
	^ 12! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 12/6/2003 17:08'!
paneHeight
	^ transform bounds height! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 10:54'!
innerBounds
	| rect |
	rect _ super innerBounds.
	^ self scrollBarIsVisible
		ifTrue: [rect withHeight: rect height - self scrollBarHeight - 1]
		ifFalse: [rect]! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 2/17/2004 18:39'!
sizing: anInteger
	sizing _ anInteger! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 01:22'!
layoutPanes
	| widths rect |
	widths _ self computeMorphWidths.
	rect _ 0@0 extent: (0 @ self paneHeight).
	transform submorphs 
					with: widths 
					do: [:m :w | 
						rect _ rect withWidth: w.
						m bounds: rect.
						rect _ rect translateBy: (w@0)]
						
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:56'!
scrollBarIsVisible
	^ submorphs includes: scrollBar! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/26/2004 21:13'!
initialize
	super initialize.
	self sizing: 1.
	self 
		color: Color red;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self 
		initializeTransform;
		initializeScrollbar.
! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 2/17/2004 18:39'!
sizing
	^ sizing min: (self panes size)! !

Object subclass: #OBButtonModel
	instanceVariableNames: 'bar label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBButtonModel commentStamp: 'cwp 3/5/2004 12:15' prior: 0!
An OBButtonModel serves as a model for PluggableButtonMorphs used OBRadioButtonBar. OBRadioButtonBar cannot be a direct model for its PBMs, since it can contain a variable number of buttons.

iVars:

bar		- the OBRadioButton bar to which this button belongs
label	- the label of the button!

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:42'!
isSelected
	^ bar isSelected: self! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 3/2/2004 21:46'!
label
	^ label! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:30'!
push
	bar push: self.! !

!OBButtonModel methodsFor: 'morphs' stamp: 'cwp 3/11/2004 22:53'!
morph
	| morph |
	morph _ PluggableButtonMorph
				on: self
				getState: #isSelected
				action: #push
				label: #labelMorph.
	morph 
		onColor: Color lightGray lighter offColor: Color lightGray twiceLighter;
		borderWidth: 2;
		borderRaised;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	^ morph! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
label: aString
	label _ aString! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 19:38'!
selectionChanged
	self changed: #isSelected! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 2/24/2004 18:35'!
withLabel: aString inBar: aRadioButtonBar
	^ self new label: aString; bar: aRadioButtonBar! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 3/2/2004 21:45'!
labelMorph
	^ StringMorph 
		contents: self label 
		font: Preferences standardButtonFont! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
bar: aRadioButtonBar
	bar _ aRadioButtonBar! !

Object subclass: #OBFakeClassA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Fakes'!

OBFakeClassA class
	instanceVariableNames: 'foo'!

!OBFakeClassA commentStamp: 'cwp 2/25/2004 01:09' prior: 0!
Comment for FakeClassA.!

!OBFakeClassA methodsFor: 'numbers' stamp: 'cwp 12/11/2003 20:57'!
three
	^ 3! !

!OBFakeClassA class methodsFor: 'capitals' stamp: 'cwp 2/9/2004 19:57'!
b
	^ $B! !

!OBFakeClassA methodsFor: 'letters' stamp: 'cwp 2/9/2004 20:05'!
a
	^ $a! !

!OBFakeClassA class methodsFor: 'testing' stamp: 'cwp 2/29/2004 13:08'!
acceptsLoggingOfCompilation
	^ false! !

!OBFakeClassA class methodsFor: 'capitals' stamp: ''!
a
	^ $A! !

!OBFakeClassA methodsFor: 'numbers' stamp: ''!
one
	^ 1! !

!OBFakeClassA methodsFor: 'other' stamp: ''!
obHopefullyUniqueSelector
	self obOtherUniqueSelector! !

!OBFakeClassA methodsFor: 'other' stamp: ''!
obOtherUniqueSelector
	^ self! !

!OBFakeClassA methodsFor: 'letters' stamp: ''!
b
	^ $b! !

!OBFakeClassA methodsFor: 'numbers' stamp: ''!
two
	^ 2! !

!OBFakeClassA methodsFor: 'numbers' stamp: ''!
five
	^ self two + self three! !

Object subclass: #OBMethodVersion
	instanceVariableNames: 'sources pointer className category stamp prior selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/14/2004 00:10'!
theClass
	| name theClass |
	name _ (className copyUpTo: $ ) asSymbol.
	theClass _ Smalltalk at: name ifAbsent: [^ nil].
	^ (className endsWith: ' class')
		ifTrue: [theClass class]
		ifFalse: [theClass]! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/13/2004 13:55'!
stamp
	^ stamp! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 3/13/2004 13:37'!
setSources: aSourceFileArray pointer: aSourcePointer
	sources _ aSourceFileArray.
	pointer _ aSourcePointer.
	self parseSource.! !

!OBMethodVersion class methodsFor: 'instance creation' stamp: 'cwp 3/13/2004 13:35'!
fromSources: sources andPointer: pointer
	^ self new setSources: sources pointer: pointer! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 3/13/2004 13:39'!
parseSource
	| file position chunk |
	file _ sources at: (sources fileIndexFromSourcePointer: pointer).
	position _ sources filePositionFromSourcePointer: pointer.
	
	file position: (0 max: position-150).  	"Skip back to before the preamble"
		[file position < (position-1)]  	"then pick it up from the front"
			whileTrue: [chunk _ file nextChunk].
	
	self parseChunk: chunk.! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 3/13/2004 13:46'!
previous
	^ prior ifNotNil: [self class fromSources: sources andPointer: prior]! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 3/14/2004 23:38'!
setSelector
	| file |
	file _ sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ selector _ Parser new parseSelector: file nextChunk.! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/14/2004 23:37'!
selector
	^ selector ifNil: [self setSelector]! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/14/2004 23:10'!
source
	| file |
	file _ sources at: (sources fileIndexFromSourcePointer: pointer).
	file position: (sources filePositionFromSourcePointer: pointer).
	^ file nextChunk asText makeSelectorBold! !

!OBMethodVersion class methodsFor: 'scanning' stamp: 'cwp 3/13/2004 15:13'!
scan: sources from: pointer 
	| versions current |
	
	versions _ OrderedCollection new.
	current _ OBMethodVersion fromSources: sources andPointer: pointer.
	[current notNil]
		whileTrue: [versions add: current.
					current _ current previous].
	^ versions! !

!OBMethodVersion class methodsFor: 'scanning' stamp: 'cwp 3/13/2004 15:14'!
versionsOfMethod: methodReference 
	| class selector |
	class _ methodReference actualClass.
	selector _ methodReference methodSymbol.
	^ self scan: SourceFiles from: (class compiledMethodAt: selector) sourcePointer! !

!OBMethodVersion methodsFor: 'compiling' stamp: 'cwp 3/14/2004 00:15'!
fileIn
	(self theClass) ifNotNilDo: [:class | class
						 				compile: self source 
										classified: self category 
										withStamp: self stamp 
										notifying: nil]
						
						! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/13/2004 13:51'!
classSymbol
	^ className! !

!OBMethodVersion methodsFor: 'initializing' stamp: 'cwp 3/13/2004 13:19'!
parseChunk: aString
	| tokens |
	tokens _ Scanner new scanTokens: aString.
	className _ tokens first.
	category _ tokens third.
	stamp _ tokens fifth.
	tokens size > 5 ifTrue: [prior _ tokens seventh].! !

!OBMethodVersion methodsFor: 'accessing' stamp: 'cwp 3/13/2004 13:52'!
category
	^ category! !

Object subclass: #OBBrowser
	instanceVariableNames: 'panels columns scroller minPanes'
	classVariableNames: 'MetaGraphs'
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser commentStamp: 'cwp 3/4/2004 21:30' prior: 0!
OBBrowser is the core of OmniBrower. It's the root object for each browser and the model for the SystemWindows which display them. Its main responsibilies are managing columns and distributing update messages.

The browser UI is divided into two parts. At the top are the columns, which display the structure of the node graph. As nodes are selected, additional columns are added to the browser to hold the children of the selected nodes. Below the columns are panels, which allow the user to interact with the selected node.  The only panel implemented so far is OBTextPanel, but others such as annotation panes, or button bars could be created.

iVars:

panels	- A collection of objects which manage the other (non-column) parts of the browser.
columns	- A collection of OBColumns, each of which manages a single pane in the scroller.
scroller	- The OBPaneScroller which owns the column panes.
minPanes - The minimum number of panes the browser will display.

cVars:

MetaGraphs - A dictionary matching names to metagraphs!

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 3/9/2004 21:38'!
addPanel: aPanel
	panels add: aPanel! !

!OBBrowser methodsFor: 'notification' stamp: 'cwp 1/30/2004 00:40'!
nodeCreated: aNode
	self jumpTo: aNode! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 3/9/2004 22:32'!
window
	| window |
	window _ SystemWindow labelled: self label.
	window model: self.
	window 
		addMorph: self scroller
		fullFrame: (LayoutFrame fractions: (0@0 extent: 1 @ self columnProportion)).
	panels isEmpty ifFalse: [self addPanelMorphsTo: window].
	^ window! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 2/11/2004 23:20'!
emptyColumn
	^ OBColumn inBrowser: self! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/17/2003 16:38'!
metaNode: aMetaNode node: aNode
	^ self new setMetaNode: aMetaNode node: aNode! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/11/2004 22:48'!
popColumn
	columns removeLast.
	scroller popPane.
! !

!OBBrowser class methodsFor: 'metaGraphs' stamp: 'cwp 12/18/2003 18:13'!
removeMetaGraphAt: aSymbol
	MetaGraphs removeKey: aSymbol! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 3/9/2004 22:26'!
columnProportion
	^ panels isEmpty
		ifTrue: [1]
		ifFalse: [0.4]! !

!OBBrowser class methodsFor: 'metaGraphs' stamp: 'cwp 12/18/2003 19:50'!
metaGraphs
	^ MetaGraphs! !

!OBBrowser class methodsFor: 'initialize-release' stamp: 'cwp 3/3/2004 22:53'!
initialize
	MetaGraphs _ Dictionary new.
	OBMetaGraphBuilder default installGraphs.
	TheWorldMenu registerOpenCommand: {'Image Browser'. {self. #browseImage}}.
	TheWorldMenu registerOpenCommand: {'Package Browser'. {self. #browsePackages}}.
	TheWorldMenu registerOpenCommand: {'File Browser'. {self. #browseFileSystem}}.! !

!OBBrowser class methodsFor: 'metaGraphs' stamp: 'cwp 2/17/2004 18:24'!
metaGraphMissing: aSymbol
	^ self error: 'There is no metagraph named ', aSymbol.! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 3/9/2004 00:00'!
addPanelMorphsTo: window
	| heights variableCount fraction offset vExtent h |
	heights _ panels collect: [:ea | ea morphHeight].
	variableCount _ heights inject: 0 into: [:count :ea | ea = 0 ifTrue: [count + 1] ifFalse: [count]].
	fraction _ self columnProportion.
	offset _ 0.
	vExtent _  1- fraction / variableCount.
	panels with: heights do:  [:panel :height | 
		h _ height = 0 ifTrue: [vExtent] ifFalse: [0].
		window 
			addMorph: panel morph
			fullFrame: (LayoutFrame
							fractions: (0@fraction extent: 1@h)
							offsets: (0@offset corner: 0@height)).
		height = 0 
			ifTrue: 	[offset _ 0.
					fraction _ fraction + vExtent]
			ifFalse: 	[offset _ offset + height]]
						
		
						! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/11/2004 23:39'!
deselected: aColumn
	self trimColumnsTo: (columns indexOf: aColumn)
! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/27/2004 00:45'!
nodeChanged
	panels do: [:ea | ea nodeChanged]! !

!OBBrowser methodsFor: 'testing' stamp: 'cwp 3/4/2004 02:38'!
isOpen
	^  scroller 
		ifNil: [false]
		ifNotNil: [scroller world notNil]! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 2/11/2004 23:12'!
jumpTo: aNode
	| column ancestor |
	column _ columns first.
	[ancestor _ column selectAncestorOf: aNode.
	ancestor = aNode or: [ancestor isNil]]
		whileFalse: [column _ columns after: column].					
! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/11/2004 21:23'!
columnAfter: aColumn
	^ [columns after: aColumn]
		on: Error
		do: [^ nil]! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:34'!
pushColumn: aColumn
	columns addLast: aColumn.
	self scroller pushPane: aColumn morph.
! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 2/26/2004 21:03'!
browse: aSymbol fromRoot: aNode minPanes: min maxPanes: max
	^ self browse: aSymbol fromRoot: aNode select: nil minPanes: min maxPanes: max ! !

!OBBrowser class methodsFor: 'convenience' stamp: 'cwp 2/26/2004 20:47'!
browseImage
	^ self 
		browse: #Image
		fromRoot: (OBSystemNode on: OBImageContext new)
		panes: 4
! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 2/26/2004 21:16'!
scroller
	^ scroller ifNil: [scroller _ OBPaneScroller new.
					scroller 
						name: 'scroller';
						pushPane: columns first morph;
						yourself]
	! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 12/10/2003 22:51'!
open
	^ self window openInWorld! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 2/26/2004 20:53'!
browse: aSymbol fromRoot: aNode
	^ self browse: aSymbol fromRoot: aNode minPanes: 1 maxPanes: 4! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/28/2004 14:06'!
okToChange
	panels do: [:ea | ea okToChange ifFalse: [^ false]].
	^ true! !

!OBBrowser class methodsFor: 'convenience' stamp: 'cwp 2/28/2004 17:05'!
browsePackage: aString
	^ self 
		browse: #Package 
		fromRoot: (OBPackageNode on: (PackageInfo named: aString))
		panes: 4! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 11/23/2003 18:02'!
label
	^ 'OmniBrowser'! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 3/9/2004 21:37'!
setMetaNode: aMetaNode node: aNode
	aNode metaNode: aMetaNode.
	panels _ OrderedCollection new.
	columns _ OrderedCollection with: (aMetaNode
										columnInBrowser: self
										node: aNode).
	minPanes _ 1! !

!OBBrowser class methodsFor: 'convenience' stamp: 'cwp 3/2/2004 22:21'!
browseMetaGraphs
	^ self 
		browse: #MetaGraphs
		fromRoot: (OBDictionaryNode on: MetaGraphs)
		panes: 4! !

!OBBrowser methodsFor: 'accessing' stamp: 'avi 3/6/2004 16:25'!
currentColumn
	^ columns reversed detect: [:ea | ea hasSelection] ifNone: []
! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 2/16/2004 22:16'!
jumpToRoot
	^ columns first selection: 0! !

!OBBrowser methodsFor: 'testing' stamp: 'cwp 1/31/2004 10:26'!
hasMorphs
	^ scroller notNil! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/17/2004 18:43'!
minPanes: anInteger
	minPanes _ anInteger.
	(minPanes - (columns size)) timesRepeat: [self pushColumn: self emptyColumn]! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 2/7/2004 15:57'!
root
	^ columns first parent! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 2/26/2004 20:50'!
browse: aSymbol fromRoot: aNode panes: anInteger
	^ self browse: aSymbol fromRoot: aNode minPanes: anInteger maxPanes: anInteger! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 3/10/2004 00:54'!
browse: aSymbol fromRoot: rootNode select: selectedNode minPanes: min maxPanes: max 
	| browser |
	browser _ self metaNode: (self metaGraphAt: aSymbol) node: rootNode.
	browser 
		minPanes: min;
		maxPanes: max.
	Preferences optionalButtons ifTrue: [browser addPanel: (OBButtonPanel inBrowser: browser)].
	browser addPanel: (OBTextPanel inBrowser: browser).
	selectedNode ifNotNil: [browser jumpTo: selectedNode].
	browser open.
	^ browser! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/17/2004 18:42'!
minPanes
	^ minPanes! !

!OBBrowser methodsFor: 'accessing' stamp: 'avi 3/6/2004 16:25'!
currentOrRootNode
	^ (columns reversed 
				detect: [:ea | ea hasSelection] 
				ifNone: [^ columns first parent]) selectedNode
! !

!OBBrowser class methodsFor: 'convenience' stamp: 'cwp 1/31/2004 10:21'!
browseFileSystem
	^ self 
		browse: #FileSystem
		fromRoot: (OBDirectoryNode on: FileDirectory root)
		select: (OBDirectoryNode default).
! !

!OBBrowser class methodsFor: 'metaGraphs' stamp: 'cwp 2/17/2004 18:24'!
metaGraphAt: aSymbol
	^ MetaGraphs at: aSymbol ifAbsent: [self metaGraphMissing: aSymbol]! !

!OBBrowser class methodsFor: 'convenience' stamp: 'cwp 2/26/2004 20:48'!
browsePackages
	^ self 
		browse: #System
		fromRoot: (OBSystemNode on: OBImageContext new)
		panes: 5! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/11/2004 23:39'!
selected: aColumn
	| count |
	count _ columns indexOf: aColumn.
	aColumn shouldBeLast ifFalse: [count _ count + 1].
	self trimColumnsTo: count
! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/17/2004 18:43'!
trimColumnsTo: anInteger
	| count |
	count _ self minPanes max: anInteger.
	columns size - count timesRepeat: [self popColumn].
! !

!OBBrowser class methodsFor: 'metaGraphs' stamp: 'cwp 12/18/2003 18:13'!
metaGraphAt: aSymbol put: aMetaGraph
	MetaGraphs at: aSymbol put: aMetaGraph! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 1/26/2004 21:47'!
currentNode
	^ self currentColumn ifNotNilDo: [:column | column selectedNode]! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 3/1/2004 18:22'!
refresh
	self isOpen ifTrue: [columns do: [:ea | ea refresh]]! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 2/26/2004 21:04'!
browse: aSymbol fromRoot: rootNode select: selectedNode
	^ self browse: aSymbol fromRoot: rootNode select: selectedNode minPanes: 1 maxPanes: 3! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 2/26/2004 20:43'!
maxPanes: anInteger
	self scroller sizing: anInteger! !

Object subclass: #OBPanel
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPanel methodsFor: 'initializing' stamp: 'cwp 3/9/2004 23:25'!
setBrowser: aBrowser
	browser _ aBrowser! !

OBPanel subclass: #OBTextPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBTextPanel commentStamp: 'cwp 3/5/2004 14:34' prior: 0!
OBTextPanel serves as the model for the text pane of the browser. It's main responsibility is to act as a relay between the currently selected node and a PluggableTextMorph.

iVars:

browser - The browser of which this panel is a part.
!

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 2/27/2004 20:46'!
menu: aMenu shifted: aBoolean
	| legacy |
	legacy _  aBoolean 
		ifTrue: [ParagraphEditor shiftedYellowButtonMenu] 
		ifFalse: [ParagraphEditor yellowButtonMenu].
	^ aMenu
		labels: legacy labelString
		lines: legacy lineArray
		selections: legacy selections! !

!OBTextPanel methodsFor: 'updating' stamp: 'cwp 2/28/2004 14:18'!
okToChange
	self canDiscardEdits ifTrue: [^ true].
	ok _ OBConfirmationRequest
			prompt: 'Changes have not been saved.
Is it OK to discard those changes?'
			confirm: 'Discard changes'.
	ok ifTrue: [self changed: #clearUserEdits].
	^  ok.! !

!OBTextPanel methodsFor: 'constructing' stamp: 'cwp 3/9/2004 21:35'!
morphHeight
	^ 0! !

!OBTextPanel methodsFor: 'updating' stamp: 'cwp 2/27/2004 00:45'!
nodeChanged
	self changed: #text! !

!OBTextPanel methodsFor: 'constructing' stamp: 'cwp 3/2/2004 21:15'!
morph
	^ (PluggableTextMorph
		on: self
		text: #text
		accept: #accept:notifying:
		readSelection: #selection
		menu: #menu:shifted:)
			font: Preferences standardCodeFont;
			yourself
			
	"see CodeHolder>>buildMorphicCodePaneWith:"! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 18:25'!
selection
	browser currentNode ifNotNilDo: [:node | ^ node textSelection].
	^ 1 to: 0! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 3/14/2004 14:16'!
accept: aText notifying: aController
	| result node |
	node _ browser currentNode ifNil: [^ self].
	[result _ node accept: aText notifying: aController]
		on: OBNotification
		do: [:notification | notification notify: browser; resume].
	^ result! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 2/27/2004 00:40'!
text
	browser currentNode ifNotNilDo: [:node | ^ node text].
	^ ''! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 3/9/2004 23:52'!
nodeChanged
	! !

!OBPanel class methodsFor: 'instance creation' stamp: 'cwp 3/9/2004 23:26'!
inBrowser: aBrowser
	^ self new setBrowser: aBrowser! !

OBPanel subclass: #OBButtonPanel
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBButtonPanel methodsFor: 'constructing' stamp: 'cwp 3/10/2004 23:55'!
morph
	^ morph  ifNil: [morph _ RectangleMorph new
					layoutPolicy: TableLayout new;
					listDirection: #leftToRight;
					hResizing: #spaceFill;
					vResizing: #spaceFill;
					borderWidth: 0;
					wrapCentering: #center;
					cellPositioning: #leftCenter;
					rubberBandCells: true;
					yourself]! !

!OBButtonPanel methodsFor: 'updating' stamp: 'cwp 3/11/2004 18:58'!
nodeChanged
	| node actions |
	node _ browser currentNode.
	actions _ (node metaNode actionsForNode: node) select: [:ea | ea wantsButton].
	self morph removeAllMorphs.
	actions do: [:ea | morph addMorphBack: ea button]! !

!OBButtonPanel methodsFor: 'constructing' stamp: 'cwp 3/10/2004 00:58'!
morphHeight
	^ 25! !

Object subclass: #OBNode
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBNode commentStamp: 'cwp 3/4/2004 22:20' prior: 0!
A node is a wrapper for an object in the browser's domain graph. OBNode is an abstract superclass for concrete nodes which might appear in the browser.

iVars:

metaNode - the MetaNode which produced this node!

!OBNode methodsFor: 'public' stamp: 'cwp 1/31/2004 14:57'!
hash
	^ self name hash! !

!OBNode methodsFor: 'public' stamp: 'cwp 1/31/2004 14:57'!
= other
	^ self class = other class and: [self name = other name]! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/10/2003 22:27'!
name
	self subclassResponsibility! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/3/2004 22:29'!
action: aSelector withLabel: aString
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:00'!
asDraggableMorph
	^(StringMorph contents: self name)
		color: Color white;
		yourself! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/25/2004 22:37'!
signalSelection
	^ (OBNodeSelected for: self) signal! !

OBNode subclass: #OBMetaNodeNode
	instanceVariableNames: 'theMetaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMetaNodeNode methodsFor: 'as yet unclassified' stamp: 'cwp 3/6/2004 22:25'!
children
	^ theMetaNode children collect: [:mn | OBMetaNodeNode on: mn]! !

!OBMetaNodeNode methodsFor: 'as yet unclassified' stamp: 'cwp 3/6/2004 22:25'!
text
	^ 'MetaNode named: ''', theMetaNode name, ''''! !

!OBMetaNodeNode methodsFor: 'as yet unclassified' stamp: 'cwp 3/6/2004 22:22'!
theMetaNode: aMetaNode
	theMetaNode _ aMetaNode! !

!OBMetaNodeNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/6/2004 22:22'!
on: aMetaNode
	^ self new theMetaNode: aMetaNode! !

!OBMetaNodeNode methodsFor: 'as yet unclassified' stamp: 'cwp 3/6/2004 22:23'!
name
	^ theMetaNode name! !

OBNode subclass: #OBDictionaryNode
	instanceVariableNames: 'dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBDictionaryNode methodsFor: 'as yet unclassified' stamp: 'cwp 3/3/2004 22:30'!
associations
	^ dictionary keys asArray sort
		collect: [:key | OBAssociationNode key: key value: (dictionary at: key)]! !

!OBDictionaryNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:01'!
on: aDictionary
	^ self new dictionary: aDictionary! !

!OBDictionaryNode methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:00'!
dictionary: aDictionary
	dictionary _ aDictionary! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
isDescendantOf: aNode
	^ false! !

OBNode subclass: #OBAssociationNode
	instanceVariableNames: 'key value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBAssociationNode methodsFor: 'public' stamp: 'cwp 12/18/2003 22:13'!
value
	^ value! !

!OBAssociationNode methodsFor: 'public' stamp: 'cwp 12/18/2003 20:15'!
name
	^ key! !

!OBAssociationNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:14'!
key: keyObject value: valueObject
	^ self new setKey: keyObject value: valueObject! !

!OBAssociationNode methodsFor: 'initializing' stamp: 'cwp 12/18/2003 20:10'!
setKey: keyObject value: valueObject
	key _ keyObject.
	value _ valueObject.! !

!OBAssociationNode methodsFor: 'public' stamp: 'cwp 3/3/2004 22:27'!
children
	^ Array with: value asNode! !

OBNode subclass: #OBFakeNode
	instanceVariableNames: 'item parent children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeNode methodsFor: 'public' stamp: 'cwp 1/23/2004 21:42'!
children
	^ children 
		ifNil: [children _ (self item endsWith: 'c')
					ifTrue: [Array new]
					ifFalse: [#($a $b $c) collect: [:i | OBFakeNode parent: self item: i asString]]]! !

!OBFakeNode class methodsFor: 'instance creation' stamp: 'cwp 12/15/2003 22:06'!
parent: aFakeNode item: aString
	^ self new parent: aFakeNode; item: aString! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:53'!
removeChild: aFakeNode
	children _ self children copyWithout: aFakeNode! !

!OBFakeNode class methodsFor: 'browsing' stamp: 'cwp 3/14/2004 14:00'!
browse
	| browser |
	browser _ (OBBrowser metaNode: self metaGraph node: OBFakeNode new).
	browser 
		minPanes: 3;
		maxPanes: 3;
		addPanel: (OBTextPanel inBrowser: browser);
		open! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 3/14/2004 13:56'!
text: aText
	self item:( aText asString allButFirst: (self parentName size)).
	self signalSelection.
	^ true! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:01'!
parent: aFakeNode
	parent _ aFakeNode! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 2/25/2004 22:39'!
adopt: aNode
	aNode parent removeChild: aNode.
	aNode parent: self.
	children _ self children copyWith: aNode.
	self signalSelection! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 2/25/2004 22:42'!
delete
	parent ifNotNil: [parent removeChild: self].
	OBNodeDeleted signal.	! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 2/25/2004 22:06'!
isAncestorOf: aNode
	| ancestor |
	ancestor _ aNode.
	[ancestor isNil]
		whileFalse: [ancestor == self ifTrue: [^ true].
					ancestor _ ancestor parent].
	^ false! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:09'!
item: aSymbol
	item _ aSymbol! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 2/3/2004 22:30'!
actions
	^ Array 
		with: (self action: #beZ withLabel: 'become z')
		with: (self action: #delete withLabel: 'delete')! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:40'!
dropSelector
	^ #dropOnFakeNode:! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:05'!
name
	^ self parentName, self item ! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:49'!
parent
	^ parent! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:04'!
parentName
	^ (parent ifNil: [''] ifNotNil: [parent name])! !

!OBFakeNode class methodsFor: 'browsing' stamp: 'cwp 3/3/2004 23:10'!
metaGraph
	| fake |
	fake _ OBMetaNode new.
	fake 
		childAt: #children put: fake;
		addActor: (OBNodeActor onNodeClass: OBFakeNode).
	^ fake! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:18'!
item
	^ item ifNil: [item _ '']! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 13:50'!
beZ
	item _ 'z'.
	OBListChanged signal! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:03'!
text
	^ self name! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:41'!
dropOnFakeNode: aNode
	aNode adopt: self! !

OBNode subclass: #OBFileNode
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBFileNode methodsFor: 'public' stamp: 'cwp 12/8/2003 22:20'!
text: aText
	| file |
	file _ self openFile.
	file
		reset;
		nextPutAll: aText asString;
		truncate: (file position);
		close! !

!OBFileNode methodsFor: 'public' stamp: 'cwp 12/8/2003 22:21'!
openFile
	^ FileStream oldFileNamed: path! !

!OBFileNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:58'!
hash
	^ self path hash! !

!OBFileNode methodsFor: 'public' stamp: 'cwp 12/7/2003 20:00'!
name
	^ FileDirectory localNameFor: path! !

!OBFileNode class methodsFor: 'instance creation' stamp: 'cwp 12/7/2003 19:43'!
on: aPath
	^ self new path: aPath! !

!OBFileNode methodsFor: 'accessing' stamp: 'cwp 12/7/2003 19:44'!
path: aPath
	path _ aPath! !

!OBFileNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:58'!
= other
	^ self path = other path! !

!OBFileNode methodsFor: 'actions' stamp: 'cwp 2/25/2004 22:42'!
delete
	FileDirectory deleteFilePath: path.
	OBNodeDeleted signal.	! !

!OBFileNode methodsFor: 'accessing' stamp: 'cwp 1/24/2004 00:16'!
path
	^ path! !

!OBFileNode methodsFor: 'public' stamp: 'cwp 2/3/2004 22:30'!
actions
	^ Array with: (self action: #delete withLabel: 'delete')! !

!OBFileNode methodsFor: 'public' stamp: 'cwp 2/12/2004 20:52'!
text
	| file text |
	file _ self openFile.
	file ifNil: [^ 'Error opening file'].
	text _ file contentsOfEntireFile.
	file close.
	^ text! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:49'!
accept: aText notifying: aController
	^ self text: aText! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:27'!
text
	^ ''! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
ancestrySelector
	^ #isDescendantOf: ! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
acceptDroppedNode: aNode
 	^ aNode perform: self dropSelector with: self
! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:30'!
textSelection
	^ 1 to: 0! !

!OBNode methodsFor: 'public' stamp: 'cwp 1/7/2004 09:25'!
actions
	^ #()! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/14/2004 14:09'!
text: aText
	^ false! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 22:11'!
fromAssociation: anAssociationNode
	^ self on: anAssociationNode value! !

!OBNode methodsFor: 'utility' stamp: 'cwp 3/10/2004 00:29'!
action: aSelector buttonLabel: buttonString menuLabel: menuString
	^ OBAction
		label: menuString
		buttonLabel: buttonString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/7/2004 21:29'!
referenceForMethod: selector ofClass: className
	| classIsMeta symbol |
	classIsMeta _ className endsWith: ' class'.
	symbol _ classIsMeta ifTrue: [(className allButLast: 6) asSymbol] ifFalse: [className].
	^ MethodReference new 
		setClassSymbol: symbol
		classIsMeta: classIsMeta
		methodSymbol: selector
		stringVersion: symbol, '>>', selector! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode
	^ metaNode! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:47'!
isAncestorOf: aNode
	^ aNode perform: self ancestrySelector with: self! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
wantsDroppedNode: aNode
	^ aNode respondsTo: self dropSelector! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:25'!
actionsForParent: aNode
	^ #()! !

OBNode subclass: #OBCodeNode
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

OBCodeNode subclass: #OBClassCreatingNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

OBClassCreatingNode subclass: #OBClassCategoryNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBClassCategoryNode class methodsFor: 'actions' stamp: 'cwp 2/28/2004 21:58'!
createInContext: aContext
	| catName |
	catName _ OBTextRequest 
					prompt: 'Please type new category name' 
					template: 'Category-Name'.
	catName ifNotNil:	[aContext addCategory: catName.
						(OBClassCategoryNode 
							on: catName 
							inContext: aContext) signalSelection]! !

!OBClassCategoryNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:30'!
dropSelector
	^ #dropOnClassCategory:! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 2/25/2004 22:41'!
remove
	context removeCategory: name.
	OBNodeDeleted signal.! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:36'!
isDescendantOfClassCat: other
	^ other name = name! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 01:25'!
browse
	Browser newOnCategory: (context longNameOf: name)! !

!OBClassCategoryNode class methodsFor: 'actions' stamp: 'cwp 2/5/2004 20:34'!
actionsForParent: aNode
	^ Array
		with: (OBAction 
				label: 'create category...'
				receiver: self
				selector: #createInContext:
				arguments: (Array with: aNode context))
		! !

!OBClassCategoryNode methodsFor: 'public' stamp: 'cwp 3/4/2004 00:11'!
name
	^ (name isEmpty)
		ifFalse: [name]
		ifTrue: [context longNameOf: name]! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 00:44'!
actions
	^ Array
		with: (self action: #browse buttonLabel: 'browse' menuLabel: 'view in standard browser')
		with: (self action: #remove withLabel: 'remove')
		with: (self action: #rename withLabel: 'rename...')! !

!OBClassCategoryNode methodsFor: 'public' stamp: 'cwp 2/25/2004 01:18'!
comments
	^ (context classesInCategory: name) 
		collect: [:class | OBClassCommentNode on: class inContext: context]! !

!OBClassCategoryNode methodsFor: 'class creation' stamp: 'cwp 1/29/2004 23:43'!
subclassDefiner
	^ nil subclassDefinerClass! !

!OBClassCategoryNode class methodsFor: 'instance creation' stamp: 'cwp 12/10/2003 23:17'!
on: aString inContext: aCodeContext
	^ self new setName: aString context: aCodeContext! !

!OBClassCategoryNode methodsFor: 'navigation' stamp: 'cwp 2/2/2004 21:47'!
classCategoryNode
	^ self! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:54'!
isDescendantOfPackage: aPackageNode
	^ (aPackageNode context categories) includes: name! !

!OBClassCategoryNode methodsFor: 'public' stamp: 'cwp 12/17/2003 18:16'!
classes
	^ (context classesInCategory: name) 
		collect: [:class | OBClassNode on: class inContext: context]! !

!OBClassCategoryNode methodsFor: 'public' stamp: 'cwp 2/7/2004 20:53'!
metaclasses
	^ (context classesInCategory: name) 
		collect: [:class | OBMetaclassNode on: class inContext: context]! !

!OBClassCategoryNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 10:56'!
isRedefinedBy: definition
	"No class is selected, so the definition can't be a redefinition."
	
	^ false! !

!OBClassCategoryNode methodsFor: 'public' stamp: 'cwp 1/29/2004 23:06'!
text
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', (context longNameOf: name), ''''! !

!OBClassCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:26'!
ancestrySelector
	^ #isDescendantOfClassCat:! !

!OBClassCategoryNode methodsFor: 'actions' stamp: 'cwp 2/25/2004 22:42'!
rename
	| category |
	category _ OBTextRequest
				prompt: 'Please type new category name' 
				template: name.
	category ifNotNil:	[context renameCategory: name toBe: category.
						name _ category.
						OBListChanged signal]! !

!OBClassCategoryNode methodsFor: 'initializing' stamp: 'cwp 12/14/2003 21:08'!
setName: aString context: aCodeContext
	name _ aString.
	context _ aCodeContext.
! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 11:45'!
definedClassExists: definition
	^ context classExists: (self nameOfClassDefinedBy: definition)! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 11:43'!
nameOfClassDefinedBy: definition
	^ (Scanner new scanTokens: definition) third! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 11:49'!
confirmDefinition: definition
	"Check to make sure the user isn't accidentally over-writing an existing class."
	
	(((self isRedefinedBy: definition) not) and: [self definedClassExists: definition])
		ifTrue: [^ self confirmRedefinition: definition]
		ifFalse: [^ true]! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 10:27'!
defineClass: definition notifying: aController
	| newClass |
	(self confirmDefinition: definition) ifFalse: [^ false].
	newClass _ self subclassDefiner 
					evaluate: definition 
					notifying: aController
					logged: true.
	newClass isMeta ifFalse: [(OBClassNode 
								on: newClass name 
								inContext: context) signalSelection].
	^ true
! !

OBClassCreatingNode subclass: #OBClassNode
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBClassNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:01'!
isDescendantOfPackage: aPackageNode
	^ aPackageNode context classExists: className! !

!OBClassNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:27'!
ancestrySelector
	^ #isDescendantOfClass:! !

!OBClassNode methodsFor: 'public' stamp: 'cwp 2/7/2004 15:10'!
name
	^ className asString! !

!OBClassNode methodsFor: 'nodes' stamp: 'cwp 2/7/2004 15:08'!
classCategoryNode
	^ OBClassCategoryNode on: (context categoryOf: className) inContext: context! !

!OBClassNode methodsFor: 'public' stamp: 'cwp 2/7/2004 17:02'!
symbol
	^ className! !

!OBClassNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 15:15'!
on: className inContext: aCodeContext
	^ self new setClassName: className context: aCodeContext! !

!OBClassNode methodsFor: 'nodes' stamp: 'cwp 1/30/2004 01:01'!
classNode 
	^ self! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 3/10/2004 19:28'!
instanceVariables
	^ (context instanceVariablesForClass: className)
		collect: [:ea | OBInstanceVariableNode on: ea inClass: className inContext: context]! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 2/16/2004 20:41'!
allCategory
	^ OBAllMethodCategoryNode on: className inContext: context! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 3/3/2004 23:23'!
rename
	| newName |
	newName _ OBTextRequest
				prompt: 'Please type new class name' 
				template: className asString.
	newName ifNotNil:	[context renameClass: className toBe: newName asSymbol.
						className _ newName.
						OBListChanged signal]! !

!OBClassNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:04'!
isDescendantOfClassCat: aClassCategoryNode
	^ (context categoryOf: className) = aClassCategoryNode name! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 00:45'!
browse
	Browser fullOnClass: self theClass! !

!OBClassNode methodsFor: 'initializing' stamp: 'cwp 2/7/2004 15:06'!
setClassName: aSymbol context: aCodeContext
	className _ aSymbol.
	context _ aCodeContext! !

!OBClassNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 11:47'!
isRedefinedBy: definition
	^ className = (self nameOfClassDefinedBy: definition)! !

!OBClassNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:04'!
isDescendantOfClass: other
	^ other symbol = className! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 3/11/2004 21:54'!
chaseIvars
	OBBrowser 
		browse: #ChaseInstanceVariables 
		fromRoot: self
		panes: 3! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 19:58'!
actions
	^ Array
		with: (self action: #browse buttonLabel: 'browse' menuLabel: 'view in standard browser')
		with: (self action: #chaseIvars buttonLabel: 'ivars' menuLabel: 'chase instance variables')
		with: (self action: #rename withLabel: 'rename...')
		with: (self action: #remove withLabel: 'remove')! !

OBClassNode subclass: #OBMetaclassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMetaclassNode methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 21:33'!
name 
	^ self nonMetaName! !

!OBMetaclassNode methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 21:32'!
nonMetaName
	^ (className allButLast: 6) asSymbol! !

!OBMetaclassNode methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 21:24'!
setClassName: aSymbol context: aCodeContext
	super 
		setClassName: (aSymbol, ' class') asSymbol
		context: aCodeContext! !

!OBClassNode methodsFor: 'navigating' stamp: 'cwp 2/16/2004 20:42'!
categories
	^ ((context categoriesForClass: className)
		collect: [:cat | OBMethodCategoryNode on: cat inClass: className inContext: context])
			copyWithFirst: self allCategory! !

!OBClassNode methodsFor: 'public' stamp: 'cwp 2/7/2004 15:08'!
text
	^ self theClass definition! !

!OBClassNode methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:20'!
dropOnClassCategory: aNode
	aNode context setCategoryOf: className to: aNode name.
	self signalSelection! !

!OBClassNode methodsFor: 'class creation' stamp: 'cwp 2/7/2004 15:09'!
subclassDefiner
	^ self theClass superclass subclassDefinerClass! !

!OBClassNode methodsFor: 'actions' stamp: 'cwp 2/28/2004 13:14'!
remove
	OBConfirmationRequest
		prompt: 'Are you certain that you 
want to REMOVE the class ', className, ' from the system?'
		confirm: 'Remove'.
	context removeClass: className.
	OBNodeDeleted signal.! !

!OBClassNode methodsFor: 'public' stamp: 'cwp 2/9/2004 20:13'!
theClass
	^ context classNamed: className! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 12:40'!
confirmRedefinition: definition
	| newName prompt |
	newName _ self nameOfClassDefinedBy: definition.
	prompt _ (newName, ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText.
	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Redefine'! !

!OBClassCreatingNode methodsFor: 'class creation' stamp: 'cwp 2/28/2004 10:57'!
isRedefinedBy: definition
	self subclassResponsibility! !

!OBClassCreatingNode methodsFor: 'callbacks' stamp: 'cwp 2/28/2004 10:25'!
accept: definition notifying: aController
	^ self defineClass: definition notifying: aController! !

!OBCodeNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 00:53'!
context
	^ context! !

OBCodeNode subclass: #OBInstanceVariableNode
	instanceVariableNames: 'className name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBInstanceVariableNode methodsFor: 'navigating' stamp: 'cwp 3/11/2004 20:54'!
accessors
	^ (context accessesToVariable: name ofClass: className)
		collect: [:ref | OBMethodNode nakedOn: ref inContext: context]! !

!OBInstanceVariableNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/10/2004 19:23'!
on: instVarName inClass: className inContext: aContext
	^ self new setName: instVarName class: className context: aContext! !

!OBInstanceVariableNode methodsFor: 'displaying' stamp: 'cwp 3/10/2004 19:25'!
name
	^ name! !

!OBInstanceVariableNode methodsFor: 'initializing' stamp: 'cwp 3/10/2004 19:24'!
setName: aString class: aSymbol context: aContext
	name _ aString.
	className _ aSymbol.
	context _ aContext! !

OBCodeNode subclass: #OBMethodCreatingNode
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMethodCreatingNode methodsFor: 'nodes' stamp: 'cwp 2/9/2004 20:14'!
theClass
	^ context classNamed: className! !

!OBMethodCreatingNode methodsFor: 'method creation' stamp: 'cwp 3/1/2004 23:22'!
methodNodeFor: aSelector
	^ OBMethodNode 
		on: aSelector
		inClass: className
		inContext: context! !

!OBMethodCreatingNode methodsFor: 'confirming' stamp: 'cwp 3/4/2004 01:08'!
selectorAlreadyDefined: aSelector
	^ ((context methodsForClass: className) collect: [:ref | ref methodSymbol]) includes: aSelector! !

!OBMethodCreatingNode methodsFor: 'public' stamp: 'cwp 1/31/2004 14:18'!
context
	^ context! !

!OBMethodCreatingNode methodsFor: 'public' stamp: 'cwp 3/2/2004 21:35'!
classSymbol
	^ className! !

!OBMethodCreatingNode methodsFor: 'nodes' stamp: 'cwp 2/7/2004 15:20'!
classCategoryNode
	^ OBClassCategoryNode on: (context categoryOf: className) inContext: context! !

!OBMethodCreatingNode methodsFor: 'confirming' stamp: 'cwp 3/4/2004 01:06'!
classIsMeta
	^ className endsWith: ' class'! !

!OBMethodCreatingNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:08'!
isDescendantOfClassCat: aClassCategoryNode
	^ (context categoryOf: className) = aClassCategoryNode name! !

!OBMethodCreatingNode methodsFor: 'confirming' stamp: 'cwp 3/4/2004 01:29'!
confirmScarySelector: aSelector
	| prompt |
	prompt _ aSelector, ' is used in the existing class system.
Overriding it could cause serious problems.
Is this really what you want to do?'.

	^ OBConfirmationRequest
		prompt: prompt
		confirm: 'Override'! !

!OBMethodCreatingNode methodsFor: 'actions' stamp: 'cwp 3/11/2004 21:59'!
chaseIvars
	OBBrowser 
		browse: #ChaseInstanceVariables 
		fromRoot: self classNode
		panes: 3! !

!OBMethodCreatingNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:30'!
isDescendantOfClass: aClassNode
	^ className = aClassNode symbol! !

OBMethodCreatingNode subclass: #OBMethodCategoryNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:01'!
isDescendantOfPackage: aPackageNode
	^ (aPackageNode context classExists: className)
		and: [(aPackageNode context categoriesForClass: className) includes: name]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: 'cwp 3/1/2004 23:23'!
accept: aText notifying: aController
	| selector |
	selector _ self compileMethod: aText notifying: aController.
	selector ifNil: [^ false].
	(self methodNodeFor: selector) signalSelection! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:03'!
isDescendantOfMethodCat: other
	^ (other classSymbol = className) and: [other name = name]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: 'cwp 12/10/2003 23:10'!
name
	^ name! !

!OBMethodCategoryNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:31'!
dropSelector
	^ #dropOnMethodCategory:! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: 'cwp 1/31/2004 09:52'!
category
	^ name! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 2/7/2004 15:52'!
remove
	context removeCategory: name fromClass: className.
	OBNodeDeleted signal! !

!OBMethodCategoryNode methodsFor: 'children' stamp: 'cwp 12/17/2003 18:25'!
methods
	^ self methodReferences collect: [:ref | OBMethodNode on: ref inContext: context]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:28'!
textSelection
	^ 1 to: self text size! !

!OBMethodCategoryNode methodsFor: 'navigation' stamp: 'cwp 2/7/2004 16:13'!
methodCategoryNode
	^ self! !

!OBMethodCategoryNode class methodsFor: 'instance creation' stamp: 'cwp 2/7/2004 15:16'!
on: aString inClass: className inContext: aCodeContext
	^ self new setName: aString class: className context: aCodeContext! !

!OBMethodCategoryNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:28'!
ancestrySelector
	^ #isDescendantOfMethodCat:! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 3/4/2004 23:31'!
rename
	| category |
	category _ OBTextRequest
				prompt: 'Please type new category name' 
				template: name.
	category ifNotNil:	[context renameCategory: name inClass: className toBe: category.
						name _ category.
						self signalSelection.]! !

!OBMethodCategoryNode methodsFor: 'initializing' stamp: 'cwp 2/7/2004 15:28'!
setName: aString class: aSymbol context: aCodeContext
	name _ aString.
	className _ aSymbol.
	context _ aCodeContext! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 3/11/2004 22:22'!
actions
	^ Array
		with: (self action: #browse buttonLabel: 'browse' menuLabel: 'view in standard browser')
		with: (self action: #chaseIvars buttonLabel: 'ivars' menuLabel: 'chase instance variables')
		with: (self action: #remove withLabel: 'remove')
		with: (self action: #rename withLabel: 'rename...')! !

!OBMethodCategoryNode class methodsFor: 'actions' stamp: 'cwp 2/29/2004 14:12'!
createInClass: aClassNode
	| catName |
	catName _ OBTextRequest 
					prompt: 'Please type new category name' 
					template: ''.
	catName ifNotNil:	[aClassNode context addCategory: catName toClass: aClassNode symbol.
						(OBMethodCategoryNode 
							on: catName 
							inClass: aClassNode symbol
							inContext: aClassNode context) signalSelection]! !

!OBMethodCategoryNode methodsFor: 'public' stamp: 'cwp 1/30/2004 23:45'!
text
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!OBMethodCategoryNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 15:27'!
methodReferences
	^ context methodsInCategory: name ofClass: className! !

!OBMethodCategoryNode class methodsFor: 'actions' stamp: 'cwp 2/29/2004 18:21'!
actionsForParent: aNode
	^ Array with: (OBAction
					label: 'create category...'
					receiver: self
					selector: #createInClass:
					arguments: (Array with: aNode))
			with: (OBAction
					label: 'alphabetize categories'
					receiver: self
					selector: #alphabetizeCategoriesInClass:
					arguments: (Array with: aNode))! !

!OBMethodCategoryNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 00:47'!
browse
	Browser fullOnClass: self theClass! !

!OBMethodCategoryNode class methodsFor: 'actions' stamp: 'cwp 2/29/2004 18:17'!
alphabetizeCategoriesInClass: aClassNode
	aClassNode context sortCategoriesInClass: aClassNode symbol.
	OBListChanged signal! !

OBMethodCategoryNode subclass: #OBAllMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBAllMethodCategoryNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 20:39'!
on: className inContext: aCodeContext
	^ self on: '-- all --' inClass: className inContext: aCodeContext! !

!OBAllMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 20:38'!
methodReferences
	^ context methodsForClass: className! !

!OBMethodCreatingNode methodsFor: 'method creation' stamp: 'cwp 3/4/2004 00:55'!
compileMethod: aText notifying: aController
	^ (self confirmMethod: aText)
		ifTrue: [self theClass compile: aText classified: self category notifying: aController]
		ifFalse: [nil]! !

OBMethodCreatingNode subclass: #OBMethodNode
	instanceVariableNames: 'selector isNaked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMethodNode methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:38'!
dropOnMethodCategory: aNode

	"We don't support dropping on another class yet."
	aNode classSymbol = className ifFalse: [^ false].
	
	context setCategoryOf: selector inClass: className toBe: aNode name.
	OBListChanged signal.
	
	^ true! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 3/4/2004 02:41'!
remove
	| senders choice |
	senders _ context allSendersOf: selector.
	senders isEmpty ifTrue: [^ self doRemove].
	choice _ OBChoiceRequest
				prompt: 'This message has ', senders size asString, ' senders.'
				labels: #('Remove it' 
						'Remove, then browse senders' 
						'Don''t remove, but show me those senders' 
						'Forget it -- do nothing -- sorry I asked')
				values: #(doRemove removeAndBrowse simpleBrowseSenders nil).
	choice ifNotNil: [^ self perform: choice]
! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 3/11/2004 21:33'!
implementorsOf: aSelector
	^ (context allImplementorsOf: aSelector) 
			collect: [:ref | OBMethodNode nakedOn: ref inContext: context]! !

!OBMethodNode methodsFor: 'nodes' stamp: 'cwp 2/29/2004 20:55'!
messageNode
	^ OBMessageNode fromMethodNode: self! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 3/11/2004 20:53'!
beNaked
	isNaked _ true! !

OBMethodNode subclass: #OBMessageNode
	instanceVariableNames: 'message range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMessageNode methodsFor: 'navigating' stamp: 'cwp 3/11/2004 21:33'!
implementors
	^ self implementorsOf: message! !

!OBMessageNode methodsFor: 'public' stamp: 'cwp 3/11/2004 21:25'!
name
	^ self isNaked
		ifTrue: [className,'>>',selector]
		ifFalse: [message asString]! !

!OBMessageNode methodsFor: 'initializing' stamp: 'cwp 3/11/2004 21:12'!
setRange: anInterval message: aSelector selector: aSelector2 class: aSymbol context: aContext
	range _ anInterval.
	message _ aSelector.
	selector _ aSelector2.
	className _ aSymbol.
	context _ aContext! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 3/11/2004 21:14'!
fromMethodNode: aMethodNode
	^ self new 
		setRange: (1 to: 0)
		message: aMethodNode selector
		selector: aMethodNode selector
		class: aMethodNode classSymbol
		context: aMethodNode context! !

!OBMessageNode methodsFor: 'initializing' stamp: 'cwp 3/11/2004 21:10'!
setRange
	| methodNode assoc |
	methodNode _ Parser new 
					parse: (context sourceForMethod: selector ofClass: className)
					class: (context classNamed: className).
	assoc _ (methodNode encoder rawSourceRanges) 
			associations detect: [:ea | ea key isMessage: message receiver: nil arguments: nil]
						ifNone: [nil -> (1 to: 0)].
						"Some messages are generated by the compiler."
	^ range _ assoc value! !

!OBMessageNode methodsFor: 'accessing' stamp: 'cwp 2/29/2004 21:57'!
range
	^ range ifNil: [self setRange]! !

!OBMessageNode methodsFor: 'navigating' stamp: 'cwp 3/11/2004 22:11'!
sendersOfMessage
	^ self sendersOf: message! !

!OBMessageNode methodsFor: 'initializing' stamp: 'cwp 3/11/2004 21:16'!
setMessage: aSelector selector: aSelector2 class: aSymbol context: aContext
	self setRange: nil message: aSelector selector: aSelector2 class: aSymbol context: aContext
! !

!OBMessageNode methodsFor: 'public' stamp: 'cwp 2/29/2004 21:57'!
textSelection
	^ self range! !

!OBMessageNode methodsFor: 'public' stamp: 'cwp 3/11/2004 21:17'!
text
	^ context sourceForMethod: selector ofClass: className! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 3/11/2004 22:10'!
on: aSelector inMethodReference: ref inContext: aContext
	| className |
	className _ ref classIsMeta
					ifTrue: [(ref classSymbol, ' class') asSymbol]
					ifFalse: [ref classSymbol].
	^ (self on: aSelector inMethod: ref methodSymbol inClass: className inContext: aContext)
		beNaked! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 3/11/2004 21:16'!
on: aSelector inMethod: aSelector2 inClass: className inContext: aContext
	^ self new setMessage: aSelector selector: aSelector2 class: className context: aContext! !

!OBMessageNode class methodsFor: 'instance creation' stamp: 'cwp 3/11/2004 21:42'!
on: aSelector inMethodNode: aNode
	^ self 
		on: aSelector 
		inMethod: aNode selector 
		inClass: aNode classSymbol 
		inContext: aNode context! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 3/11/2004 21:33'!
implementors
	^ self implementorsOf: selector! !

!OBMethodNode methodsFor: 'initializing' stamp: 'cwp 2/7/2004 16:02'!
setSelector: aSelector class: aSymbol context: aCodeContext
	className _ aSymbol.
	selector _ aSelector.
	context _ aCodeContext! !

!OBMethodNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 16:03'!
on: aSelector inClass: aSymbol inContext: aCodeContext
	^ self new setSelector: aSelector class: aSymbol context: aCodeContext! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 3/4/2004 02:41'!
removeAndBrowse
	self simpleBrowseSenders.
	context removeMethod: selector fromClass: className.
	OBNodeDeleted signal! !

!OBMethodNode methodsFor: 'public' stamp: 'cwp 3/3/2004 23:08'!
accept: aText notifying: aController
	| newSelector |
	newSelector _ self compileMethod: aText notifying: aController.
	newSelector ifNil: [^ false].
	newSelector = selector ifFalse: [(self methodNodeFor: newSelector) signalSelection].
	^ true! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 2/26/2004 20:50'!
browseImplementors
	OBBrowser browse: #Implementors fromRoot: self selectMessage! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 23:11'!
browseVersions
	OBBrowser browse: #Versions fromRoot: self! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 3/11/2004 21:36'!
simpleBrowseSenders
	OBBrowser browse: #Senders fromRoot: self.! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 2/26/2004 20:56'!
chaseImplementors
	OBBrowser 
		browse: #ChaseImplementors 
		fromRoot: self
		panes: 4! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:16'!
isDescendantOfPackage: aPackageNode
	^ (aPackageNode context classExists: className)
		and: [((aPackageNode context methodsForClass: className) 
				collect: [:ea | ea methodSymbol]) includes: selector]! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 3/14/2004 23:31'!
versions
	^ (OBMethodVersion scan: context sourceFiles from: (context sourcePointerFor: self reference))
		collect: [:ea | OBMethodVersionNode on: ea inContext: context]! !

!OBMethodNode methodsFor: 'public' stamp: 'cwp 2/27/2004 00:50'!
text
	^ context sourceForMethod: selector ofClass: className! !

!OBMethodNode methodsFor: 'comparing' stamp: 'cwp 2/8/2004 14:26'!
= other
	^ (other isKindOf: OBMethodNode)
		and: [self className = self className] 
		and: [self selector = other selector]! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 2/26/2004 20:56'!
chaseSenders
	OBBrowser 
		browse: #ChaseSenders 
		fromRoot: self
		panes: 4! !

!OBMethodNode methodsFor: 'initializing' stamp: 'cwp 2/8/2004 13:12'!
setReference: aMethodReference context: aCodeContext
	className _ aMethodReference classSymbol.
	aMethodReference classIsMeta ifTrue: [className _ (className, ' class') asSymbol].
	self 
		setSelector: aMethodReference methodSymbol
		class: className
		context: aCodeContext! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:06'!
ancestrySelector
	^ #isDescendantOfMethod: ! !

!OBMethodNode methodsFor: 'public' stamp: 'cwp 3/11/2004 21:03'!
name
	^ self isNaked
		ifTrue: [className, '>>', selector]
		ifFalse: [selector]! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 3/11/2004 21:32'!
senders
	^ self sendersOf: selector! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 3/11/2004 21:43'!
selectMessage
	| messages sel |
	messages _ (context messagesSentBy: selector inClass: className) copyWithFirst: selector.
	sel _ (OBChoiceRequest labels: messages).
	^ OBMessageNode on: sel inMethodNode: self! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 02:07'!
isDescendantOfMethod: other
	^ (other classSymbol = className) and: [other selector = selector]! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 15:21'!
category
	^ context methodCategoryOf: self reference! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 2/8/2004 14:25'!
selector
	^ selector! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 2/29/2004 20:55'!
selectorAndMessages
	^ self messages copyWithFirst: self messageNode! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 3/10/2004 00:48'!
browse
	Browser fullOnClass: self theClass selector: selector! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 23:45'!
actions
	^ 	{self action: #browse buttonLabel: 'browse' menuLabel: 'view in standard browser'.
		self action: #chaseSenders buttonLabel: 'senders' menuLabel: 'chase senders'.
		self action: #chaseImplementors buttonLabel: 'impl' menuLabel: 'chase implementors'.
		self action: #browseVersions buttonLabel: 'versions' menuLabel: 'browse versions'.
		self action: #chaseIvars buttonLabel: 'ivars' menuLabel: 'chase instance variables'.
		self action: #browseSenders withLabel: 'browse senders...'.
		self action: #browseImplementors withLabel: 'browse implementors...'.
		self action: #remove withLabel: 'remove'}! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 3/4/2004 02:26'!
doRemove
	context removeMethod: selector fromClass: className.
	OBNodeDeleted signal! !

!OBMethodNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/10/2003 23:51'!
on: aMethodReference inContext: aCodeContext
	^ self new setReference: aMethodReference context: aCodeContext! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 3/11/2004 20:54'!
isNaked
	^ isNaked ifNil: [isNaked _ false]! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 2/29/2004 15:57'!
classSymbol
	^ className! !

!OBMethodNode methodsFor: 'actions' stamp: 'cwp 2/26/2004 20:52'!
browseSenders
	OBBrowser browse: #Senders fromRoot: self selectMessage! !

!OBMethodNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 15:20'!
reference
	^ self referenceForMethod: selector ofClass: className! !

!OBMethodNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2004 20:55'!
nakedOn: aMethodReference inContext: aCodeContext
	^ (self on: aMethodReference inContext: aCodeContext) beNaked! !

!OBMethodNode methodsFor: 'nodes' stamp: 'cwp 2/7/2004 15:21'!
methodCategoryNode
	^ OBMethodCategoryNode 
		on: self category
		inClass: className
		inContext: context! !

!OBMethodNode methodsFor: 'private' stamp: 'cwp 3/11/2004 21:53'!
sendersOf: aSelector
	^ (context allSendersOf: aSelector) 
			collect: [:ref | OBMessageNode on: aSelector inMethodReference: ref inContext: context]! !

!OBMethodNode methodsFor: 'ancestry' stamp: 'cwp 3/4/2004 23:52'!
isDescendantOfMethodCat: aNode
	^ (aNode classSymbol = className) 
		and: [(context methodCategoryOf: self reference) = aNode name]! !

!OBMethodNode methodsFor: 'navigating' stamp: 'cwp 2/29/2004 19:18'!
messages
	^ (context messagesSentBy: selector inClass: className)
		collect: [:sel | OBMessageNode on: sel inMethod: selector inClass: className inContext: context]! !

!OBMethodCreatingNode methodsFor: 'confirming' stamp: 'cwp 3/4/2004 01:01'!
confirmMethod: aText
	| sel |
	sel _ Parser new parseSelector: aText.
	^ (self classIsMeta and: [(self selectorAlreadyDefined: sel) not] and: [Metaclass isScarySelector: sel])
		ifTrue: [self confirmScarySelector: sel]
		ifFalse: [^ true]! !

OBMethodCreatingNode subclass: #OBMethodVersionNode
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBMethodVersionNode methodsFor: 'displaying' stamp: 'cwp 3/14/2004 23:09'!
text
	^ version source! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 23:51'!
browse
	| class method |
	class _ self theClass.
	method _ (class compiledMethodAt: version selector).
	VersionsBrowser
		browseVersionsOf: method
		class: class
		meta: class isMeta
		category: self category
		selector: version selector! !

!OBMethodVersionNode methodsFor: 'initializing' stamp: 'cwp 3/14/2004 23:33'!
setVersion: aMethodVersion context: aCodeContext
	version _ aMethodVersion.
	className _ version classSymbol.
	context _ aCodeContext! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 3/14/2004 23:36'!
reference
	^ self referenceForMethod: version selector ofClass: self classSymbol.! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 23:55'!
actions
	^ {self action: #browse buttonLabel: 'browse' menuLabel: 'browse in standard browser'.
	   self action: #revert buttonLabel: 'revert' menuLabel: 'revert to selected version'}! !

!OBMethodVersionNode methodsFor: 'compiling' stamp: 'cwp 3/14/2004 23:26'!
accept: aText notifying: aController
	| newSelector |
	newSelector _ self compileMethod: aText notifying: aController.
	newSelector ifNil: [^ false].
	newSelector = version selector ifFalse: [(self methodNodeFor: newSelector) signalSelection].
	^ true! !

!OBMethodVersionNode methodsFor: 'accessing' stamp: 'cwp 3/14/2004 23:34'!
category
	^ context methodCategoryOf: self reference! !

!OBMethodVersionNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 00:05'!
revert
	version fileIn! !

!OBMethodVersionNode class methodsFor: 'instance creation' stamp: 'cwp 3/14/2004 23:30'!
on: aMethodVersion inContext: aCodeContext
	^ self new setVersion: aMethodVersion context: aCodeContext! !

!OBMethodVersionNode methodsFor: 'displaying' stamp: 'cwp 3/14/2004 23:05'!
name
	^ version stamp, ' ', version classSymbol, '>>', version selector! !

!OBMethodCreatingNode methodsFor: 'nodes' stamp: 'cwp 2/7/2004 16:12'!
classNode
	^ OBClassNode on: className inContext: context! !

OBCodeNode subclass: #OBSystemNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBSystemNode methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 20:22'!
packages
	^ context packages collect: [:aPackage | OBPackageNode on: aPackage]! !

!OBSystemNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/10/2004 00:26'!
forImage
	^ self on: OBImageContext new! !

!OBSystemNode methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 15:58'!
context
	^ context! !

!OBSystemNode methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 20:26'!
categories
	^ context categories collect: [:cat | OBClassCategoryNode on: cat inContext: context]! !

!OBSystemNode methodsFor: 'as yet unclassified' stamp: 'cwp 1/7/2004 06:21'!
name 
	^ 'Squeak'! !

!OBSystemNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 20:19'!
on: aCodeContext
	^ self new context: aCodeContext! !

!OBSystemNode methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 20:18'!
context: aCodeContext
	context _ aCodeContext! !

OBCodeNode subclass: #OBPackageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBPackageNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:33'!
ancestrySelector
	^ #isDescendantOfPackage:! !

!OBPackageNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:50'!
context: aCodeContext
	context _ aCodeContext! !

!OBPackageNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:53'!
package
	^ context package! !

!OBPackageNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 13:58'!
on: aPackage
	^ self new context: (OBPackageContext on: aPackage)! !

!OBPackageNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:52'!
categories
	^ context categories collect: [:cat | OBClassCategoryNode on: cat inContext: context]! !

!OBPackageNode methodsFor: 'actions' stamp: 'cwp 3/4/2004 23:39'!
browse
	OBBrowser browsePackage: self name! !

!OBPackageNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:52'!
classes
	^ context classes collect: [:class | OBClassNode on: class inContext: context]! !

!OBPackageNode methodsFor: 'public' stamp: 'cwp 2/7/2004 13:49'!
name
	^ context package packageName! !

!OBPackageNode methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:49'!
context
	^ context! !

!OBPackageNode methodsFor: 'actions' stamp: 'cwp 3/4/2004 23:37'!
actions
	^ Array with: (self action: #browse withLabel: 'browse')! !

OBCodeNode subclass: #OBClassCommentNode
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBClassCommentNode methodsFor: 'initializing' stamp: 'cwp 2/25/2004 01:08'!
setClassName: aSymbol context: aContext
	className _ aSymbol.
	context _ aContext! !

!OBClassCommentNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/25/2004 01:08'!
on: className inContext: aContext
	^ self new setClassName: className context: aContext! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 2/25/2004 01:19'!
name
	^ className asString! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:52'!
text: aText
	context setCommentForClass: className to: aText.
	^ true! !

!OBClassCommentNode methodsFor: 'public' stamp: 'cwp 2/25/2004 01:20'!
text
	^ context commentForClass: className! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:29'!
dropSelector
	"Override in subclasses"
	
	^ #dropOnNode: ! !

OBNode subclass: #OBDirectoryNode
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBDirectoryNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:51'!
name
	^ directory localName! !

!OBDirectoryNode class methodsFor: 'instance creation' stamp: 'cwp 1/31/2004 10:17'!
default
	^ self on: (FileDirectory default)! !

!OBDirectoryNode class methodsFor: 'instance creation' stamp: 'cwp 3/10/2004 23:27'!
root
	^ self on: (FileDirectory root)! !

!OBDirectoryNode class methodsFor: 'instance creation' stamp: 'cwp 11/28/2003 21:57'!
on: aFileDirectory
	^ self new directory: aFileDirectory! !

!OBDirectoryNode methodsFor: 'accessing' stamp: 'cwp 1/31/2004 10:15'!
path
	^ directory fullName! !

!OBDirectoryNode methodsFor: 'private' stamp: 'cwp 12/15/2003 21:37'!
nodeForEntry: anEntry
	^ (anEntry at: 4)
		ifTrue: [OBDirectoryNode on: (directory directoryNamed: (anEntry at: 1))]
		ifFalse: [OBFileNode on: (directory fullNameFor: (anEntry at: 1))]! !

!OBDirectoryNode methodsFor: 'private' stamp: 'cwp 12/6/2003 17:43'!
textForDirectory: aFilename
	| entry |
	entry _ directory entryAt: aFilename.
	^ String streamContents: [:stream |
			stream 
				nextPutAll: 'Directory: '; 
				nextPutAll: (directory fullNameFor: aFilename); cr;
				nextPutAll: 'Created: '; nextPutAll: (entry at: 2) asString; cr;
				nextPutAll: 'Modified: '; nextPutAll: (entry at: 3) asString; cr;
				nextPutAll: 'Size: '; nextPutAll: (entry at: 5) asString; cr;
				cr; nextPutAll: '----------'; cr.

			directory entries do: [:e | self displayEntry: e on: stream]

	]! !

!OBDirectoryNode methodsFor: 'private' stamp: 'cwp 11/28/2003 21:57'!
directory: aFileDirectory
	directory _ aFileDirectory! !

!OBDirectoryNode methodsFor: 'private' stamp: 'cwp 1/31/2004 15:15'!
shouldDisplay: name
	^ name first ~= $.! !

!OBDirectoryNode methodsFor: 'private' stamp: 'cwp 12/6/2003 18:01'!
displayEntry: entry on: aStream
	aStream nextPutAll: (entry at: 1).
	aStream nextPutAll: ' - '.
	aStream nextPutAll: (entry at: 5) asString, ' bytes'.
	aStream cr.! !

!OBDirectoryNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:59'!
hash
	^ self path hash! !

!OBDirectoryNode methodsFor: 'children' stamp: 'cwp 1/31/2004 15:14'!
files
	^ directory fileNames 
		select: [:name | self shouldDisplay: name]
		thenCollect: [:name | OBFileNode on: (directory fullNameFor: name)]
	! !

!OBDirectoryNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:59'!
= other
	^ self path = other path! !

!OBDirectoryNode methodsFor: 'public' stamp: 'cwp 1/31/2004 10:14'!
isAncestorOf: aNode
	^ aNode path beginsWith: self path! !

!OBDirectoryNode methodsFor: 'children' stamp: 'cwp 1/31/2004 15:14'!
directories
	^ directory directoryNames 
		select: [:name | self shouldDisplay: name]
		thenCollect: [:name | OBDirectoryNode on: (directory directoryNamed: name)]
	! !

!OBDirectoryNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:47'!
text
	| entry |
	entry _ directory directoryEntry.
	^ String streamContents: [:stream |
			stream 
				nextPutAll: 'Directory: '; 
				nextPutAll: (directory fullNameFor: (entry at: 1)); cr;
				nextPutAll: 'Created: '; nextPutAll: (entry at: 2) asString; cr;
				nextPutAll: 'Modified: '; nextPutAll: (entry at: 3) asString; cr;
				nextPutAll: 'Size: '; nextPutAll: (entry at: 5) asString; cr;
				cr; nextPutAll: '----------'; cr.

			directory entries do: [:e | self displayEntry: e on: stream]

	]! !

Object subclass: #OBFakeClassC
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Fakes'!

Object subclass: #OBFakeClassB
	instanceVariableNames: 'ivar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Fakes'!

!OBFakeClassB methodsFor: 'accessing' stamp: ''!
accessesIvar
	^ ivar! !

!OBFakeClassB methodsFor: 'senders' stamp: ''!
sendsSendsAccessesIvar
	self sendsAccessesIvar! !

!OBFakeClassB class methodsFor: 'testing' stamp: 'cwp 2/29/2004 13:11'!
acceptsLoggingOfCompilation
	^ false! !

!OBFakeClassB methodsFor: 'senders' stamp: ''!
sendsFirst
	self first! !

!OBFakeClassB methodsFor: 'senders' stamp: ''!
sendsAccessesIvar
	self accessesIvar! !

!OBFakeClassB methodsFor: 'accessing' stamp: ''!
foo
	^ self! !

Object subclass: #OBMetaNode
	instanceVariableNames: 'name filterClass columnClass edges actors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaNode commentStamp: 'cwp 3/5/2004 11:19' prior: 0!
A MetaNode represents a hypothetical node in the browser's domain graph.

iVars:

filterClass	- the class used to filter this hypothetical node's children
columnClass	- the class of column used to display this node's children
edges		- messages that could be sent to the node to obtain children
actors		- providers of functionality for manipulating the node!

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:44'!
children
	^ edges collect: [:edge | edge metaNode]! !

!OBMetaNode methodsFor: 'initializing' stamp: 'cwp 3/3/2004 23:04'!
initialize
	edges _ OrderedCollection new.
	actors _ OrderedCollection new.! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:26'!
filter
	filterClass ifNil: [filterClass _ OBFilter].
	^ filterClass forMetaNode: self! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:22'!
nodesForParent: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'columns' stamp: 'avi 3/4/2004 03:11'!
columnClass: aClass
	columnClass _ aClass! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/6/2004 21:06'!
named: aString
	^ self new name: aString! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:35'!
childAt: aSelector put: aMetaNode
	edges add: (OBMetaEdge selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'columns' stamp: 'avi 3/4/2004 03:12'!
columnInBrowser: aBrowser node: aNode
	columnClass ifNil: [columnClass _ OBColumn].
	^ columnClass
		inBrowser: aBrowser
		metaNode: self
		node: aNode! !

!OBMetaNode methodsFor: 'converting' stamp: 'cwp 3/2/2004 22:17'!
asNode
	^ OBMetaNodeNode on: self! !

!OBMetaNode methodsFor: 'metagraph' stamp: 'cwp 2/9/2004 21:12'!
edges
	^ edges! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 2/5/2004 20:15'!
actionsForParent: aNode
	^ actors gather: [:actor | actor actionsForParent: aNode]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 1/23/2004 23:25'!
actionsForNode: aNode
	^ actors gather: [:actor | actor actionsForNode: aNode]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 3/3/2004 22:48'!
addActor: anActor
	actors add: anActor.! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:02'!
filterClass: aClass
	filterClass _ aClass! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:38'!
hasChildren
	^ edges isEmpty not! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 23:08'!
new
	^ self basicNew initialize! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:00'!
metaNode
	^ self! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:05'!
name: aString
	name _ aString! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/9/2004 20:28'!
childAt: aSelector labeled: aString put: aMetaNode
	edges add: (OBMetaEdge label: aString selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:54'!
name
	^ name ifNil: ['MetaNode']! !

Object subclass: #OBNotifier
	instanceVariableNames: ''
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBNotifier commentStamp: 'cwp 3/5/2004 14:30' prior: 0!
This class refreshes in response to SystemChangeNotifications. There is only one instance.!

!OBNotifier class methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 23:58'!
createInstance
	^ Instance _ super new.! !

!OBNotifier class methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 23:51'!
new
	self error: 'Only one instance should be exist.'! !

!OBNotifier class methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 23:53'!
initialize
	^ self uniqueInstance register! !

!OBNotifier methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 00:45'!
event: anEvent
	[OBBrowser allInstancesDo: [:ea | ea refresh]] fork! !

!OBNotifier methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 23:56'!
register
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:! !

!OBNotifier class methodsFor: 'as yet unclassified' stamp: 'cwp 3/1/2004 23:51'!
uniqueInstance
	^ Instance ifNil: [self createInstance]! !

Object subclass: #OBImageContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBImageContext commentStamp: 'cwp 3/5/2004 12:38' prior: 0!
The nodes representing Smalltalk program elements operate within a context which defines the scope of the code visible in the browser. OBImageContext operates on  all the code in the image.!

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/7/2004 14:48'!
removeCategory: aString fromClass: className 
	(self classNamed: className) organization removeCategory: aString! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/3/2004 22:42'!
removeCategory: aString
	SystemOrganization removeSystemCategory: aString! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 2/7/2004 14:39'!
renameClass: oldName toBe: newName
	Smalltalk renameClassNamed: oldName as: newName! !

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/7/2004 15:32'!
addCategory: aString toClass: className 
	(self classNamed: className asSymbol) organization addCategory: aString! !

!OBImageContext methodsFor: 'private' stamp: 'cwp 1/31/2004 13:29'!
longNameOf: classCategory
	^ classCategory! !

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/13/2004 00:03'!
categoriesForClass: className
	^ (self classNamed: className) organization categories! !

!OBImageContext methodsFor: 'navigation' stamp: 'cwp 3/3/2004 22:44'!
allImplementorsOf: aSelector
	^ (self systemNavigation allImplementorsOf: aSelector) asArray sort! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/9/2004 20:08'!
categoryOf: className
	^ (self classNamed: className) theNonMetaClass category! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 2/7/2004 15:26'!
methodsInCategory: catName ofClass: className
	^ ((self classNamed: className)
		 organization listAtCategoryNamed: catName)
		 	collect: [:sel | self referenceForMethod: sel ofClass: className]! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/2/2004 22:54'!
renameCategory: oldName toBe: newName
	SystemOrganization renameCategory: oldName toBe: newName! !

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/29/2004 18:08'!
sortCategoriesInClass: className
	(self classNamed: className) organization sortCategories! !

OBImageContext subclass: #OBPackageContext
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBPackageContext commentStamp: 'cwp 3/5/2004 14:31' prior: 0!
This is a code context that is limited to a single package. See the comment in OBImageContext.!

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 13:44'!
renameCategory: oldName toBe: newName
	super 
		renameCategory: (package longNameOf: oldName)
		toBe: (package longNameOf: newName)! !

!OBPackageContext methodsFor: 'classes' stamp: 'cwp 3/3/2004 02:01'!
classExists: className
	^ (super classExists: className) and: [package includesClass: (self classNamed: className)]! !

!OBPackageContext methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:08'!
package: aPackage
	package _ aPackage! !

!OBPackageContext methodsFor: 'navigation' stamp: 'cwp 2/29/2004 23:54'!
allImplementorsOf: aSelector
	^ (super allImplementorsOf: aSelector)
		select: [:ea | package includesMethodReference: ea]! !

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/28/2004 21:17'!
setCategoryOf: className to: catName
	^ super setCategoryOf: className to: (package longNameOf: catName).
! !

!OBPackageContext methodsFor: 'accessing' stamp: 'cwp 2/7/2004 13:09'!
package
	^ package! !

!OBPackageContext methodsFor: 'methods' stamp: 'cwp 3/14/2004 13:34'!
methodsForClass: className
	| class |
	class _ self classNamed: className.
	^ ((package coreMethodsForClass: class), (package extensionMethodsForClass: class))
		asArray sort! !

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 13:45'!
removeCategory: aString
	super removeCategory: (package longNameOf: aString)! !

!OBPackageContext methodsFor: 'private' stamp: 'cwp 2/7/2004 14:08'!
longNameOf: aString
	^ package longNameOf: aString! !

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 14:49'!
categoryOf: className
	^ package shortNameOf: (super categoryOf: className)! !

!OBPackageContext class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 13:07'!
named: aString
	^ self on: (PackageInfo named: aString)! !

!OBPackageContext methodsFor: 'classes' stamp: 'cwp 2/7/2004 13:11'!
classesInCategory: aString
	^ super classesInCategory: (package longNameOf: aString)! !

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 13:33'!
categories
	^ package systemCategories collect: [:cat | package shortNameOf: cat]! !

!OBPackageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 13:40'!
addCategory: aString
	super addCategory: (package longNameOf: aString)! !

!OBPackageContext class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 13:07'!
on: aPackageInfo
	^ self new package: aPackageInfo! !

!OBPackageContext methodsFor: 'navigation' stamp: 'cwp 2/29/2004 23:54'!
allSendersOf: aSelector
	^ (super allSendersOf: aSelector)
		select: [:ea | package includesMethodReference: ea]! !

!OBPackageContext methodsFor: 'methods' stamp: 'cwp 3/10/2004 19:00'!
accessesToVariable: instVarName ofClass: className
	^ (super accessesToVariable: instVarName ofClass: className)
		 select: [:ea | package includesMethodReference: ea]! !

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/7/2004 14:48'!
renameCategory: oldName inClass: className toBe: newName 
	(self classNamed: className) organization renameCategory: oldName toBe: newName! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/7/2004 14:44'!
classesInCategory: aCategory
	^ SystemOrganization listAtCategoryNamed: aCategory asSymbol! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 2/25/2004 01:22'!
commentForClass: className
	^ (self classNamed: className) comment! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 3/10/2004 19:20'!
instanceVariablesForClass: className
	^ (self classNamed: className) allInstVarNames! !

!OBImageContext methodsFor: 'method categories' stamp: 'cwp 2/29/2004 22:56'!
setCategoryOf: aSelector inClass: className toBe: catName
	(self classNamed: className) organization classify: aSelector under: catName! !

!OBImageContext methodsFor: 'private' stamp: 'cwp 3/14/2004 23:32'!
classNamed: aSymbol
	^ (aSymbol endsWith: ' class')
		ifTrue: [self metaclassNamed: (aSymbol allButLast: 6) asSymbol]
		ifFalse: [Smalltalk at: aSymbol]! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 3/14/2004 13:33'!
methodsForClass: className
	^ ((self classNamed: className)
		 selectors asArray collect: [:sel | self referenceForMethod: sel ofClass: className])
			sort! !

!OBImageContext methodsFor: 'private' stamp: 'cwp 2/7/2004 21:12'!
metaclassNamed: aSymbol
	^ (self classNamed: aSymbol) class! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 2/27/2004 01:11'!
sourceForMethod: aSelector ofClass: className
	| class |
	class _ self classNamed: className.
	^ (class sourceCodeAt: aSelector)
			asText makeSelectorBoldIn: class! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 2/7/2004 14:36'!
removeClass: className
	(self classNamed: className) removeFromSystem.! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 2/7/2004 19:04'!
removeMethod: aSelector fromClass: className
	(self classNamed: className) removeSelector: aSelector! !

!OBImageContext methodsFor: 'source' stamp: 'cwp 3/14/2004 23:15'!
sourceFiles
	^ SourceFiles! !

!OBImageContext methodsFor: 'navigation' stamp: 'cwp 3/3/2004 22:44'!
allSendersOf: aSelector
	^ (self systemNavigation allCallsOn: aSelector) asArray sort! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 12/17/2003 20:40'!
categories
	^ SystemOrganization categories! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 2/29/2004 13:31'!
setCommentForClass: className to: aText
	^ (self classNamed: className) comment: aText stamp: Utilities changeStamp! !

!OBImageContext methodsFor: 'packages' stamp: 'cwp 12/17/2003 20:23'!
packages
	^ PackageInfo allPackages! !

!OBImageContext methodsFor: 'source' stamp: 'cwp 3/14/2004 23:19'!
sourcePointerFor: aMethodReference
	| class selector |
	class _ aMethodReference actualClass.
	selector _ aMethodReference methodSymbol.
	^ (class compiledMethodAt: selector) sourcePointer! !

!OBImageContext methodsFor: 'classes' stamp: 'cwp 2/28/2004 11:46'!
classExists: className
	| name |
	name _ (className endsWith: ' class') ifTrue: [className allButLast: 6] ifFalse: [className].
	^ Smalltalk hasClassNamed: name! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/25/2004 22:10'!
setCategoryOf: className to: catName
	(self classNamed: className) theNonMetaClass category: catName! !

!OBImageContext methodsFor: 'private' stamp: 'cwp 2/7/2004 15:02'!
referenceForMethod: selector ofClass: className
	^ MethodReference new 
		setClassSymbol: className
		classIsMeta: false
		methodSymbol: selector
		stringVersion: className, '>>', selector! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 3/10/2004 18:48'!
accessesToVariable: instVarName ofClass: className
	| accessors |
	accessors _ OrderedCollection new.
	(self classNamed: className) withAllSubAndSuperclassesDo: [:class |
		(class whichSelectorsAccess: instVarName) 
			do: [:ea | ea = #DoIt ifFalse: [accessors add: (self referenceForMethod: ea ofClass: class name)]]].
	^ accessors! !

!OBImageContext methodsFor: 'class categories' stamp: 'cwp 2/2/2004 21:17'!
addCategory: aString
	SystemOrganization addCategory: aString! !

!OBImageContext methodsFor: 'navigation' stamp: 'cwp 3/3/2004 22:45'!
messagesSentBy: aSelector inClass: aSymbol
	^ (((self classNamed: aSymbol) compiledMethodAt: aSelector) messages) asArray sort! !

!OBImageContext methodsFor: 'methods' stamp: 'cwp 1/31/2004 13:40'!
methodCategoryOf: aMethodReference
	^ aMethodReference actualClass organization 
		categoryOfElement: aMethodReference methodSymbol! !

Object subclass: #OBColumn
	instanceVariableNames: 'browser filter parent children selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn commentStamp: 'cwp 3/4/2004 21:48' prior: 0!
An OBColumn manages a list of nodes, which it displays in a PluggableListMorph in the pane scroller at the top of the browser. It's main responsibility is keeping its list - and those of its neighbours - up to date. Each column has a MetaNode, which provides the list contents. It uses a filter to meditate between it and the MetaNode.

iVars:

browser		- the browser which owns the column
filter		- the filter which manages the column's MetaNode.
parent		- the node selected in the column to the left of this column
children 	- the nodes which make up this column's list
selection 	- the index of the node selected by the user!

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 18:47'!
filter: aFilter
	filter _ aFilter monitor: self.
	self changed: #filter.! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 16:22'!
clear
	self clearNext.
	filter _ nil.
	parent _ nil.
	children _ #().
	self clearSelection.
	self changed: #list.
	self changed: #selection.
	self changed: #filter.! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 2/26/2004 22:06'!
getChildren
	children _ self filter nodesForParent: parent.
	^ children! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 23:07'!
hasSelection
	^ self selection > 0! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 23:11'!
nextChanged
	self hasSelection ifTrue:
		[self next ifNotNilDo:
			[:next |
			next
				metaNode: self nextMetaNode 
				parent: self selectedNode]]! !

!OBColumn methodsFor: 'initializing' stamp: 'avi 3/6/2004 16:23'!
setBrowser: aBrowser metaNode: aMetaNode node: aNode
	browser _ aBrowser.
	filter _ aMetaNode filter monitor: self.
	parent _ aNode.
	self getChildren.
	self clearSelection.! !

!OBColumn methodsFor: 'printing' stamp: 'cwp 2/11/2004 23:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self descriptor.
	aStream nextPut: $)! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 2/29/2004 21:49'!
listMorph
	^ (PluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			yourself! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 2/26/2004 22:08'!
selectAncestorOf: aNode
	| ancestor |
	ancestor _ self filter selectAncestorOf: aNode withParent: parent.
	ancestor ifNotNil: [self getChildren; select: ancestor; selected].
	^ ancestor! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:36'!
dragTransferTypeForMorph: listMorph
	^ self dragTransferType! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:08'!
select: aNode
	self selection: (children indexOf: (children detect: [:ea | ea name = aNode name] ifNone: []))! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 16:27'!
shouldBeLast
	^ self hasSelection not or: [self nextMetaNode hasChildren not]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:50'!
next
	^ self basicNext
		ifNil: [(self hasSelection and: [self shouldBeLast not])
				ifTrue: [self createNext]]
	! !

!OBColumn methodsFor: 'actions' stamp: 'avi 3/6/2004 16:26'!
actionsForSelectedNode
	| node |
	node _ self selectedNode ifNil: [^ #()].
	^ node metaNode actionsForNode: node	! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 21:26'!
wantsDroppedMorph: transferMorph event: evt inMorph: listMorph
	| node |
	(transferMorph isKindOf: TransferMorph) ifFalse: [^ false].
	transferMorph dragTransferType == self dragTransferType ifFalse: [^ false].
	node _ self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph.
	^ node notNil and: [node wantsDroppedNode: transferMorph passenger]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/5/2004 20:13'!
menu: aMenu
	self actionsForParentNode, self actionsForSelectedNode
		do: [:action | 
			action monitor: self.
			action addItemToMenu: aMenu].
	^ aMenu! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/7/2004 15:56'!
parent
	^ parent! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 3/14/2004 12:33'!
notification: aNotification
	aNotification notify: self.! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 21:48'!
listSize
	^ children
		ifNil: [0]
		ifNotNil: [children size]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/24/2004 01:03'!
list
	^ children 
		ifNil: [#()]
		ifNotNil: [children collect: [:ea | ea name]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 21:51'!
listAt: index
	^ (children at: index) name! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:22'!
isEmpty
	^ parent isNil! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:35'!
dragPassengerFor: item inMorph: listMorph
	^ self nodeForItem: item inMorph: listMorph! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:05'!
clearSelection
	selection _ 0! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 19:53'!
metaNode: aMetaNode parent: aNode
	self filter: aMetaNode filter.
	self parent: aNode.
	self clearNext.! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:49'!
basicNext
	^ browser columnAfter: self
		! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 2/9/2004 20:45'!
actionsForParentNode
	^ self filter children gather: [:child | child actionsForParent: parent].	! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:58'!
filter
	^ filter! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/11/2004 23:36'!
selected
	self next ifNotNilDo: [:column | column 
									metaNode: self nextMetaNode 
									parent: self selectedNode].
	browser selected: self! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 16:26'!
clicked
	self hasSelection
		ifFalse: [self deselected]
		ifTrue: [self selected]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/28/2004 13:29'!
nodeDeleted
	"This gets called by OBNodeDelete if an action causes the currently selected to be deleted."

	self getChildren.
	self changed: #list.
	self selection: 0.
	! !

!OBColumn methodsFor: 'initializing' stamp: 'avi 3/6/2004 16:23'!
setBrowser: aBrowser
	browser _ aBrowser.
	children _ #().
	self clearSelection.! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/7/2004 08:39'!
keystroke: aChar from: aMorph! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 13:45'!
dragEnabled
	^ true! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 2/9/2004 21:06'!
columnWithHeader
	| col bh |
	col _ BorderedMorph new
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
	
	bh _ self buttonHeight negated.
	col
		addMorph: self listMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@0 corner: 0@bh)).
	col
		addMorph: self filter buttonMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@bh corner: 0@0)).
		

	^ col! !

!OBColumn methodsFor: 'accessing' stamp: 'avi 3/6/2004 16:23'!
parent: aNode
	parent _ aNode.
	self getChildren.
	self clearSelection.
	self changed: #list.
	self nextChanged.
	browser nodeChanged.! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 3/1/2004 23:31'!
listChanged
	self refresh.
	self nextChanged.
	browser nodeChanged.! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:50'!
clearNext
	self basicNext ifNotNilDo: [:next | next clear].! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 12/14/2003 21:18'!
browser: aBrowser
	browser _ aBrowser! !

!OBColumn class methodsFor: 'as yet unclassified' stamp: 'cwp 1/24/2004 00:05'!
inBrowser: aBrowser metaNode: aMetaNode node: aNode
	^ self new
			setBrowser: aBrowser
			metaNode: aMetaNode
			node: aNode! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/4/2004 03:07'!
createNext
	| nextMetaNode created |
	nextMetaNode _ self nextMetaNode.
	created _ nextMetaNode
				columnInBrowser: browser
				node: self selectedNode.
	browser pushColumn: created.
	^ created! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 22:25'!
acceptDroppingMorph: transferMorph event: evt inMorph: listMorph
	^ [(self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph)
		acceptDroppedNode: transferMorph passenger]
			on: OBNotification
			do: [:aNotification |
				self notification: aNotification.
				aNotification resume]
! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 16:22'!
selectionChanged
	self clicked.
	self changed: #selection.
	browser nodeChanged! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 22:18'!
nodeForItem: item inMorph: pluggableListMorph
	^ children 
		detect: [:child | child name = item contents]
		ifNone: [item contents]! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:18'!
wantsButton
	^ self filter notNil and: [self filter wantsButton]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/28/2004 14:07'!
okToChange
	^ browser okToChange! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 11/5/2003 19:01'!
selection
	^ selection ifNil: [0]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:59'!
metaNode
	^ self filter metaNode! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 2/2/2004 21:54'!
selectedNode
	^ children at: self selection ifAbsent: [].! !

!OBColumn methodsFor: 'printing' stamp: 'avi 3/6/2004 16:23'!
descriptor
	self isEmpty ifTrue: [^ 'empty'].
	self hasSelection ifTrue: [^ self selectedNode name].
	^ ''! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:11'!
nextMetaNode
	^ self selectedNode metaNode! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 2/12/2004 18:11'!
paneWithHeader
	^ OBPane forColumn: self withFilter: self filter! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/30/2004 00:38'!
accept: aText notifying: aController
	self selectedNode accept: aText notifying: aController.! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/14/2004 12:33'!
refresh
	| node oldChildren |
	self isEmpty ifTrue: [^ self].
	node _ self selectedNode.
	oldChildren _ children.
	self getChildren.
	children = oldChildren 
		ifFalse: [self select: node.
				self hasSelection ifFalse: [self clearNext. browser nodeChanged]].
	self changed: #list.! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/12/2004 19:53'!
deselected
	browser deselected: self.
	self clearNext.! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 2/12/2004 18:10'!
simplePane
	^ OBPane forColumn: self! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:30'!
dropEnabled
	^ true! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 3/14/2004 12:29'!
nodeCreated: aNode
	browser jumpTo: aNode! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:23'!
nodeForDroppedMorph: transferMorph event: evt inMorph: pluggableListMorph
	| index item |
	index _ pluggableListMorph rowAtLocation: evt position.
	index = 0 ifTrue: [^ nil].
	item _ pluggableListMorph listMorph item: index.
	^ self nodeForItem: item inMorph: pluggableListMorph! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 16:22'!
selection: anInteger
	selection _ anInteger.
	self selectionChanged.
! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 12/14/2003 22:15'!
buttonHeight
	^ 20! !

!OBColumn class methodsFor: 'as yet unclassified' stamp: 'cwp 2/11/2004 23:25'!
inBrowser: aBrowser
	^ self new setBrowser: aBrowser! !

!OBColumn methodsFor: 'morphs' stamp: 'cwp 2/12/2004 19:17'!
morph
	^ self isEmpty 
		ifTrue: [self simplePane]
		ifFalse: [self filter wantsButton
							ifTrue: [self paneWithHeader]
							ifFalse: [self simplePane]]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:37'!
dragTransferType
	^ #OmniBrowser! !

Object subclass: #OBMetaEdge
	instanceVariableNames: 'label selector metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaEdge commentStamp: 'cwp 3/4/2004 22:02' prior: 0!
An OBMetaEdge represents an edge in the browser's metagraph. It represents a message sent to a node to obtain further nodes. It is refered to by the "parent" metanode, and refers to the "child" metanode.

iVars:

label 		-  a string describing the metaNode, for filters which allow the user 
			   to choose which edges to follow
selector		- when a node is selected by the user, this message will be sent to it to obtain its children
metaNode 	- a MetaNode corresponding to the nodes answered by the above message
		!

!OBMetaEdge methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:36'!
setLabel: aString selector: aSelector metaNode: aMetaNode
	label _ aString.
	selector _ aSelector.
	metaNode _ aMetaNode! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:35'!
label: aString selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:40'!
metaNode
	^ metaNode! !

!OBMetaEdge methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:40'!
selector
	^ selector! !

!OBMetaEdge methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:40'!
label
	^ label! !

!OBMetaEdge methodsFor: 'as yet unclassified' stamp: 'cwp 3/3/2004 00:12'!
nodesForParent: aNode
	^ (aNode perform: selector) do: [:ea | ea metaNode: metaNode]
! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 22:32'!
selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aSelector asString selector: aSelector metaNode: aMetaNode! !

Object subclass: #OBFilter
	instanceVariableNames: 'metaNode monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBFilter commentStamp: 'cwp 3/4/2004 21:53' prior: 0!
A browser's metagraph defines the way in which the user may traverse the graph of objects which make up the browser's domain. But it's not always desirable to have all the nodes made available by the metagraph to be visible in the browser. An filter provides a strategy for filtering out some of the nodes from the display. 

OBFilter provides a "null" filter - one that does no filtering at all - and serves as a superclass for other filters. !

!OBFilter class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 20:54'!
forMetaNode: aMetaNode
	^ self new setMetaNode: aMetaNode! !

OBFilter subclass: #OBModalFilter
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBModalFilter commentStamp: 'cwp 3/5/2004 15:16' prior: 0!
OBModalFilter is used to implement the 'instance/?/class' buttons in a standard class browser. In functional terms it filters the nodes of a column according to the edge of the metagraph that they correspond to. OBModalFilter displays an OBRadioButtonBar in its column's panel, with one button per edge. Only nodes from the currently selected edge are allowed in the column.

iVars:

selection	- the currently selected edge!

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/9/2004 21:10'!
selection
	^ selection ifNil: [selection _ 1]! !

!OBModalFilter methodsFor: 'morphs' stamp: 'cwp 2/25/2004 00:22'!
buttonMorph
	^ OBRadioButtonBar
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:28'!
list
	^ metaNode edges collect: [:edge | edge label]! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 3/3/2004 00:23'!
nodesForParent: aNode
	^ self selectedEdge nodesForParent: aNode! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:35'!
selectAncestorOf: aNode withParent: parentNode
	| ancestor |
	1 	to: metaNode edges size 
	  	do: [:i | selection _ i.
			ancestor _ (self nodesForParent: parentNode)
				detect: [:child | child isAncestorOf: aNode] 
				ifNone: [nil].
			ancestor ifNotNil: [self changed: #selection. ^ ancestor]].
	^ nil! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
selectedEdge
	^ metaNode edges at: self selection ! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:33'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection.
	monitor listChanged.
! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
incrementSelection
	selection _ selection + 1.
	(selection > metaNode edges size) ifTrue: [selection _ 1].! !

!OBModalFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:08'!
wantsButton
	^ true! !

!OBFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:06'!
wantsButton
	^ false! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:52'!
nodesForParent: aNode
	^ metaNode nodesForParent: aNode! !

!OBFilter methodsFor: 'initalizing' stamp: 'cwp 2/9/2004 20:54'!
setMetaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:56'!
monitor: aMonitor
	monitor _ aMonitor! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:50'!
children
	^ metaNode children! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:48'!
metaNode
	^ metaNode! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:34'!
selectAncestorOf: aNode withParent: parentNode
	^ (metaNode nodesForParent: parentNode)
		detect: [:child | child isAncestorOf: aNode] 
		ifNone: [nil]! !

Object subclass: #OBMetaGraphBuilder
	instanceVariableNames: ''
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBMetaGraphBuilder commentStamp: 'cwp 3/5/2004 12:39' prior: 0!
This is a utility class for building and installing all the metagraphs used by the standard browsers.!

!OBMetaGraphBuilder methodsFor: 'private' stamp: 'cwp 3/7/2004 21:43'!
catClassCatMethod
	| classCategory class methodCategory method metaclass comment |
	classCategory _ OBMetaNode named: 'ClassCategory'.
	class _ OBMetaNode named: 'Class'.
	comment _ OBMetaNode named: 'ClassComment'.
	metaclass _ OBMetaNode named: 'Metaclass'.
	methodCategory _ OBMetaNode named: 'MethodCategory'.
	method _ OBMetaNode named: 'Method'.

	classCategory 
		childAt: #classes labeled: 'instance' put: class;
		childAt: #comments labeled: '?' put: comment;
		childAt: #metaclasses labeled: 'class' put: metaclass;
		addActor: (OBNodeActor onNodeClass: OBClassCategoryNode);
		filterClass: OBModalFilter.
	class 
		childAt: #categories put: methodCategory;
		addActor: OBNodeActor new.
	metaclass
		childAt: #categories put: methodCategory.
	methodCategory 
		childAt: #methods put: method;
		addActor: (OBNodeActor onNodeClass: OBMethodCategoryNode).
	method 
		addActor: OBNodeActor new.
	
	^ classCategory! !

!OBMetaGraphBuilder methodsFor: 'browsers-misc' stamp: 'cwp 3/7/2004 23:23'!
metaGraphBrowser
	| dict assoc metaNode |
	dict _ OBMetaNode named: 'Dictionary'.
	assoc _ OBMetaNode named: 'Association'.
	metaNode _ OBMetaNode named: 'MetaNode'.

	dict childAt: #associations put: assoc.
	assoc childAt: #children put: metaNode.
	metaNode childAt: #children put: metaNode.
	
	^ dict ! !

!OBMetaGraphBuilder methodsFor: 'browsers-chasing' stamp: 'cwp 3/6/2004 21:15'!
chaseImplementorsBrowser
	| method message |
	method _ OBMetaNode named: 'Method'.
	message _ OBMetaNode named: 'Message'.
	
	method 
		childAt: #selectorAndMessages labeled: 'messages' put: message;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	message
		childAt: #implementors put: method;
		childAt: #senders put: method;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	^ method! !

!OBMetaGraphBuilder methodsFor: 'browsers-misc' stamp: 'cwp 3/6/2004 21:26'!
fileSystemBrowser
	| directory file |
	directory _ OBMetaNode named: 'Directory'.
	file _ OBMetaNode named: 'File'.
	file 
		addActor: OBNodeActor new;
		addActor: OBFileRegistryActor new.
	directory 
		childAt: #directories put: directory;
		childAt: #files put: file.
	^ directory! !

!OBMetaGraphBuilder class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 22:54'!
default
	^ Instance ifNil: [Instance _ self new]! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/6/2004 21:49'!
packageBrowser
	^ (OBMetaNode named: 'Package')
		childAt: #categories put: self catClassCatMethod;
		addActor: (OBNodeActor onNodeClass: OBPackageNode)
	
	! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/6/2004 21:51'!
sendersBrowser
	| method senders |
	method _ OBMetaNode named: 'Method'.
	senders _ OBMetaNode named: 'Implementors'.
	
	method
		childAt: #senders put: senders;
		addActor: OBNodeActor new.
	senders
		addActor: OBNodeActor new.
		
	^ method! !

!OBMetaGraphBuilder methodsFor: 'installing' stamp: 'cwp 3/13/2004 22:12'!
installGraphs
	OBBrowser 
	
		"Code browsers"
		metaGraphAt: #System put: self systemBrowser;
		metaGraphAt: #Image put: self imageBrowser;
		metaGraphAt: #Package put: self packageBrowser;
		metaGraphAt: #Implementors put: self implementorsBrowser;
		metaGraphAt: #Senders put: self sendersBrowser;
		metaGraphAt: #Versions put: self versionsBrowser;
		
		"Experimental chasing browsers"
		metaGraphAt: #ChaseImplementors put: self chaseImplementorsBrowser;
		metaGraphAt: #ChaseSenders put: self chaseSendersBrowser;
		metaGraphAt: #ChaseInstanceVariables put: self chaseIVarsBrowser;
		
		"Misc browsers"
		metaGraphAt: #FileSystem put: self fileSystemBrowser;
		metaGraphAt: #MetaGraphs put: self metaGraphBrowser
	! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/6/2004 21:51'!
systemBrowser
	^ (OBMetaNode named: 'System')
		childAt: #packages put: self packageBrowser! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/6/2004 21:41'!
implementorsBrowser
	| implementors method |
	method _ OBMetaNode named: 'Method'.
	implementors _ OBMetaNode named: 'Implementors'.
	
	method
		childAt: #implementors put: implementors;
		addActor: OBNodeActor new.
	implementors
		addActor: OBNodeActor new.
		
	^ method! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/6/2004 21:37'!
imageBrowser
	^ (OBMetaNode named: 'Image')
		childAt: #categories put: self catClassCatMethod! !

!OBMetaGraphBuilder methodsFor: 'browsers-chasing' stamp: 'cwp 3/6/2004 21:16'!
chaseSendersBrowser
	| method message |
	method _ OBMetaNode named: 'Method'.
	message _ OBMetaNode named: 'Message'.
	
	method 
		childAt: #selectorAndMessages labeled: 'messages' put: message;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	message
		childAt: #senders put: method;
		childAt: #implementors put: method;
		addActor: OBNodeActor new;
		filterClass: OBModalFilter.
	^ method! !

!OBMetaGraphBuilder methodsFor: 'browsers-code' stamp: 'cwp 3/14/2004 23:53'!
versionsBrowser
	| version |
	version _ OBMetaNode named: 'Version'.
	version addActor: OBNodeActor new.
	^ (OBMetaNode named: 'Method') 
		childAt: #versions put: version.! !

!OBMetaGraphBuilder methodsFor: 'browsers-chasing' stamp: 'cwp 3/11/2004 22:17'!
chaseIVarsBrowser
	| class ivar method message |
	class _ OBMetaNode named: 'Class'.
	ivar _ OBMetaNode named: 'InstanceVariable'.
	method _ OBMetaNode named: 'Method'.
	message _ OBMetaNode named: 'Message'.
	class
		addActor: OBNodeActor new;
		childAt: #instanceVariables labeled: 'ivars' put: ivar;
		filterClass: OBModalFilter.
	ivar
		childAt: #accessors put: method;
		filterClass: OBModalFilter.
	method
		addActor: OBNodeActor new;
		childAt: #senders put: message;
		filterClass: OBModalFilter.
	message
		addActor: OBNodeActor new;
		childAt: #senders put: message;
		filterClass: OBModalFilter.
	
	^ class! !

Object subclass: #OBActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBActor commentStamp: 'cwp 3/4/2004 21:02' prior: 0!
Actors the basic units of domain-related functionality. They are attached to metanodes, and supply the Actions used to build context menus and buttons in the browser. Actors can be proxies for functionality supplied by other classes, as in OBFileRegistryActor, or they may supply the funtionality them selves. 

OBActor is an abstract class which defines the protocol Actors must implement.Actors need not be subclasses of OBActor, however.!

!OBActor methodsFor: 'public' stamp: 'cwp 2/5/2004 20:12'!
actionsForParent: aNode
	^#()! !

OBActor subclass: #OBFileRegistryActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Actors'!

!OBFileRegistryActor commentStamp: 'cwp 3/5/2004 12:17' prior: 0!
This Actor serves as an intermediary between OBFileNodes and the file reader registry maintained by FileList. It makes the registries services available as file node actions.!

!OBFileRegistryActor methodsFor: 'as yet unclassified' stamp: 'cwp 3/10/2004 23:37'!
actionsForNode: aFileNode
	| services actions |
	services _ FileList itemsForFile: aFileNode path.
	services do: [:svc | svc addDependent: self].
	actions _ services collect: [:ea | OBAction
							label: ea label
							buttonLabel: ea buttonLabel
							receiver: ea
							selector: ea requestSelector
							arguments: (Array with: aFileNode path)].
	^ actions! !

!OBFileRegistryActor methodsFor: 'as yet unclassified' stamp: 'cwp 1/24/2004 01:05'!
update: aParameter
	(aParameter == #fileListChanged) ifTrue: [self updateFileList].
	super update: aParameter! !

!OBFileRegistryActor methodsFor: 'as yet unclassified' stamp: 'cwp 2/25/2004 22:42'!
updateFileList
	OBListChanged signal! !

OBActor subclass: #OBNodeActor
	instanceVariableNames: 'nodeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Actors'!

!OBNodeActor commentStamp: 'cwp 3/5/2004 12:23' prior: 0!
For simplicity, many actions are implemented by the nodes themselves. When the contextual menu for a node is displayed, this Actor gathers those actions and makes them available in to the user.

If a node class is specified, the OBNodeActor will also make available any class side actions provided by the node class. This is convenient for the user, since it makes it possible to invoke an action without first selecting a node. Many 'create' actions are implemented in this way.

iVars:

nodeClass - if not nil, this class's class-side actions will be available to the user as well.!

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:46'!
actionsForParent: aNode
	^ nodeClass 
		ifNotNil: [nodeClass actionsForParent: aNode]
		ifNil: [#()]! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:42'!
nodeClass: aClass
	nodeClass _ aClass! !

!OBNodeActor class methodsFor: 'as yet unclassified' stamp: 'cwp 3/3/2004 22:51'!
onNodeClass: aClass
	^ self new nodeClass: aClass! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 1/23/2004 23:24'!
actionsForNode: aNode
	^ aNode actions! !

!OBActor methodsFor: 'public' stamp: 'cwp 2/4/2004 15:25'!
actionsForNode: aNode
	^#()! !

Object subclass: #OBProtocolMock
	instanceVariableNames: 'messages answers exceptions isRecording'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/13/2004 00:41'!
resume: aNotificationClass with: aValue
	exceptions ifNil: [exceptions _ OrderedCollection new].
	exceptions add: (Array with: aNotificationClass with: aValue)! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:30'!
deny: aBoolean
	self assert: aBoolean not! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/29/2004 15:49'!
handleException: exception
	| expectation |
	(exception isKindOf: TestFailure) ifTrue: [exception pass].
	(exceptions isNil or: [exceptions isEmpty]) ifTrue: [exception pass].
	expectation _ exceptions removeFirst.
	exception class = expectation first ifFalse: [exception pass].
	exception isResumable ifTrue: [expectation second
									ifNil: [exception resume]
									ifNotNil: [exception resume: expectation second]]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:08'!
fail: aString
	^ TestResult failure sunitSignalWith: aString! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 01:41'!
examine: aBlock
	[self do: aBlock]
		on: Exception
		do: [:exception | self handleException: exception].

	self assert: messages isEmpty
		message: 'Expected message not sent'.
	self deny: (exceptions notNil and: [exceptions notEmpty])
		message: 'Expected exception not raised'! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
deny: aBoolean message: aString
	self assert: aBoolean not message: aString! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
assert: aBoolean
	self assert: aBoolean message: 'Assertion failed'! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 22:04'!
resume: aNotificationClass 
	^ self resume: aNotificationClass with: nil! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:28'!
answer
	^ answers 
		ifNil: [self]
		ifNotNil: [answers isEmpty 
							ifTrue: [self]
							ifFalse: [answers removeFirst]]! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:26'!
message: aMessage
	| expected |
	isRecording ifTrue: [messages add: aMessage. ^ self].
	
	self 
		deny: messages isEmpty 
		message: 'No more messages expected'.
	expected _ messages removeFirst.
	self assert: (expected sends: aMessage selector)
		message: 'Unexpected message'.
	self assert: expected arguments = aMessage arguments
		message: 'Incorrect arguments'.
		
	^ self answer! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/28/2004 17:22'!
answer: anObjectOrCollection
	answers ifNil: [answers _ OrderedCollection new].
	anObjectOrCollection isCollection
		ifTrue: [answers addAll: anObjectOrCollection]
		ifFalse: [answers add: anObjectOrCollection]! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 00:51'!
do: aBlock
	^ aBlock value: (OBMessageRecorder forMock: self)! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:28'!
assert: aBoolean message: aString
	aBoolean ifFalse: [self fail: aString]! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 00:51'!
expect: aBlock
	isRecording _ true.
	messages _ OrderedCollection new.
	self do: aBlock.
	isRecording _ false.! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:09'!
getListDelicately
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy item: i].
	! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:47'!
getListObtrusively
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy getListItem: i].
	! !

!LazyListMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/25/2004 21:17'!
rectForRow: index
	"return a rectangle containing the row at index"
	
	| top |
	top _ self top + (index - 1 * font height).
	^ (self left @ top) extent: self width @ font height
! !

TestCase subclass: #OBFileNodeTest
	instanceVariableNames: 'node notification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/23/2004 22:06'!
testDelete
	| actions delete |
	actions _ node actions.
	delete _ actions detect: [:a | a label = 'delete'] ifNone: [self signalFailure: 'No delete action'].
	delete monitor: self.
	delete trigger.
	self deny: ((FileDirectory default) fileExists: self dataFile).
	self assert: notification class = OBNodeDeleted

	! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/15/2003 21:41'!
testText
	self assert: (node text = 'alpha')! !

!OBFileNodeTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:32'!
resources
	^ Array with: OBDirectoryResource! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/7/2004 21:38'!
tearDown
	| file |
	file _ FileStream fileNamed: self dataFile.
	file
		reset;
		nextPutAll: 'alpha';
		truncate: (file position);
		close.! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/15/2003 21:40'!
setUp
	node _ OBFileNode on: self dataFile.! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:15'!
dataFile
	^ OBDirectoryResource current dataDir fullNameFor: 'alpha'! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/23/2004 23:15'!
notification: aNotification
	notification _ aNotification! !

!OBFileNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/15/2003 21:41'!
testName
	self assert: node name = 'alpha'! !

TestCase subclass: #OBRadioButtonBarTest
	instanceVariableNames: 'bar selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:15'!
testUpdateSelection
	self changed: #list.
	selection _ 2.
	self changed: #selection.
	self assertSelectionIs: 2.! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:31'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection! !

!OBRadioButtonBarTest methodsFor: 'simulating' stamp: 'cwp 3/11/2004 23:02'!
push: aString
	| button |
	button _ bar submorphs detect: [:b | b label contents = aString].
	button performAction! !

!OBRadioButtonBarTest methodsFor: 'asserting' stamp: 'cwp 2/24/2004 19:35'!
assertSelectionIs: anInteger
	self assert: selection = anInteger.
	bar submorphs withIndexDo: [:button :index |
								index = anInteger
									ifTrue: [self assert: button isOn]
									ifFalse: [self deny: button isOn]]! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 17:42'!
list
	^ #('alpha' 'beta' 'gamma')! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 23:04'!
testUpdateList
	| labels |
	self changed: #list.
	labels _ bar submorphs collect: [:b | b label contents].
	self assert: labels size = self list size.
	self assert: (labels includesAllOf: self list)! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:35'!
testPushAlphaBeta
	self changed: #list.
	self push: 'alpha'.
	self assertSelectionIs: 1.
	self push: 'beta'.
	self assertSelectionIs: 2.! !

!OBRadioButtonBarTest methodsFor: 'running' stamp: 'cwp 2/25/2004 00:43'!
setUp
	selection _ 0.
	bar _ OBRadioButtonBar
			on: self
			list: #list
			selected: #selection
			changeSelected: #selection:.
! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/22/2004 16:53'!
testList
	self assert: bar list = self list! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 16:39'!
selection
	^ selection! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 3/2/2004 20:40'!
testLayout
	| rects |
	bar bounds: (10@10 corner: 310@30).
	self changed: #list.
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	bar openInWorld.
	rects _ bar submorphs collect: [:b | b bounds].
	bar delete.

	rects do: [:ea | self assert: ea top = 10; assert: ea bottom = 30].
	self assert: rects first left = 10.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = 310.! !

TestCase subclass: #OBSilentTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBSilentTestCase methodsFor: 'running' stamp: 'cwp 2/29/2004 12:36'!
runCaseAsFailure: aSemaphore
	SystemChangeNotifier uniqueInstance doSilently: [super runCaseAsFailure: aSemaphore]! !

OBSilentTestCase subclass: #OBCodeSanityCheckTest
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: 'cwp 2/28/2004 11:22'!
codeContext
	^ OBPackageContext on: OBFakePackage default! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 12:47'!
testClassDefinition
	node _ OBClassNode on: #OBFakeClassA inContext: self codeContext.
	self 
		should: [node accept: self fakeCDefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBCodeSanityCheckTest methodsFor: 'callbacks' stamp: 'cwp 2/28/2004 11:24'!
selectionInterval
	^ 1 to: 0! !

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: 'cwp 2/28/2004 12:47'!
fakeCDefinition
	^ 'Object subclass: #OBFakeClassC
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeSanityCheckTest methodsFor: 'callbacks' stamp: 'cwp 2/28/2004 11:27'!
text
	^ node text asText! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 12:51'!
testClassCreation
	node _ OBClassCategoryNode on: 'Fakes' inContext: self codeContext.
	self 
		should: [node accept: self fakeCDefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBCodeSanityCheckTest methodsFor: 'constants' stamp: 'cwp 2/28/2004 12:49'!
fakeARedefinition
	^ OBFakeClassA definition! !

!OBCodeSanityCheckTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 12:49'!
testClassRedefinition
	node _ OBClassNode on: #OBFakeClassA inContext: self codeContext.
	self 
		shouldnt: [node accept: self fakeARedefinition notifying: self]
		raise: OBConfirmationRequest! !

!OBSilentTestCase methodsFor: 'running' stamp: 'cwp 2/29/2004 12:34'!
runCase
	SystemChangeNotifier uniqueInstance doSilently: [super runCase]! !

OBSilentTestCase subclass: #OBContextTest
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBContextTest commentStamp: 'cwp 2/29/2004 13:52' prior: 0!
This is an abstract superclass for testing code contexts. It implements a suite of tests, but relies on subclasses to supply constants and expected results appropriate to the context being tested.!

!OBContextTest methodsFor: 'queries' stamp: 'cwp 2/7/2004 14:52'!
fakeASelectors
	^ (context methodsForClass: #OBFakeClassA)
		collect: [:ref | ref methodSymbol]! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 13:25'!
testClassCategories
	self assert: (context categories includesAllOf: self internalCategories).
	self deny: (context categories includesAnyOf: self externalCategories)! !

!OBContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 11:45'!
fakeCategory
	^ 'OmniBrowser-Fakes'! !

!OBContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 11:55'!
createFakeD
	Object subclass: #OBFakeClassD
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'OmniBrowser-Fakes'! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 12:17'!
testMethodCategoryOf
	| ref |
	ref _ MethodReference new setStandardClass: OBFakeClassA methodSymbol: #one.
	self assert: (context methodCategoryOf: ref) = 'numbers'! !

!OBContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:31'!
obHopefullyUniqueSelector
	self obOtherUniqueSelector! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 13:32'!
testMethodsForClass
	self assert: (self fakeASelectors includesAllOf: self fakeACoreSelectors).
	self assert: (self fakeASelectors = self fakeASelectors sort).
	self deny: (self fakeASelectors includesAnyOf: self fakeAExtensionSelectors).! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 23:34'!
testImplementors
	| actual expected |
	actual _ context allImplementorsOf: #obHopefullyUniqueSelector.
	expected _ self uniqueImplementors.
	self assert: actual size = expected size.
	self assert: (actual includesAllOf: expected)! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 23:51'!
testSenders
	| actual expected |
	actual _ context allSendersOf: #obOtherUniqueSelector.
	expected _ self uniqueSenders.
	self assert: actual size = expected size.
	self assert: (actual includesAllOf: expected)! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/8/2004 12:07'!
testMethodsInCategoryOfClass
	| refs selectors |
	refs _ context methodsInCategory: 'numbers' ofClass: #OBFakeClassA.
	selectors _ refs collect: [:ref | ref methodSymbol].
	self assert: (selectors includesAllOf: #(one two three five))! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 23:22'!
testSourceFiles
	| ref pointer |
	ref _ MethodReference new.
	ref setStandardClass: OBContextTest methodSymbol: #testSourceFiles.
	pointer _ context sourcePointerFor: ref.
	self assert: (SourceFiles fileIndexFromSourcePointer: pointer) = 2.
	self assert: (SourceFiles filePositionFromSourcePointer: pointer) > 0.! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 12:53'!
testClassesInCategory
	self assert: ((context classesInCategory: self fakeCategory)
					 includesAllOf: self fakeClasses).
	! !

!OBContextTest methodsFor: 'asserting' stamp: 'cwp 2/7/2004 12:02'!
assertClassExists: aString
	self assert: (self classExists: aString)! !

!OBContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:29'!
obOtherUniqueSelector! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 13:39'!
testClassCategoryOperations
	context addCategory: 'OBContext-Added'.
	self assert: (context categories includes: 'OBContext-Added').
	context renameCategory: 'OBContext-Added' toBe: 'OBContext-Renamed'.
	self assert: (context categories includes: 'OBContext-Renamed').
	self deny: (context categories includes: 'OBContext-Added').
	context removeCategory: 'OBContext-Renamed'.
	self deny: (context categories includes: 'OBContext-Renamed')
! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 00:03'!
testMethodCategories
	context addCategory: 'added' toClass: #OBFakeClassB.
	self assert: ((context categoriesForClass: #OBFakeClassB) includes: 'added').
	context renameCategory: 'added' inClass: #OBFakeClassB toBe: 'renamed'.
	self assert: ((context categoriesForClass: #OBFakeClassB) includes: 'renamed').
	self deny: ((context categoriesForClass: #OBFakeClassB) includes: 'added').
	context removeCategory: 'renamed' fromClass: #OBFakeClassB.
	self deny: ((context categoriesForClass: #OBFakeClassB) includes: 'renamed')! !

!OBContextTest methodsFor: 'asserting' stamp: 'cwp 2/7/2004 12:02'!
denyClassExists: aString
	self deny: (self classExists: aString)! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 14:42'!
testClasses
	self createFakeD.
	self assertClassExists: #OBFakeClassD.
	context renameClass: #OBFakeClassD toBe: #OBFakeClassE.
	self assertClassExists: #OBFakeClassE.
	self denyClassExists: #OBFakeClassD.
	context removeClass: #OBFakeClassE.
	self denyClassExists: #OBFakeClassE! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:31'!
testAccessesTo
	| selectors |
	selectors _ (context accessesToVariable: 'ivar' ofClass: #OBFakeClassB)
				 collect: [:ea | ea methodSymbol].
	self assert: (selectors size = self accessorsOfIVar size).
	self assert: (selectors includesAllOf: self accessorsOfIVar).! !

!OBContextTest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 12:36'!
isAbstract
	^ self name = 'OBContextTest'! !

OBContextTest subclass: #OBPackageContextTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPackageContextTest methodsFor: 'running' stamp: 'cwp 3/4/2004 00:20'!
setUp
	context _ OBPackageContext on: (PackageInfo hidden: 'OmniBrowserFakes')! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 12:43'!
fakeCategory
	^ 'Fakes'! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 13:27'!
internalCategories
	^ #('Fakes')! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 12:46'!
fakeACoreSelectors
	^ #(one two three a b)! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 13:27'!
externalCategories
	^ #('Kernel')! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 12:46'!
fakeAExtensionSelectors
	^ #(foo)! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 3/11/2004 22:33'!
accessorsOfIVar
	^ #(accessesIvar)! !

!OBPackageContextTest methodsFor: 'queries' stamp: 'cwp 2/7/2004 12:37'!
classExists: aString
	^ Smalltalk hasClassNamed: aString! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:41'!
uniqueImplementors
	| ref |
	ref _ MethodReference new.
	ref setStandardClass: OBFakeClassA methodSymbol: #obHopefullyUniqueSelector.
	^ Array with: ref! !

!OBPackageContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:51'!
uniqueSenders
	^ self uniqueImplementors! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 12:30'!
testPackages
	| names |
	names _ context packages collect: [:p | p packageName].
	self assert: (names includes: 'OmniBrowser').! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/9/2004 20:11'!
testCategoryOf
	self assert: (context categoryOf: #OBFakeClassA) = self fakeCategory.
	self assert: (context categoryOf: #'OBFakeClassA class') = self fakeCategory.! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 3/2/2004 20:50'!
testSetMethodCategory
	| ref |
	ref _ MethodReference new.
	ref setStandardClass: OBFakeClassA methodSymbol: #one.
	context setCategoryOf: #one inClass: #OBFakeClassA toBe: 'letters'.
	self assert: (OBFakeClassA organization categoryOfElement: #one) = 'letters'.
	context setCategoryOf: #one inClass: #OBFakeClassA toBe: 'numbers'.
	self assert: (OBFakeClassA organization categoryOfElement: #one) = 'numbers'.
! !

!OBContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 14:41'!
fakeClasses
	^ #(OBFakeClassA OBFakeClassB OBFakeClassC)! !

!OBContextTest methodsFor: 'testing' stamp: 'cwp 2/9/2004 23:42'!
testMessagesSentByInClass
	| messages |
	messages _ context messagesSentBy: #five inClass: #OBFakeClassA.
	self assert: (messages asSet = #(two + three) asSet)
	! !

OBContextTest subclass: #OBImageContextTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBImageContextTest methodsFor: 'running' stamp: 'cwp 2/7/2004 12:35'!
setUp
	context _ OBImageContext new! !

!OBImageContextTest class methodsFor: 'initializing' stamp: 'cwp 3/10/2004 18:53'!
initialize
	OBFakeClassA compile: 'foo' classified: '*obforeign'.
	OBFakeClassB compile: 'foreign ^ first' classified: '*obforeign'.! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:50'!
uniqueImplementors
	| results ref |
	results _ OrderedCollection new.
	#(OBFakeClassA obHopefullyUniqueSelector 
	OBContextTest obHopefullyUniqueSelector)
		 pairsDo: [:class :selector |
					ref _ MethodReference new.
					ref 
						setClassSymbol: class
						classIsMeta: false
						methodSymbol: selector
						stringVersion: ''.
					results add: ref].
	^ results
	
	! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 12:48'!
fakeACoreSelectors
	^ #(one two three a b foo)! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 12:48'!
fakeAExtensionSelectors
	^ #()! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 13:31'!
internalCategories
	^ #('OmniBrowser-Kernel' 'OmniBrowser-Nodes' 'OmniBrowser-Actors')! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 3/11/2004 22:32'!
accessorsOfIVar
	^ #(accessesIvar foreign)! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/29/2004 23:50'!
uniqueSenders
	| results ref |
	results _ OrderedCollection new.
	#(OBFakeClassA obHopefullyUniqueSelector 
	OBContextTest obHopefullyUniqueSelector 
	OBContextTest testSenders)
		 pairsDo: [:class :selector |
					ref _ MethodReference new.
					ref 
						setClassSymbol: class
						classIsMeta: false
						methodSymbol: selector
						stringVersion: ''.
					results add: ref].
	^ results
	
	! !

!OBImageContextTest methodsFor: 'queries' stamp: 'cwp 2/7/2004 12:35'!
classExists: aString
	^ Smalltalk hasClassNamed: aString! !

!OBImageContextTest methodsFor: 'fakes' stamp: 'cwp 2/7/2004 13:29'!
externalCategories
	^ #()! !

TestCase subclass: #OBPanelLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 3/9/2004 23:38'!
testTwoTextPanels
	"Yeah, ok, this is a bit ridiculous"
	
	| window |
	window _ self windowWithPanels: {OBTextPanel. OBTextPanel}.	
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.7)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.7 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 3/9/2004 23:37'!
windowWithPanels: panelClasses
	| browser |
	browser _ OBBrowser metaNode: self metaNode node: self node.
	browser minPanes: 4; maxPanes: 4.
	panelClasses do: [:class |  browser addPanel: (class inBrowser: browser)].
	^ browser window.
! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 3/9/2004 23:56'!
testTextAndButtonPanels
	| window |
	window _ self windowWithPanels: {OBButtonPanel. OBTextPanel}.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@25).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@25 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 3/9/2004 23:37'!
testNoPanels
	| window |
	window _ self windowWithPanels: #().
	self assert: window submorphs size = 2.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 3/9/2004 23:38'!
testTextPanel
	| window |
	window _ self windowWithPanels: {OBTextPanel}.
	self assert: window submorphs size = 3.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 3/9/2004 21:53'!
node
	^ OBSystemNode on: (OBImageContext new)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 3/9/2004 21:53'!
metaNode
	^ OBBrowser metaGraphAt: #Image! !

!OBPanelLayoutTest methodsFor: 'asserting' stamp: 'cwp 3/9/2004 22:21'!
assertFrame: frame hasFractions: fraction andOffsets: offset
	self assert: frame leftFraction = fraction left.
	self assert: frame topFraction = fraction top.
	self assert: frame rightFraction = fraction right.
	self assert: frame bottomFraction = fraction bottom.
	
	self assert: frame leftOffset = offset left.
	self assert: frame topOffset = offset top.
	self assert: frame rightOffset = offset right.
	self assert: frame bottomOffset = offset bottom.
! !

TestCase subclass: #OBMethodVersionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMethodVersionTest methodsFor: 'testing' stamp: 'cwp 3/13/2004 15:11'!
testScanFromMethodRef
	| changes |
	changes _ OBMethodVersion versionsOfMethod: self methodReference.
	self deny: changes isEmpty! !

!OBMethodVersionTest methodsFor: 'constants' stamp: 'cwp 3/13/2004 14:03'!
methodReference
	| ref |
	ref _ MethodReference new.
	ref setStandardClass: self class methodSymbol: #testScanFromMethodRef.
	^ ref! !

!OBMethodVersionTest methodsFor: 'testing' stamp: 'cwp 3/13/2004 21:49'!
testScanFromPointer
	| changes sourcePointer sources |
	sources _  self fakeSourceFileArray.
	sourcePointer _ sources sourcePointerFromFileIndex: 2 andPosition: 518.
	changes _ OBMethodVersion scan: sources from: sourcePointer.
	self assert: changes size = 6.
	changes do: [:ea | 
					self assert: ea classSymbol = #OBFakeClassC.
					self assert: ea category = 'as yet unclassified'.
					self assert: (ea stamp beginsWith: 'cwp 3/11/2004 20:0')]! !

!OBMethodVersionTest methodsFor: 'constants' stamp: 'cwp 3/13/2004 22:39'!
fakeSourceFileArray
	^ OBSourceFilesResource current sources! !

TestCase subclass: #OBFileNodeEditTest
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFileNodeEditTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:30'!
createFile
	| file |
	file _ FileStream newFileNamed: self dataFile.
	file
		nextPutAll: 'omega';
		close.
! !

!OBFileNodeEditTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:27'!
dataFile
	^ OBDirectoryResource current dataDir fullNameFor: 'omega'! !

!OBFileNodeEditTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/30/2004 00:39'!
testAcceptText
	| file |
	node text: 'foo'.
	file _ FileStream readOnlyFileNamed: self dataFile.
	self assert: file contents = 'foo'! !

!OBFileNodeEditTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/15/2003 21:40'!
setUp
	self createFile.
	node _ OBFileNode on: self dataFile.
	! !

!OBFileNodeEditTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:28'!
tearDown
	FileDirectory deleteFilePath: self dataFile! !

TestCase subclass: #OBActionTest
	instanceVariableNames: 'mock notification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBActionTest commentStamp: 'cwp 2/29/2004 13:36' prior: 0!
This is an abstract superclass for testing actions on nodes that represent code elements. My subclasses use a ProtocolMock to verify that triggering the action causes the correct messages to be sent to the node's context.!

!OBActionTest methodsFor: 'simulating' stamp: 'cwp 2/29/2004 14:05'!
trigger: aString inContext: aContext
	^ self 
		trigger: aString 
		nodeSelector: #nodeInContext: 
		parentSelector: #parentInContext: 
		inContext: aContext! !

OBActionTest subclass: #OBMethodCategoryActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 22:47'!
testRemoveCategory
	mock
		resume: OBNodeDeleted;
		expect: [:context | context removeCategory: 'foo' fromClass: #OBFakeClassB];
		examine: [:context | self trigger: 'remove' inContext: context]! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 2/29/2004 22:33'!
nodeClass
	^ OBMethodCategoryNode! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 18:06'!
testSortCategories
	mock
		resume: OBListChanged;
		expect: [:context | context sortCategoriesInClass: #OBFakeClassB];
		examine: [:context | self trigger: 'alphabetize categories' inContext: context]! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 2/29/2004 14:07'!
metaClassNodeInContext: aContext
	^ OBMetaclassNode on: #OBFakeClassB inContext: aContext! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 22:46'!
testCreateCategory
	mock
		resume: OBTextRequest with: 'quux';
		resume: OBNodeSelected;
		expect: [:context | context addCategory: 'quux' toClass: #OBFakeClassB];
		examine: [:context | self trigger: 'create category...' inContext: context]! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 14:07'!
testCreateClassCategory
	mock
		resume: OBTextRequest with: 'quux';
		resume: OBNodeSelected;
		expect: [:context | context addCategory: 'quux' toClass: #'OBFakeClassB class'];
		examine: [:context | self 
							trigger: 'create category...' 
							nodeSelector: #nodeInContext: 
							parentSelector: #metaClassNodeInContext:
							inContext: context]! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 22:48'!
testRenameCategory
	mock
		resume: OBTextRequest with: 'quux';
		resume: OBNodeSelected;
		expect: [:context | context renameCategory: 'foo' inClass: #OBFakeClassB toBe: 'quux'];
		examine: [:context | self trigger: 'rename...' inContext: context]! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 3/4/2004 01:45'!
testCompileScaryMethod
	mock
		resume: OBConfirmationRequest with: false;
		expect: [:context | context methodsForClass: #'OBFakeClassB class'];
		answer: {#(). OBFakeClassB class};
		examine: [:context | (self metaMethodCatNodeInContext: context)
							accept: self scaryMethod notifying: nil]! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 3/2/2004 21:50'!
methodNodeInContext: aContext
	^ OBMethodNode on: #quux inClass: #OBFakeClassB inContext: aContext! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 3/4/2004 01:20'!
metaMethodCatNodeInContext: aContext
	^ OBMethodCategoryNode on: 'instance creation' inClass:  #'OBFakeClassB class' inContext: aContext! !

!OBMethodCategoryActionTest methodsFor: 'testing' stamp: 'cwp 3/2/2004 21:50'!
testDropMethod
	mock
		resume: OBNodeSelected;
		expect: [:context | context setCategoryOf: #quux inClass: #OBFakeClassB toBe: 'foo'];
		examine: [:context | self
							drop: (self methodNodeInContext: context)
							on: (self nodeInContext: context)]! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 2/29/2004 22:34'!
nodeInContext: aContext
	^ OBMethodCategoryNode on: 'foo' inClass: #OBFakeClassB inContext: aContext! !

!OBMethodCategoryActionTest methodsFor: 'constants' stamp: 'cwp 3/4/2004 01:18'!
scaryMethod
	^ 'new ^ super new'! !

!OBMethodCategoryActionTest methodsFor: 'node creation' stamp: 'cwp 2/29/2004 22:44'!
parentInContext: aContext
	^ OBClassNode on: #OBFakeClassB inContext: aContext! !

!OBActionTest methodsFor: 'simulating' stamp: 'cwp 2/29/2004 23:03'!
drop: aNode on: targetNode
	self assert: (targetNode wantsDroppedNode: aNode).
	targetNode acceptDroppedNode: aNode! !

!OBActionTest methodsFor: 'simulating' stamp: 'cwp 3/3/2004 22:59'!
actor
	^ OBNodeActor onNodeClass: self nodeClass! !

!OBActionTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 21:41'!
parentInContext: aContext
	^ OBSystemNode on: aContext! !

OBActionTest subclass: #OBMessageActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMessageActionTest methodsFor: 'private' stamp: 'cwp 2/29/2004 19:07'!
nodeInContext: aContext
	^ OBMessageNode on: #quux inMethod: #baz inClass: #OBFakeClassA inContext: aContext! !

!OBMessageActionTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:19'!
testSenders
	| methods |
	mock
		expect: [:context | context allSendersOf: #quux];
		answer: (Array with: self quuxNodes);
		examine: [:context | methods _ (self nodeInContext: context) sendersOfMessage].
	
	self assertMatchesQuux: methods.! !

!OBMessageActionTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:05'!
assertMatchesQuux: methods
	methods with: self quuxNodes do: [:a :b | self assert: a reference  = b]! !

!OBMessageActionTest methodsFor: 'private' stamp: 'cwp 2/29/2004 19:05'!
quuxNodes
	| ref |
	ref _ MethodReference new.
	ref setStandardClass: OBFakeClassA methodSymbol: #five.
	^ Array with: ref! !

!OBMessageActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 19:05'!
testImplementors
	| methods |
	mock
		expect: [:context | context allImplementorsOf: #quux];
		answer: (Array with: self quuxNodes);
		examine: [:context | methods _ (self nodeInContext: context) implementors].
	
	self assertMatchesQuux: methods! !

!OBActionTest methodsFor: 'simulating' stamp: 'cwp 2/29/2004 14:04'!
trigger: aString nodeSelector: nodeSelector parentSelector: parentSelector inContext: aContext
	| action actor node parent |
	actor _ self actor.
	node _ self perform: nodeSelector with: aContext.
	parent _ self perform: parentSelector with: aContext.
	action _ ((actor actionsForNode: node), (actor actionsForParent: parent))
		detect: [:a | a label = aString]
		ifNone: [self signalFailure: 'No action with label: ', aString].
	action monitor: self.
	action trigger! !

!OBActionTest methodsFor: 'notification' stamp: 'cwp 2/28/2004 22:02'!
notification: aNotification
	aNotification pass! !

OBActionTest subclass: #OBClassActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBClassActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:11'!
testRemoveClass
	mock 
		resume: OBConfirmationRequest with: true;
		resume: OBNodeDeleted;
		expect: [:context | context removeClass: #OBFakeClassA];
		examine: [:context | self trigger: 'remove' inContext: context]
				! !

!OBClassActionTest methodsFor: 'private' stamp: 'cwp 2/13/2004 02:03'!
nodeClass
	^ OBClassNode! !

!OBClassActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:12'!
testRenameClass
	mock 
		resume: OBTextRequest with: 'OBFakeClassE';
		resume: OBNodeSelected;
		expect: [:context | context renameClass: #OBFakeClassA toBe: #OBFakeClassE];
		examine: [:context | self trigger: 'rename...' inContext: context]! !

!OBClassActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 16:32'!
nodeInContext: aContext
	^ OBClassNode on: #OBFakeClassA inContext: aContext! !

!OBActionTest class methodsFor: 'testing' stamp: 'cwp 2/28/2004 16:34'!
isAbstract
	^ self name = #OBActionTest! !

OBActionTest subclass: #OBMethodActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMethodActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 17:33'!
testImplementors
	mock
		expect: [:context | context allImplementorsOf: #five];
		answer: (Array with: self fiveImplementors);
		examine: [:context | (self nodeInContext: context) implementors]! !

!OBMethodActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 17:32'!
fiveImplementors
	| ref |
	ref _ MethodReference new.
	ref setClassSymbol: #OBFakeClassA classIsMeta: false methodSymbol: #five stringVersion: ''.
	^ Array with: ref! !

!OBMethodActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 16:50'!
nodeInContext: aContext
	^ OBMethodNode on: #five inClass: #OBFakeClassA inContext: aContext! !

!OBMethodActionTest methodsFor: 'testing' stamp: 'cwp 3/4/2004 02:45'!
testRemoveMethod
	mock
		resume: OBChoiceRequest with: #doRemove;
		resume: OBNodeDeleted with: true;
		expect: [:context | context
							allSendersOf: #five;
							removeMethod: #five fromClass: #OBFakeClassA];
		answer: #((foo));
		examine: [:context | (self nodeInContext: context) remove]! !

!OBMethodActionTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 19:15'!
testMessages
	| selectors messages |
	selectors _ #(+ two three five).
	mock
		expect: [:context | context messagesSentBy: #five inClass: #OBFakeClassA];
		answer: (Array with: selectors);
		examine: [:context | messages _ (self nodeInContext: context) messages].

	messages do: [:ea | self assert: (ea isKindOf: OBMessageNode).
				self assert: (selectors includes: ea name)]! !

OBActionTest subclass: #OBClassCategoryActionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBClassCategoryActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 21:33'!
nodeClass
	^ OBClassCategoryNode! !

!OBClassCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:07'!
testCreateCategory
	mock
		resume: OBTextRequest with: 'Quux';
		resume: OBNodeSelected;
		expect: [:context | context addCategory: 'Quux'];
		examine: [:context | self trigger: 'create category...' inContext: context]! !

!OBClassCategoryActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 22:28'!
classNodeInContext: aContext
	^ OBClassNode on: #OBFakeClassA inContext: aContext! !

!OBClassCategoryActionTest methodsFor: 'private' stamp: 'cwp 2/28/2004 21:36'!
nodeInContext: aContext
	^ OBClassCategoryNode on: 'Foo' inContext: aContext! !

!OBClassCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:19'!
testRenameCategory
	mock
		resume: OBTextRequest with: 'Quux';
		resume: OBListChanged;
		expect: [:context | context renameCategory: 'Foo' toBe: 'Quux'];
		examine: [:context | self trigger: 'rename...' inContext: context]! !

!OBClassCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:16'!
testRemoveCategory
	mock
		resume: OBNodeDeleted;
		expect: [:context | context removeCategory: 'Foo'];
		examine: [:context | self trigger: 'remove' inContext: context]! !

!OBClassCategoryActionTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:27'!
testDropClass
	mock
		resume: OBNodeSelected;
		expect: [:context | context setCategoryOf: #OBFakeClassA to: 'Foo'];
		examine: [:context | 
				(self nodeInContext: context) acceptDroppedNode: (self classNodeInContext: context)]! !

!OBActionTest methodsFor: 'running' stamp: 'cwp 2/28/2004 16:43'!
setUp
	mock _ OBProtocolMock new! !

TestCase subclass: #OBBrowserTest
	instanceVariableNames: 'model morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

OBBrowserTest subclass: #OBFakeBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 14:25'!
testAcceptText
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self setText: 'x'.
	
	"This is a gratuitous refresh to make the test pass. It isn't necessary when the 
	browser is actually open. It's a deficiency of the test environment..."
	self listMorphs do: [:ea | ea update: #list].
	
	self assertListAt: 1 contains: #(x b c).
	self assertListAt: 2 contains: #(xa xb xc).
	self assertTextIs: 'x'
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectC
	self select: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #().
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectA
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 12/17/2003 15:13'!
metaGraph
	^ OBFakeNode metaGraph! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/17/2003 16:44'!
testInitialDisplay
	self assert: self listMorphs size = 1.
	self assertListAt: 1 contains: #(a b c).
	self assertTextIs: ''.
	! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 13:49'!
testConfirmAbandon
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self editText: 'x'.
	[self select: #b]
		on: OBConfirmationRequest
		do: [:notification | notification cancel].
	self assertCurrentItemIs: #a.
	self assertTextIs: 'x'.
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:29'!
testDeselectA
	self select: #a.
	self deselect: #a.
	self assertListAt: 1 contains: #(a b c).
	self assert: self listMorphs size = 1.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/25/2004 21:58'!
testDragAndDrop
	| node |
	self select: #a.
	node _ self drag: #ab.
	self assert: (node isKindOf: OBFakeNode).
	self assert: node name = #ab.
	self drop: node on: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(cb).
	self assertCurrentItemIs: #c.
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/8/2003 20:33'!
testPanes
	| scroller |
	self assert: (self morphsOfClass: PluggableListMorph) size = 1.
	self assert: (self morphsOfClass: PluggableTextMorph) size = 1.
	scroller _ self morphNamed: 'scroller'.
	self assert: scroller panes size = model minPanes.! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/23/2003 13:12'!
testCreateWindow
	self assert: (morph isKindOf: SystemWindow).
	self assert: (morph label = 'OmniBrowser').
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 1/7/2004 08:27'!
testAction
	self select: #a.
	self selectMenuItem:  'become z'.
	self assertListAt: 1 contains:  #(z b c)! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 13:33'!
testDeleteB
	self select: #b.
	self selectMenuItem: 'delete'.
	self assertListAt: 1 contains: #(a c).
	self assert: self listMorphs size = 1.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 12/17/2003 15:12'!
node
	^ OBFakeNode new! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:10'!
assertAListMatches: strings
	| list |
	self listMorphs
			detect:	[:m | 
					list _ m getListDelicately.
					(list size = strings size) and: [list includesAllOf: strings]]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 3/14/2004 13:11'!
findListContaining: aString
	^ self listMorphs detect: [:m | m getListDelicately includes: aString] ifNone: []! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphsOfClass: aClass
	| found |
	found _ OrderedCollection new.
	morph allMorphsDo: [:m | (m isMemberOf: aClass) ifTrue: [found add: m]].
	^ found! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:38'!
assertSelectedTextIs: aString
	| text selection |
	morph _ self textMorph.
	text _ morph getText.
	selection _ morph getSelection.
	self assert: (text atAll: selection) asString = aString.! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/23/2004 22:35'!
assertMenuContains: aString
	| item |
	item _ self menuItemNamed: aString.
	self assert: item notNil! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:14'!
mouseMoveEventIn: name
	| rect plm |
	plm _ self findListContaining: name.
	rect _ plm listMorph rectForRow: (plm getListDelicately indexOf: name).
	^ MouseMoveEvent new
		setType: #mouseEnter
		startPoint: rect center + (rect width @ 0)
		endPoint: rect center
		trail: #()
		buttons: 4
		hand: ActiveHand
		stamp: Time millisecondClockValue! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
deselect: aString
	| listMorph |
	listMorph _ self findListContaining: aString.
	listMorph changeModelSelection: 0.! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
listMorphs
	^ self morphsOfClass: PluggableListMorph! !

OBBrowserTest subclass: #OBMetaGraphBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMetaGraphBrowserTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 19:50'!
node
	^ OBDictionaryNode on: OBBrowser metaGraphs! !

!OBMetaGraphBrowserTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:22'!
testInitialDisplay
	self listMorphs size = 1.
	self assertAListIncludes: #(System Package Image FileSystem).
	self assertTextIs: ''.
	! !

!OBMetaGraphBrowserTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:23'!
metaGraph
	^ OBMetaGraphBuilder default metaGraphBrowser! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 12:47'!
assertListAt: index contains: items
	| list |
	list _ self listItemsAt: index.
	self assert: (list size = items size).
	self assert: (list includesAllOf: items).! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 12/17/2003 15:07'!
assertTextIs: aString
	self assert: self textMorph text = aString.! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/12/2004 20:39'!
assertNoButtonsVisible
	self assert: (self morphsOfClass: PluggableButtonMorph) isEmpty.! !

!OBBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 15:10'!
isAbstract
	^ self name = 'OBBrowserTest'! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:52'!
editText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:46'!
selectSequence: aCollection
	| plm |
	aCollection withIndexDo: [:item :index | 
					plm _ self listMorphs at: index.
					plm changeModelSelection: (plm getListObtrusively indexOf: item)]! !

OBBrowserTest subclass: #OBChasingBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBChasingBrowserTest methodsFor: 'contants' stamp: 'cwp 2/29/2004 19:24'!
fiveSource
	^ OBFakeClassA sourceCodeAt: #five! !

!OBChasingBrowserTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 20:55'!
testMessageText
	self select: 'five'.
	self assertTextIs: self fiveSource.
	self assertSelectedTextIs: ''! !

!OBChasingBrowserTest methodsFor: 'running' stamp: 'cwp 2/26/2004 22:37'!
metaGraph
	^ OBBrowser metaGraphAt: #ChaseSenders ! !

!OBChasingBrowserTest methodsFor: 'testing' stamp: 'cwp 2/26/2004 22:50'!
testSelectionClearing
	self select: 'five'.
	self select: 'OBChasingBrowserTest>>node'.
	self clickButtonLabeled: 'implementors'.
	self assertListAt: 1 includes: #('five').
	self assertListAt: 2 includes: #('OBFakeClassA>>five').
	self denyAListIncludes: #(default on: on:inClass:inContext:)! !

!OBChasingBrowserTest methodsFor: 'running' stamp: 'cwp 2/29/2004 00:05'!
node
	^ OBMethodNode
		on: #five
		inClass: #OBFakeClassA
		inContext: (OBImageContext new)! !

!OBChasingBrowserTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 19:39'!
testMessageTextThree
	self select: 'three'.
	self assertTextIs: self fiveSource.
	self assertSelectedTextIs: 'three'! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/11/2004 23:02'!
clickButtonLabeled: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	button performAction! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/30/2004 01:20'!
assertCurrentItemIs: aString
	self assert: self currentItem = aString! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/11/2004 23:03'!
assertButtonSelected: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	self assert: button isOn! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:10'!
assertAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getListDelicately includesAllOf: anArrayOfStrings]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drop: aPassenger on: targetName
	| list transfer event |
	list _ self findListContaining: targetName.
	self assert: list notNil.
	transfer _ TransferMorph withPassenger: aPassenger.
	transfer dragTransferType: #OmniBrowser.
	event _ self mouseMoveEventIn: targetName.
	self assert: (list model wantsDroppedMorph: transfer event: event inMorph: list).
	^ list model acceptDroppingMorph: transfer event: event inMorph: list! !

OBBrowserTest subclass: #OBAccessBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBAccessBrowserTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:38'!
testInitalDisplay
	self assertListAt: 1 contains: #('ivar')! !

!OBAccessBrowserTest methodsFor: 'running' stamp: 'cwp 3/10/2004 19:11'!
node
	^ OBClassNode on: #OBFakeClassB inContext: (OBPackageContext on: OBFakePackage new)! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:35'!
testSelectAccessor
	self select: 'ivar'.
	self assertListAt: 2 contains: #('OBFakeClassB>>accessesIvar')! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:39'!
testSelectSendingMethod
	self select: 'ivar'.
	self select: 'OBFakeClassB>>accessesIvar'.
	self select: 'OBFakeClassB>>sendsAccessesIvar'.
	self assertListAt: 4 contains: #('OBFakeClassB>>sendsSendsAccessesIvar')
! !

!OBAccessBrowserTest methodsFor: 'running' stamp: 'cwp 3/10/2004 19:12'!
metaGraph
	^ OBBrowser metaGraphAt: #ChaseInstanceVariables! !

!OBAccessBrowserTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 22:35'!
testSelectAccessingMethod
	self select: 'ivar'.
	self select: 'OBFakeClassB>>accessesIvar'.
	self assertListAt: 3 includes: #('OBFakeClassB>>sendsAccessesIvar')! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 1/23/2004 23:30'!
selectMenuItem: aString
	| item |
	item _ self menuItemNamed: aString.
	item ifNil: [self signalFailure: 'Menu item ''', aString, ''' does not exist.'].
	item doButtonAction! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphNamed: aString
	^ morph 
		findDeepSubmorphThat: [:m | m knownName = aString]
		ifAbsent: [self signalFailure: aString, 'not found']! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: passangerName to: targetName
	| passanger |
	passanger _ self drag: passangerName.
	self drop: passanger on: targetName! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
setText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText;
		accept.
	! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:33'!
assertTextIsSelected
	self assert: self textMorph getSelection size = self textMorph getText size! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
textMorph
	^ (self morphsOfClass: PluggableTextMorph) last! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:46'!
select: aString
	"We don't use #getListDelicately because the user would have to scroll through the list
	to find the item we're selecting, thus triggering updates to the list."
	
	| listMorph |
	listMorph _ self findListContaining: aString.
	self assert: listMorph notNil.
	listMorph model okToChange 
		ifTrue: [listMorph changeModelSelection: (listMorph getListObtrusively indexOf: aString)].! !

!OBBrowserTest methodsFor: 'running' stamp: 'cwp 3/9/2004 21:43'!
setUp
	model _ OBBrowser metaNode: self metaGraph node: self node.
	model addPanel: (OBTextPanel inBrowser: model).
	morph _ model window.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: nodeName
	| item list |
	item _ StringMorph contents: nodeName asString.
	list _ self findListContaining: nodeName.
	^ list model dragPassengerFor: item inMorph: list! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:11'!
denyAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getListDelicately includesAllOf: anArrayOfStrings]
			ifNone: [^ self].
	self assert: false.! !

OBBrowserTest subclass: #OBCodeBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:48'!
methodOne
	^ 'one
	^ 1'! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/12/2004 20:36'!
deselectFakeCategory
	self deselect: self fakeCategoryName! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 16:52'!
testJumpToNumbers
	self selectFakePackage.
	model jumpTo: (OBMethodCategoryNode 
					on: 'numbers' 
					inClass: #OBFakeClassA 
					inContext: model currentOrRootNode context).
	self assertCurrentItemIs: 'numbers'.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:37'!
testButtonRemoval
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self deselectFakeCategory.
	self assertNoButtonsVisible.! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 2/10/2004 00:27'!
fakeAClassTemplate
	^  'OBFakeClassA class
	instanceVariableNames: ''foo'''! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/7/2004 16:23'!
selectFakePackage
	! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 2/25/2004 01:31'!
clearFakeBComment
	OBFakeClassB organization classComment: ''! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:00'!
fakeClassDDefinition
 ^ 'Object subclass: #OBFakeClassD
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 2/25/2004 01:28'!
fakeClassBComment
	^ 'A comment for OBFakeClassB'! !

OBCodeBrowserTest subclass: #OBPackageBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPackageBrowserTest methodsFor: 'testing' stamp: 'cwp 1/31/2004 13:58'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 contains: #('Fakes').
	self assertTextIs: ''.
	! !

!OBPackageBrowserTest methodsFor: 'constants' stamp: 'cwp 2/16/2004 22:05'!
fakeCategoryName
	^ 'Fakes'! !

!OBPackageBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:06'!
resources
	^ Array with: OBPackageBrowserResource! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/25/2004 01:47'!
testEditComment
	self selectFakeCategory.
	self select: 'OBFakeClassB'.
	self switchToComment.
	self setText: self fakeClassBComment.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertCurrentItemIs: #OBFakeClassB.
	self assert: OBFakeClassB comment = self fakeClassBComment.! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 2/25/2004 01:14'!
fakeAComment
	^ OBFakeClassA comment asString! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 2/16/2004 22:09'!
browserResource
	^ self class resources first current! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 2/16/2004 22:16'!
setUp
	model _ self browserResource model.
	morph _ self browserResource morph. 
	model jumpToRoot.	
! !

OBCodeBrowserTest subclass: #OBImageBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBImageBrowserTest class methodsFor: 'resources' stamp: 'cwp 2/16/2004 21:59'!
resources
	^ Array with: OBImageBrowserResource ! !

!OBImageBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 13:58'!
fakeCategoryName
	^ 'OmniBrowser-Fakes'! !

!OBImageBrowserTest methodsFor: 'testing' stamp: 'cwp 1/31/2004 14:44'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 includes: #('OmniBrowser-Fakes').
	self assertTextIs: ''.
	! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:35'!
testSelectOne
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'numbers'.
	self select: 'one'.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three').
	self assertTextIs: ((MethodReference class: OBFakeClassA selector: #one) source)! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 3/1/2004 00:03'!
runCaseAsFailure: aSemaphore
	SystemChangeNotifier uniqueInstance doSilently: [super runCaseAsFailure: aSemaphore]! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 1/31/2004 14:47'!
restoreMethodOne
	OBFakeClassA compile: self methodOne! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:35'!
testSelectTests
	self selectFakeCategory.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertTextIs: self classTemplate! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/12/2004 20:37'!
selectFakeCategory
	self select: self fakeCategoryName! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:38'!
fakeClassCDefinition
 ^ 'Object subclass: #OBFakeClassC
	instanceVariableNames: ''foo''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 14:27'!
testJumpToCapitals
	self selectFakePackage.
	model jumpTo: (OBMethodCategoryNode 
					on: 'capitals' 
					inClass: #'OBFakeClassA class'
					inContext: model currentOrRootNode context).
	self assertCurrentItemIs: 'capitals'.
	self assertIsClassSide.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testEditClass
	self selectFakeCategory.
	self select: 'OBFakeClassC'.
	self setText: self fakeClassCDefinition.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertCurrentItemIs: #OBFakeClassC.
	self assert: OBFakeClassC instVarNames = #(foo)! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 3/1/2004 00:03'!
runCase
	SystemChangeNotifier uniqueInstance doSilently: [super runCase]! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 2/9/2004 20:00'!
methodA
	^ 'a
	^ $A'! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testColumnClearing
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'numbers'.
	self select: 'one'.
	self switchToClassSide.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('capitals').
	self denyAListIncludes: #('letters' 'numbers').
	self denyAListIncludes: #('one' 'two' 'three').
	self assertTextIs: self fakeAClassTemplate! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 16:52'!
testJumpToFakeA
	self selectFakePackage.
	model jumpTo: (OBClassNode 
					on: #OBFakeClassA 
					inContext: model currentOrRootNode context).
	self assertCurrentItemIs: 'OBFakeClassA'! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testEditClassMethod
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self switchToClassSide.
	self select: 'capitals'.
	self select: 'a'.
	self setText: self altMethodA.
	self assertAListMatches: #('a' 'b').
	self assertCurrentItemIs: 'a'.
	self assert: $B = OBFakeClassA a! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/16/2004 20:36'!
testSelectAllMethodCategory
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: '-- all --'.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three' 'five' 'a' 'b').
	self assertTextIs: self methodTemplate! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 13:59'!
fakeATemplate
	^ 'Object subclass: #OBFakeClassA
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/26/2004 21:40'!
testSelectFakeAComment
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self switchToComment.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self denyAListIncludes: #('letters' 'numbers').
	self assertCurrentItemIs: #OBFakeClassA.
	self assertTextIs: self fakeAComment.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 16:52'!
testJumpToFakes
	self selectFakePackage.
	model jumpTo: (OBClassCategoryNode 
					on: self fakeCategoryName 
					inContext: model currentOrRootNode context).
	self assertCurrentItemIs: self fakeCategoryName! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/26/2004 21:38'!
testSelectFakeAClassSide
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self switchToClassSide.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('capitals').
	self assertCurrentItemIs: #OBFakeClassA.
	self assertTextIs: self fakeAClassTemplate! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/25/2004 01:11'!
switchToComment
	self clickButtonLabeled: '?'! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 2/9/2004 20:04'!
restoreMethodA
	OBFakeClassA class compile: self methodA! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 2/9/2004 19:57'!
altMethodA
	^ 'a
	^ $B'! !

!OBCodeBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 14:30'!
assertIsClassSide
	self assertButtonSelected: 'class'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/30/2004 23:59'!
methodFour
	^ 'four
	^ 4'! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/7/2004 16:52'!
testJumpToMethodOne
	self selectFakePackage.
	model jumpTo: (OBMethodNode 
					on: #one 
					inClass: #OBFakeClassA
					inContext: model currentOrRootNode context).
	self assertCurrentItemIs: 'one'! !

!OBCodeBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 1/31/2004 13:08'!
isAbstract
	^ self name = #OBCodeBrowserTest! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 1/31/2004 14:41'!
restoreFakeClassC
	Compiler evaluate: 'Object subclass: #OBFakeClassC
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 18:31'!
testSelectNumbers
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'numbers'.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('letters' 'numbers').
	self assertAListIncludes: #('one' 'two' 'three').
	self assertTextIs: self methodTemplate.
	self assertTextIsSelected.! !

OBCodeBrowserTest subclass: #OBSystemBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBSystemBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:04'!
resources
	^ Array with: OBSystemBrowserResource! !

!OBSystemBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:07'!
fakeCategoryName
	^ 'Fakes'! !

!OBSystemBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/4/2004 00:22'!
selectFakePackage
	self select: 'OmniBrowser'! !

!OBSystemBrowserTest methodsFor: 'testing' stamp: 'cwp 1/31/2004 14:43'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 includes: #('OmniBrowser').
	self assertTextIs: ''.
	! !

!OBSystemBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/12/2004 20:34'!
selectFakeCategory
	self selectSequence: #('OmniBrowser' 'Fakes')! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testCreateClassFromClass
	self selectFakeCategory.
	self select: 'OBFakeClassC'.
	self setText: self fakeClassDDefinition.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC OBFakeClassD).
	self assertCurrentItemIs: #OBFakeClassD ! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testDeselectFakeA
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self deselect: 'OBFakeClassA'.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self denyAListIncludes: #('letters' 'numbers').
	self assertTextIs: self classTemplate.! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testCreateClassFromTemplate
	self selectFakeCategory.
	self setText: self fakeClassDDefinition.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC OBFakeClassD).
	self assertCurrentItemIs: #OBFakeClassD ! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testEditMethod
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'numbers'.
	self select: 'one'.
	self setText: self altMethodOne.
	self assertAListMatches: #('one' 'two' 'three' 'five').
	self assertCurrentItemIs: 'one'.
	self assert: 11 = OBFakeClassA new one! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testCreateMethodFromMethod
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'numbers'.
	self select: 'one'.
	self setText: self methodFour.
	self assertAListMatches: #('one' 'two' 'three' 'four' 'five').
	self assertCurrentItemIs: 'four' 
! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/4/2004 23:54'!
testCreateMethodFromTemplate
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self select: 'letters'.
	self setText: self methodFour.
	self assertAListMatches: #('a' 'b' 'four').
	self assertCurrentItemIs: 'four' 
! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/30/2004 23:44'!
methodTemplate
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!OBCodeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:34'!
testSelectFakeA
	self selectFakeCategory.
	self select: 'OBFakeClassA'.
	self assertAListIncludes: {self fakeCategoryName}.
	self assertAListIncludes: #(OBFakeClassA OBFakeClassB OBFakeClassC).
	self assertAListIncludes: #('letters' 'numbers').
	self assertTextIs: self fakeATemplate! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:47'!
altMethodOne
	^ 'one
	^ 11'! !

!OBCodeBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 14:01'!
classTemplate
	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''OmniBrowser-Fakes'''! !

!OBCodeBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/25/2004 01:10'!
switchToClassSide
	self clickButtonLabeled: 'class'! !

!OBCodeBrowserTest methodsFor: 'running' stamp: 'cwp 2/25/2004 01:28'!
tearDown
	super tearDown.
	(Smalltalk hasClassNamed: 'OBFakeClassD') 
		ifTrue: [Smalltalk removeClassNamed: 'OBFakeClassD'].
	OBFakeClassA removeSelector: #four.
	self restoreFakeClassC.
	self restoreMethodOne.
	self restoreMethodA.
	self clearFakeBComment! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/31/2004 13:24'!
currentList
	^ self listMorphs reversed detect: [:list | list selection notNil]! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 3/14/2004 13:11'!
listItemsAt: index
	^ (self listMorphs 
		at: index 
		ifAbsent: [self signalFailure: 'List ', index asString, ' does not exist'])
			getListDelicately
	! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/31/2004 13:23'!
currentItem
	^ self currentList selection! !

OBBrowserTest subclass: #OBDirectoryBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDirectoryBrowserTest methodsFor: 'constants' stamp: 'cwp 1/31/2004 10:13'!
epsilonNode
	^ OBFileNode 
		on: ((OBDirectoryResource current dataDir directoryNamed: 'delta') fullNameFor: 'epsilon')! !

!OBDirectoryBrowserTest methodsFor: 'testing' stamp: 'cwp 12/17/2003 15:19'!
testInitialDisplay
	self listMorphs size = 1.
	self assertListAt: 1 contains: #(alpha beta gamma delta).
	self assertTextIs: ''.
	! !

!OBDirectoryBrowserTest methodsFor: 'constants' stamp: 'cwp 12/17/2003 15:14'!
node
	^ OBDirectoryNode on: OBDirectoryResource current dataDir! !

!OBDirectoryBrowserTest methodsFor: 'testing' stamp: 'cwp 1/31/2004 10:13'!
testJumpToEpsilon
	| node |
	node _ self epsilonNode.
	model jumpTo: node.
	self assertListAt: 1 contains: #(alpha beta gamma delta).
	self assertListAt: 2 contains: #(epsilon).
	self assertCurrentItemIs: 'epsilon'.! !

!OBDirectoryBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 15:09'!
resources
	^ Array with: OBDirectoryResource! !

!OBDirectoryBrowserTest methodsFor: 'constants' stamp: 'cwp 12/18/2003 18:33'!
metaGraph
	^ OBMetaGraphBuilder default fileSystemBrowser! !

!OBDirectoryBrowserTest methodsFor: 'testing' stamp: 'cwp 12/17/2003 17:01'!
testSelectAlpha
	self select: 'alpha'.
	self assertListAt: 1 contains: #(alpha beta gamma delta).
	self assertTextIs: 'alpha'! !

!OBDirectoryBrowserTest methodsFor: 'testing' stamp: 'cwp 1/23/2004 23:33'!
testMenuItems
	self select: 'alpha'.
	self assertMenuContains: 'delete'! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:01'!
assertListAt: index includes: items
	self assert: ((self listItemsAt: index) includesAllOf: items).! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/26/2004 21:53'!
menuItemNamed: aString
	| lists current |
	lists _ self listMorphs.
	current _ lists reversed detect: [:list | list selection notNil].
	^ (current getMenu: true) itemWithWording: aString.
! !

OBBrowserTest subclass: #OBVersionsBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBVersionsBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 23:07'!
testSourceDisplay
	self select: 'cwp 3/11/2004 20:07 OBFakeClassC>>two'.
	self assertTextIs: self twoLatestVersionText! !

!OBVersionsBrowserTest methodsFor: 'constants' stamp: 'cwp 3/14/2004 23:08'!
twoLatestVersionText
	^  'two
	^ 6' asText makeSelectorBold! !

!OBVersionsBrowserTest methodsFor: 'emulating' stamp: 'cwp 3/13/2004 22:41'!
sourceFiles
	^ OBSourceFilesResource current sources! !

!OBVersionsBrowserTest methodsFor: 'testing' stamp: 'cwp 3/13/2004 22:04'!
testInitialDisplay
	self assertListAt: 1 contains: self expectedHeadings! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: 'cwp 3/13/2004 22:01'!
node
	^ OBMethodNode on: #two inClass: #OBFakeClassC inContext: self! !

!OBVersionsBrowserTest methodsFor: 'running' stamp: 'cwp 3/13/2004 22:02'!
metaGraph
	^ OBBrowser metaGraphAt: #Versions! !

!OBVersionsBrowserTest methodsFor: 'constants' stamp: 'cwp 3/14/2004 23:05'!
expectedHeadings
	^ #('cwp 3/11/2004 20:07 OBFakeClassC>>two' 'cwp 3/11/2004 20:06 OBFakeClassC>>two' 'cwp 3/11/2004 20:06 OBFakeClassC>>two' 'cwp 3/11/2004 20:07 OBFakeClassC>>two' 'cwp 3/11/2004 20:06 OBFakeClassC>>two' 'cwp 3/11/2004 20:06 OBFakeClassC>>two')! !

!OBVersionsBrowserTest methodsFor: 'emulating' stamp: 'cwp 3/14/2004 22:52'!
sourcePointerFor: aMethodReference
	| pointer |
	pointer _ self sourceFiles sourcePointerFromFileIndex: 2 andPosition: 624.
	^ pointer.! !

TestCase subclass: #OBDirectoryNodeTest
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDirectoryNodeTest methodsFor: 'testing' stamp: 'cwp 12/15/2003 21:39'!
testName
	self assert: node name = 'obtest'! !

!OBDirectoryNodeTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/8/2003 22:32'!
resources
	^ Array with: OBDirectoryResource! !

!OBDirectoryNodeTest methodsFor: 'testing' stamp: 'cwp 12/17/2003 15:01'!
testDirectoryNodes
	| children |
	children _ node directories collect: [:n | n name].
	self assert: children = #(delta)
	! !

!OBDirectoryNodeTest methodsFor: 'running' stamp: 'cwp 12/15/2003 21:39'!
setUp
	node _ OBDirectoryNode on: self dataDir.! !

!OBDirectoryNodeTest methodsFor: 'data' stamp: 'cwp 11/28/2003 22:47'!
dataDir
	^ OBDirectoryResource current dataDir! !

!OBDirectoryNodeTest methodsFor: 'testing' stamp: 'cwp 12/15/2003 21:39'!
testText
	self assert: (node text endsWith: '
----------
alpha - 5 bytes
beta - 4 bytes
delta - 0 bytes
gamma - 5 bytes
')
! !

!OBDirectoryNodeTest methodsFor: 'testing' stamp: 'cwp 12/17/2003 14:49'!
testFileNodes
	| children |
	children _ node files.
	self assert: (children collect: [:n | n name]) = #(alpha beta gamma)
	! !

TestCase subclass: #OBFileRegistryActorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFileRegistryActorTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/24/2004 00:26'!
assertActions: actions include: required
	| labels |
	labels _ actions collect: [:act | act label].
	self assert: (required includesAllOf: labels)! !

!OBFileRegistryActorTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/24/2004 00:26'!
testTextFile
	| actor node actions |
	actor _ OBFileRegistryActor new.
	node _ OBFileNode on: (FileDirectory default fullNameFor: 'foo.txt').
	actions _ actor actionsForNode: node.
	self assertActions: actions include: #('add file to new zip' 'compress file').! !

TestCase subclass: #OBDictionaryNodeTest
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDictionaryNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:15'!
setUp
	| dict |
	dict _ Dictionary new.
	dict
		at: #one put: 1;
		at: #two put: 2;
		at: #three put: 3.
	node _ OBDictionaryNode on: dict! !

!OBDictionaryNodeTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 20:16'!
testNames
	| list |
	list _ node associations collect: [:n | n name].
	self assert: (list includesAllOf: #(one two three))! !

TestCase subclass: #OBButtonPanelTest
	instanceVariableNames: 'model morph pushed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 3/11/2004 23:05'!
testButtons
	model nodeChanged.
	self assert: morph submorphs size = 3.
	#(foo bar baz) 
		with: morph submorphs 
		do: [:sel :m | self assert: m label contents = sel asString]! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 3/10/2004 00:10'!
testLayout
	| rects |
	morph bounds: (10@10 corner: 310@30).
	model nodeChanged.
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	morph openInWorld.
	rects _ morph submorphs collect: [:b | b bounds].
	morph delete.

	rects do: [:ea | self assert: ea top = 10; assert: ea bottom = 30].
	self assert: rects first left = 10.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = 310.! !

!OBButtonPanelTest methodsFor: 'running' stamp: 'cwp 3/10/2004 23:47'!
setUp
	model _ OBButtonPanel inBrowser: self.
	morph _ model morph.
! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:59'!
actionsForNode: aNode
	^ #(foo bar baz) collect: [:ea | OBAction
								label: ea asString
								buttonLabel: ea asString
								receiver: self
								selector: #push:
								arguments: (Array with: ea)]
		! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:08'!
currentNode
	^ (OBFakeNode new) metaNode: self metaNode! !

!OBButtonPanelTest methodsFor: 'callbacks' stamp: 'cwp 3/10/2004 23:59'!
push: aSymbol
	pushed _ aSymbol! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 3/10/2004 00:01'!
testPushFoo
	model nodeChanged.
	morph submorphs first performAction.
	self assert: pushed = #foo
	! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:09'!
metaNode
	^ OBMetaNode new addActor: self! !

TestCase subclass: #OBProtocolMockTest
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMockTest methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:17'!
shouldFail: aBlock
	self should: aBlock raise: TestResult failure
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSelectorFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub bar]]
	! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 16:05'!
testTooManyExceptionsFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should:  [mock examine: [:stub | stub truth: Notification signal. Notification signal]]
		raise: Exception! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:20'!
testResume
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true];
		examine: [:stub | stub truth: Notification signal]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:44'!
testNoExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self shouldFail: [mock examine: [:stub | stub truth: true]] ! !

!OBProtocolMockTest methodsFor: 'running' stamp: 'cwp 2/13/2004 01:15'!
setUp
	mock _ OBProtocolMock new.! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testExtraFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub foo; bar]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 17:17'!
testAnswers
	| one two three |
	mock 
		expect: [:stub | stub int; int; int];
		answer: #(1 2 3);
		examine: [:stub | 	one _ stub int.
						two _ stub int.
						three _ stub int].
		self assert: one = 1.
		self assert: two = 2.
		self assert: three = 3! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testMissingFailure
	mock expect: [:stub | stub foo; bar].
	self shouldFail: [mock examine: [:stub | stub foo]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSuccess
	mock expect: [:stub | stub foo; foo: #bar].
	mock examine: [:stub | stub foo; foo: #bar]
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testArgFailure
	mock expect: [:stub | stub foo: #bar].
	self shouldFail: [mock examine: [:stub | stub foo: #baz]]
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:09'!
testWrongExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should: [mock examine: [:stub | stub truth: Error signal]] 
		raise: Error! !

TestResource subclass: #OBBrowserResource
	instanceVariableNames: 'model morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBBrowserResource methodsFor: 'running' stamp: 'cwp 3/9/2004 21:42'!
setUp
	model _ OBBrowser metaNode: self metaGraph node: self node.
	model addPanel: (OBTextPanel inBrowser: model).
	morph _ model window.! !

OBBrowserResource subclass: #OBImageBrowserResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBImageBrowserResource methodsFor: 'running' stamp: 'cwp 2/16/2004 22:01'!
metaGraph
	^ OBMetaGraphBuilder default imageBrowser! !

!OBImageBrowserResource methodsFor: 'running' stamp: 'cwp 2/16/2004 22:01'!
node
	^ OBSystemNode on: OBImageContext new! !

!OBBrowserResource methodsFor: 'accessing' stamp: 'cwp 2/16/2004 21:19'!
model
	^ model! !

OBBrowserResource subclass: #OBSystemBrowserResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBSystemBrowserResource methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:03'!
node
	^ OBSystemNode on: OBImageContext new! !

!OBSystemBrowserResource methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:03'!
metaGraph
	^ OBMetaGraphBuilder default systemBrowser! !

!OBBrowserResource methodsFor: 'accessing' stamp: 'cwp 2/16/2004 21:19'!
morph
	^ morph! !

OBBrowserResource subclass: #OBPackageBrowserResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPackageBrowserResource methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:06'!
metaGraph
	^ OBMetaGraphBuilder default packageBrowser ! !

!OBPackageBrowserResource methodsFor: 'as yet unclassified' stamp: 'cwp 2/16/2004 22:06'!
node
	^ OBPackageNode on: OBFakePackage default! !

TestResource subclass: #OBSourceFilesResource
	instanceVariableNames: 'sources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBSourceFilesResource methodsFor: 'initializing' stamp: 'cwp 3/14/2004 22:57'!
changesString
	^ '!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:05'' prior: 16777702!!
one
	^ 4!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777808!!
two
	^ 4!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 33554526!!
one
	^ 5!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 33554632!!
two
	^ 5!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 33554738!!
one
	^ 6!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 33554844!!
two
	^ 6!! !!'! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: 'cwp 3/13/2004 22:39'!
sourceString
	^ '!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:05''!!
one
	^ 1!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06''!!
two
	^ 1!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777294!!
one
	^ 2!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:06'' prior: 16777384!!
two
	^ 2!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 16777490!!
one
	^ 3!! !!
!!OBFakeClassC methodsFor: ''as yet unclassified'' stamp: ''cwp 3/11/2004 20:07'' prior: 16777596!!
two
	^ 3!! !!'! !

!OBSourceFilesResource methodsFor: 'accessing' stamp: 'cwp 3/13/2004 22:40'!
sources
	^ sources! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: 'cwp 3/13/2004 22:38'!
changesStream
	^ ReadStream on: self changesString! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: 'cwp 3/13/2004 22:38'!
setUp
	sources _ StandardSourceFileArray new: 2.
	sources at: 1 put: self sourceStream.
	sources at: 2 put: self changesStream.! !

!OBSourceFilesResource methodsFor: 'initializing' stamp: 'cwp 3/13/2004 22:38'!
sourceStream
	^ ReadStream on: self sourceString! !

TestResource subclass: #OBDirectoryResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 1/23/2004 23:15'!
tearDown
	self dataDir recursiveDelete! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:44'!
dataDirName
	^ 'obtest'! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:47'!
dataDir
	^ FileDirectory default directoryNamed: self dataDirName! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 1/23/2004 23:15'!
setUp
	| dir |
	dir _ self dataDir.
	dir assureExistence.
	#(alpha beta gamma) do: [:filename | | file | file _ dir forceNewFileNamed: filename.
							  file nextPutAll: filename.
							  file close].
	dir createDirectory: 'delta'.
	(dir directoryNamed: 'delta') createDirectory: 'epsilon'.! !

ProtoObject subclass: #OBMessageRecorder
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:52'!
asExplorerString
	^ 'an OBMessageRecorder'! !

!OBMessageRecorder methodsFor: 'initializing' stamp: 'cwp 2/13/2004 00:24'!
obmrSetMock: aMockObject
	mock _ aMockObject! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:53'!
hasContentsInExplorer
	^ false! !

!OBMessageRecorder methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:23'!
doesNotUnderstand: aMessage
	^ mock message: aMessage! !

!OBMessageRecorder class methodsFor: 'as yet unclassified' stamp: 'cwp 2/13/2004 00:25'!
forMock: aMock
	^ self new obmrSetMock: aMock! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/13/2004 01:14'!
inspectorClass
	^ Inspector! !

MessageSend subclass: #OBAction
	instanceVariableNames: 'label buttonLabel monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBAction commentStamp: 'cwp 3/4/2004 20:52' prior: 0!
Actions represent commands for manipulating the graph domain. They can be made available through menus or buttons in the browser. They carry information on how they should be presented to the user, and are responsible for handling exceptions generated when they are triggered. Actions are created by Actors.

iVars: 

label 		a string which describes the action for the user.
monitor  	when the action is triggered, any notifications raised 
			will be passed to this object for processing!

!OBAction methodsFor: 'evaluating' stamp: 'cwp 1/7/2004 18:22'!
perform: aSelector orSendTo: anObject
	^ self perform: aSelector! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:59'!
monitor: anObject
	monitor _ anObject! !

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 3/10/2004 23:36'!
label: aString buttonLabel: aString2 receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		buttonLabel: aString2! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:40'!
wantsButton
	^ buttonLabel notNil! !

!OBAction methodsFor: 'evaluating' stamp: 'cwp 3/14/2004 13:35'!
trigger
	[self value]
		on: OBNotification
		do: [:aNotification | 
			monitor notification: aNotification. 
			aNotification resume].! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:59'!
buttonLabelMorph
	^ StringMorph 
		contents: self buttonLabel 
		font: Preferences standardButtonFont! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 1/7/2004 09:52'!
addItemToMenu: aMenu
	aMenu
		add: label
		target: self 
		selector: #trigger! !

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 1/7/2004 09:20'!
label: aString receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) label: aString! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:12'!
label: aString
	label _ aString! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel: aString
	buttonLabel _ aString
	! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:58'!
button
	^ (PluggableButtonMorph
		on: self
		getState: nil
		action: #trigger
		label: #buttonLabelMorph)	
			onColor: Color lightGray lighter offColor: Color lightGray twiceLighter;
			borderWidth: 2;
			borderRaised;
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			setBalloonText: label! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel
	^ buttonLabel! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:11'!
label
	^ label! !

Notification subclass: #OBChoiceRequest
	instanceVariableNames: 'prompt labels values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBChoiceRequest commentStamp: 'cwp 3/5/2004 12:26' prior: 0!
This notification is used to ask the user for to choose from a list of alternatives. Its defaultAction is to open a PopUpMenu. Test cases an intercept the notification and respond programmatically.

prompt	- A string describing the choice the user is asked to make.
labels	- A list of strings describing the alternatives.
values	- When the user chooses an alternative, the corresponding item from this list is returned!

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:05'!
labels: anArray
	^ (self new setPrompt: nil labels: anArray values: anArray) signal! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 3/4/2004 02:04'!
defaultAction
	^ values at: ((PopUpMenu labelArray: labels) startUpWithCaption: prompt)
	! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:27'!
prompt: aString labels: labelArray values: valueArray
	^ (self new setPrompt: aString labels: labelArray values: valueArray) signal! !

!OBChoiceRequest methodsFor: 'initializing' stamp: 'cwp 3/4/2004 02:03'!
setPrompt: aString labels: labelArray values: valueArray
	prompt _ aString.
	labels _ labelArray.
	values _ valueArray! !

Notification subclass: #OBNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNotification commentStamp: 'cwp 3/5/2004 12:33' prior: 0!
An abstract superclass for UI-related notifications. This is necessary to allow subclasses to be trapped by OBAction, while allowing other notifications to pass.!

OBNotification subclass: #OBNodeDeleted
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNodeDeleted commentStamp: 'cwp 3/5/2004 12:31' prior: 0!
This notification is signaled when an action deletes the currently selected node.!

!OBNodeDeleted methodsFor: 'as yet unclassified' stamp: 'cwp 1/30/2004 00:33'!
notify: aMonitor
	aMonitor nodeDeleted! !

OBNotification subclass: #OBNodeSelected
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNodeSelected commentStamp: 'cwp 3/5/2004 12:32' prior: 0!
This notification is signaled to make the indicated node the current selection.

iVars:

node	- the node which should be selected!

!OBNodeSelected methodsFor: 'as yet unclassified' stamp: 'cwp 1/29/2004 23:37'!
node
	^ node! !

!OBNodeSelected methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 17:01'!
notify: aMonitor
	aMonitor nodeCreated: node! !

!OBNodeSelected methodsFor: 'as yet unclassified' stamp: 'cwp 1/29/2004 23:37'!
node: aNode
	node _ aNode! !

!OBNodeSelected class methodsFor: 'as yet unclassified' stamp: 'cwp 1/29/2004 23:37'!
for: aNode
	^ self new node: aNode! !

OBNotification subclass: #OBListChanged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBListChanged commentStamp: 'cwp 3/5/2004 12:31' prior: 0!
This notification is raised when some action changes the list in the current column of the browser, but the selected node should not change.!

!OBListChanged methodsFor: 'as yet unclassified' stamp: 'cwp 1/30/2004 00:33'!
notify: aMonitor
	aMonitor listChanged! !

Notification subclass: #OBConfirmationRequest
	instanceVariableNames: 'prompt confirm cancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBConfirmationRequest commentStamp: 'cwp 3/5/2004 12:30' prior: 0!
This notification is used to ask the user to confirm some kind potentially dangerous operation. Its default action is to open a PopUpMenu.

iVars:

prompt		- a string describing the situation the user is asked to confirm
confirm		- a string describing the action that will be taken if the user confirms
cancel		- a string describing the action that will be taken if the user does not confirm!

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:09'!
newPrompt: prompt confirm: confirm cancel: cancel
	^ self new setPrompt: prompt confirm: confirm cancel: cancel! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:02'!
prompt: prompt confirm: confirm cancel: cancel
	^ (self newPrompt: prompt confirm: confirm cancel: cancel) signal! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
cancel
	self resume: false! !

!OBConfirmationRequest methodsFor: 'user interaction' stamp: 'cwp 2/28/2004 12:08'!
defaultAction
	^ PopUpMenu confirm: prompt trueChoice: confirm falseChoice: cancel! !

!OBConfirmationRequest methodsFor: 'initalizing' stamp: 'cwp 2/28/2004 12:00'!
setPrompt: promptString confirm: confirmString cancel: cancelString
	prompt _ promptString.
	confirm _ confirmString.
	cancel _ cancelString! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:12'!
prompt: prompt confirm: confirm
	^ self prompt: prompt confirm: confirm cancel: 'Cancel'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:13'!
prompt: prompt
	^ self prompt: prompt confirm: 'Ok'! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
ok
	self resume: true! !

Notification subclass: #OBTextRequest
	instanceVariableNames: 'prompt template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBTextRequest commentStamp: 'cwp 3/5/2004 12:35' prior: 0!
This notification is used to ask the user to supply a short piece of text. Its defaultAction is to open a FillInTheBlank.

iVars:

prompt		- a string describing the text the user is asked to supply
template	- a default reply !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:23'!
prompt: aString
	prompt _ aString! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
requestText
	^ FillInTheBlankMorph 
		request: prompt
		initialAnswer: template
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
defaultAction
	self resume: self requestText! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 20:59'!
template: aString
	template _ aString! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:24'!
prompt
	^ prompt! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:31'!
prompt: promptString template: templateString
	^ (self new prompt: promptString; template: templateString) signal! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:00'!
template
	^ template! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/2/2004 21:30'!
prompt: aString
	^ self new prompt: aString; template: ''! !

PackageInfo subclass: #OBFakePackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakePackage methodsFor: 'as yet unclassified' stamp: 'cwp 1/31/2004 13:56'!
classes
	^ (SystemOrganization listAtCategoryNamed: self systemCategoryPrefix, '-Fakes')
		collect: [:name | Smalltalk at: name].! !

!OBFakePackage methodsFor: 'as yet unclassified' stamp: 'cwp 2/29/2004 00:02'!
includesSystemCategory: catName
	^ catName = 'OmniBrowser-Fakes'! !

!OBFakePackage methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 13:42'!
systemCategories
	^ (SystemOrganization categories select: [:cat | cat includesSubString: 'OBContext']) 
		copyWith: #'OmniBrowser-Fakes'! !

!OBFakePackage methodsFor: 'as yet unclassified' stamp: 'cwp 1/31/2004 15:19'!
packageName
	^ 'OmniBrowserFakes'! !

!OBFakePackage methodsFor: 'as yet unclassified' stamp: 'cwp 12/10/2003 23:31'!
systemCategoryPrefix
	^ 'OmniBrowser'! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:10'!
longNameOf: shortName
	^ shortName isEmpty
		ifTrue: [self systemCategoryPrefix]
		ifFalse: [self systemCategoryPrefix, '-', shortName]
! !

!PackageInfo class methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:17'!
hidden: aString
	self allSubclassesDo: [:c | c default packageName = aString ifTrue: [^ c default]].
	^ self new packageName: aString! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:08'!
shortNameOf: aSystemCategoryName
	^ (aSystemCategoryName = self systemCategoryPrefix)
		ifTrue: ['']
		ifFalse: [aSystemCategoryName allButFirst: self systemCategoryPrefix size + 1]
! !

!PluggableButtonMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/24/2004 19:07'!
isOn
	^ self color = onColor! !

Slider subclass: #OBScrollBar
	instanceVariableNames: 'upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay'
	classVariableNames: 'CachedImages UpArrow8Bit UpArrow'
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBScrollBar commentStamp: 'cwp 3/5/2004 12:01' prior: 0!
This class is a relic. It was originally created to work around bugs in ScrollBar which prevented it from working correctly in horizontal orientation. At some point the bugs should be fixed, and ScrollBar should be used instead.!

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
extent: p
        p x > p y
                ifTrue: [super extent: (p max: 42@8)]
                ifFalse: [super extent: (p max: 8@42)]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
totalSliderArea
	| upperBoundsButton |
	upperBoundsButton _ upButton.
	upButton bottom > upperBoundsButton bottom
		ifTrue: [upperBoundsButton _ upButton].
	^ bounds isWide
		ifTrue: [upperBoundsButton bounds topRight corner: downButton bounds bottomLeft]
		ifFalse: [upperBoundsButton bounds bottomLeft corner: downButton bounds topRight].
! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
alternativeScrollbarLook
	^Preferences alternativeScrollbarLook! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown
	self flag: #obsolete.
	downButton eventHandler: nil.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollDownInit! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
resetTimer
	timeOfMouseDown _ Time millisecondClockValue.
	timeOfLastScroll _ timeOfMouseDown - 1000 max: 0.
	nextPageDirection _ nil.
	currentScrollDelay _ nil! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta
	^ scrollDelta! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUpInit
	upButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollUp.
	self startStepping.! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
sliderThickness
	^ self roundedScrollbarLook ifTrue:[15] ifFalse:[super sliderThickness]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializePagingArea
	pagingArea := RectangleMorph newBounds: self totalSliderArea
								color: (Color r: 0.6 g: 0.6 b: 0.8).
	pagingArea borderWidth: 0.
	pagingArea on: #mouseDown send: #scrollPageInit: to: self.
	pagingArea on: #mouseUp send: #finishedScrolling to: self.
	self addMorph: pagingArea.
	self roundedScrollbarLook 
		ifTrue:[pagingArea color: (Color gray: 0.9)].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
stepTime
	^ currentScrollDelay ifNil: [300]! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction
	^self valueOfProperty: #scrollBarAction! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeDownButton
	downButton := RectangleMorph
		newBounds: (self innerBounds bottomRight - self buttonExtent extent: self buttonExtent)
		color: self thumbColor.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	downButton addMorphCentered: (ImageMorph new image: 
		(self 
			cachedImageAt: (bounds isWide ifTrue: ['right'] ifFalse: ['down']) 
			ifAbsentPut: [
				self upArrow8Bit
					rotateBy: (bounds isWide ifTrue: [#right] ifFalse: [#pi])
					centerAt: 0@0
			]
		)
	).
	self roundedScrollbarLook ifTrue:[
		downButton color: Color veryLightGray.
		downButton borderStyle: (BorderStyle complexRaised width: 3).
	] ifFalse:[
		downButton setBorderWidth: 1 borderColor: #raised.
	].
	self addMorph: downButton.
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
setNextDirectionFromEvent: event

	nextPageDirection _ bounds isWide ifTrue: [
		event cursorPoint x >= slider center x
	]
	ifFalse: [
		event cursorPoint y >= slider center y
	]

! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDownInit
	downButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollDown.
	self startStepping.! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeEmbedded: aBool
	"aBool == true => inboard scrollbar
	aBool == false => flop-out scrollbar"
	self roundedScrollbarLook ifFalse:[^self].
	aBool ifTrue:[
		self borderStyle: (BorderStyle inset width: 2).
		self cornerStyle: #square.
	] ifFalse:[
		self borderStyle: (BorderStyle width: 1 color: Color black).
		self cornerStyle: #rounded.
	].
	self removeAllMorphs.
	self initializeSlider.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderShadowColor
	^ self roundedScrollbarLook
		ifTrue: [self sliderColor darker]
		ifFalse: [super sliderShadowColor]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollPageInit: evt
	self resetTimer.
	self setNextDirectionFromEvent: evt.
	self scrollBarAction: #doScrollByPage.
	self startStepping.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
interval: d
	"Supply an optional floating fraction so slider can expand to indicate range"
	interval _ d min: 1.0.
	self expandSlider.
	self computeSlider.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp: count
	self setValue: (value - (scrollDelta * count) - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
sliderExtent
	"The sliderExtent is now stored in the slider itself, not hardcoded as it is in the superclass."
	^slider extent! !

!OBScrollBar methodsFor: 'initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize
	super initialize.
	scrollDelta _ 0.02.
	pageDelta _ 0.2.
	self roundedScrollbarLook ifTrue:[
		self borderStyle: ((BorderStyle complexFramed width: 2) "baseColor: Color gray")].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
wantsSteps
	^self scrollBarAction notNil! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp
	self flag: #obsolete.
	upButton eventHandler: nil.
	upButton on: #mouseDown send: #scrollUpInit to: self.
	upButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollUpInit! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollDown
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollByPage
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
finishedScrolling
	self stopStepping.
	self scrollBarAction: nil.
	self roundedScrollbarLook ifTrue:[
		upButton borderStyle: (BorderStyle complexRaised width: upButton borderWidth).
		downButton borderStyle: (BorderStyle complexRaised width: downButton borderWidth).
	] ifFalse:[
		downButton borderRaised.
		upButton borderRaised.
	].

! !

!OBScrollBar class methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:07'!
alwaysShowFlatScrollbarForAlternativeLook
	"Set this value to true, if you want to see the flat scrollbar look in flop-out mode as well as inboard.  Otherwise the flop-out scrollbar will be rounded and inboard will be flat."
	^ false! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
mouseDownInSlider: event
	interval = 1.0 ifTrue:
		["make the entire scrollable area visible if a full scrollbar is clicked on"
		self setValue: 0.
		self model hideOrShowScrollBar].
	super mouseDownInSlider: event! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeSlider
	self roundedScrollbarLook ifTrue:[
		self 
			initializeUpButton;
			initializeDownButton;
			initializePagingArea.
	] ifFalse:[
		self initializeUpButton; 
			initializeDownButton; 
			initializePagingArea.
	].
	super initializeSlider.
	self roundedScrollbarLook ifTrue:[
		slider cornerStyle: #rounded.
		slider borderStyle: (BorderStyle complexRaised width: 3).
		sliderShadow cornerStyle: #rounded.
	].
	self sliderColor: self sliderColor.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta: d1 pageDelta: d2
	"Supply optional increments for better scrolling of, eg, text"
	scrollDelta _ d1.
	pageDelta _ d2.! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
adoptPaneColor: aColor
	"Adopt the given pane color"
	self alternativeScrollbarLook ifFalse:[^self].
	aColor ifNil:[^self].
	self sliderColor: aColor.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown: count
	self setValue: (value + (scrollDelta * count) + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction: aSymbol
	self setProperty: #scrollBarAction toValue: aSymbol! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeUpButton
	upButton := self roundedScrollbarLook 
				ifTrue:  
					[RectangleMorph 
						newBounds: (self innerBounds topLeft extent: self buttonExtent)]
				ifFalse: 
					[RectangleMorph 
						newBounds: ((self innerBounds topLeft) extent: self buttonExtent)].
	upButton color: self thumbColor.
	upButton 
		on: #mouseDown
		send: #scrollUpInit
		to: self.
	upButton 
		on: #mouseUp
		send: #finishedScrolling
		to: self.
	upButton 
		addMorphCentered: (ImageMorph new image: (self 
						cachedImageAt: (bounds isWide ifTrue: ['left'] ifFalse: ['up'])
						ifAbsentPut: 
							[bounds isWide 
								ifTrue: [self upArrow8Bit rotateBy: #left centerAt: 0 @ 0]
								ifFalse: [self upArrow8Bit]])).
	self roundedScrollbarLook 
		ifTrue: 
			[upButton color: Color veryLightGray.
			upButton borderStyle: (BorderStyle complexRaised width: 3)]
		ifFalse: [upButton setBorderWidth: 1 borderColor: #raised].
	self addMorph: upButton! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
expandSlider
	"Compute the new size of the slider (use the old sliderThickness as a minimum)."
	| r |
	r _ self totalSliderArea.
	slider extent: (bounds isWide
		ifTrue: [((r width * interval) asInteger max: self sliderThickness) @ slider height]
		ifFalse: [slider width @ ((r height * interval) asInteger max: self sliderThickness)])! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollUp
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
thumbColor
	"Problem: Part of the ScrollBar/Slider code uses 'slider' to mean the entire scrollbar/slider widget, and part of it uses 'slider' to mean only the draggable 'thumb'.  This should be cleaned up so that 'thumb' is used instead of 'slider' where appropriate.  For now, the meaning of thumbColor is clear, at least."

	^ self alternativeScrollbarLook 
		ifTrue: [self sliderColor alphaMixed: 0.7 with: (Color gray: 0.95)]
		ifFalse: [Color veryLightGray]! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
pagingArea
	^pagingArea! !

!OBScrollBar methodsFor: 'stepping and presenter' stamp: 'cwp 11/23/2003 18:07'!
step
	| action |
	action _ self scrollBarAction.
	action ifNotNil:[self perform: action].! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
waitForDelay1: delay1 delay2: delay2 
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."

	| now scrollDelay |
	timeOfLastScroll isNil ifTrue: [self resetTimer].	"Only needed for old instances"
	now := Time millisecondClockValue.
	(scrollDelay := currentScrollDelay) isNil 
		ifTrue: [scrollDelay := delay1	"initial delay"].
	currentScrollDelay := scrollDelay * 9 // 10 max: delay2.	"decrease the delay"
	timeOfLastScroll := now.
	^true! !

!OBScrollBar methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:06'!
cachedImageAt: aKey ifAbsentPut: aBlock

	CachedImages ifNil: [CachedImages _ Dictionary new].
	^CachedImages at: aKey ifAbsentPut: aBlock! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
roundedScrollbarLook
	"Rounded look currently only shows up in flop-out mode"
	^Preferences alternativeScrollbarLook and: [Preferences inboardScrollbars not and: [self class alwaysShowFlatScrollbarForAlternativeLook not]]
! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
menuButtonMouseDown: event
	event hand showTemporaryCursor: nil.
	self use: menuSelector orMakeModelSelectorFor: 'MenuButtonPressed:'
		in: [:sel | menuSelector _ sel.  model perform: sel with: event]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
buttonExtent
	^ bounds isWide
		ifTrue: [11 @ self innerBounds height]
		ifFalse: [self innerBounds width @ 11]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
upArrow8Bit

	"convert to 8-bit and convert white to transparent to avoid gratuitous conversion every time we put one in an ImageMorph"

	^UpArrow8Bit ifNil: [
		UpArrow8Bit _ (ColorForm mappingWhiteToTransparentFrom: UpArrow) asFormOfDepth: 8
	]! !

!OBScrollBar class methodsFor: 'class initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize "ScrollBar initialize"

	UpArrow _ Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderColor: aColor
	"Change the color of the scrollbar to go with aColor."
	| buttonColor |
	super sliderColor: aColor.
	buttonColor _ self thumbColor.
	upButton color: buttonColor.
	downButton color: buttonColor.
	slider color: buttonColor.
	self alternativeScrollbarLook ifTrue:
		[self roundedScrollbarLook
			ifTrue:
				[self color: Color transparent.
				pagingArea color: aColor muchLighter.
				self borderStyle style == #simple 
					ifTrue:[self borderColor: aColor darker darker]
					ifFalse:[self borderStyle baseColor: aColor]]
			ifFalse:
				[pagingArea color: (aColor alphaMixed: 0.3 with: Color white).
				self borderWidth: 0]]
! !

!OBScrollBar methodsFor: 'model access' stamp: 'cwp 11/23/2003 18:06'!
setValue: newValue
	"Using roundTo: instead of truncateTo: ensures that scrollUp will scroll the same distance as scrollDown."
	^ super setValue: (newValue roundTo: scrollDelta)! !
OBBrowser initialize!
OBNotifier initialize!
OBImageContextTest initialize!
OBScrollBar initialize!
